var BRIDGE = (function(){

var exports = {};

exports.name = "bridge";

var mIsAndroid = (navigator.userAgent.indexOf("Android") > 0) ||
                 (navigator.userAgent.indexOf("android") > 0);

var mIsIPhone  = (navigator.userAgent.indexOf("iPhone")  > 0) ||
                 (navigator.userAgent.indexOf("iPad")    > 0);      

var mIsNative  = mIsAndroid || mIsIPhone;

var mCmdResponseName   = null;
var mCmdResponseStatus = null;
var mCmdResponseData   = null;

function itrace(aText)
{
    if (console.log)
    {
        console.log(aText);
        return;
    }
}

function doCmd()
{
//--    itrace(">> doCmd, Enter");

    mCmdResponseName   = null;
    mCmdResponseStatus = null;
    mCmdResponseData   = null;

    var argsArray = [];

    for (var index = 0; index < arguments.length; index++)
    {
        var thisArg = arguments[index];
        argsArray.push(thisArg);
    }

    var cmdJson   = JSON.stringify(argsArray);

    var cmdBase64 = btoa(cmdJson);

    var cmdUrl    = "alive:" + cmdBase64;

//--    itrace("doCmd, Cmd: " +cmdUrl);

    if (mIsAndroid)
    {
//--        itrace("doCmd, Android");

        var result = new String(window.NATIVE.doCommand(cmdUrl));

        //trace("doCmd, Result: " +result);

        var resultJson = atob(result);

//--        itrace("doCmd, Result: " +resultJson);

        var fields = [];

        try
        {
            fields = JSON.parse(resultJson);
        }
        catch(err)
        {
            error("doCmd, Could not parse command response: " +err);
        }

        mCmdResponseStatus = fields[0];
        mCmdResponseName   = fields[1];
        mCmdResponseData   = fields[2];

//--        itrace("doCmd, Fields: " +fields.length);
//--        itrace("doCmd, Status: " +mCmdResponseStatus);
//--        itrace("doCmd, Cmd:    " +mCmdResponseName);
//--        itrace("doCmd, Data:   " +mCmdResponseData);
        
//--        itrace("<< doCmd");
        return;
    }

    if (mIsIPhone)
    {
//--        itrace("doCmd, iPhone");

        var iframe = document.createElement("IFRAME");
        iframe.setAttribute("src", cmdUrl);
        document.documentElement.appendChild(iframe);
        iframe.parentNode.removeChild(iframe);
        iframe = null;

//--        itrace("<< doCmd");
        return; 
    }
 
    if (! mIsNative)
    {
//--        itrace("<< doCmd, We are not native - Probably on a web browser");
        return;
    }

//--    itrace("<< doCmd, *** Unknown Platform ***");
}


function onCmdResponse(aStatus, aCmd, aData)
{
    //var fields = aData.split(';');
    
    mCmdResponseStatus = aStatus;
    mCmdResponseName   = aCmd;
    
    mCmdResponseData = atob(aData); //(aData == null) ? null : decodeURIComponent(aData);

    //itrace("onCmdResponse: Cmd: " +aCmd+ ", Status: " +aStatus+ ", Data: " +aData+ ", Decoded: " +mCmdResponseData);
}

function logit(aLevel, aModule, aMessage)
{
    if (mIsAndroid)
    {
        window.NATIVE.logit(aLevel, aModule, aMessage);        
    }
    else
    {
        doCmd("log", aLevel, aModule, aMessage);
    }
}

function getPageQuery()
{
//--    itrace("getPageQuery(I)");
    
    doCmd("getPageQuery");
    
    if (mCmdResponseStatus != "1")
    {
//--        itrace("getPageQuery, Error Response");
        return null;
    }
    
    if (mCmdResponseData == "") 
    {
//--        itrace("getPageQuery, No Query Returned");
        return null;
    }

//--    itrace("getPageQuery, Query: " +mCmdResponseData);

    return mCmdResponseData;
}

function getPageUrl()
{
    doCmd("getPageUrl");
    
    if (mCmdResponseStatus != "1")
    {
        return null;
    }
    
    if (mCmdResponseData == "") 
    {
        return null;
    }

    return mCmdResponseData;
}

function showSpinner(aActive, aPos, aHasBackground)
{
    var active = aActive ? "1" : "0";

    var pos = "";

    if (aPos != null)
    {
        pos = "" + aPos;
    }

    var background = "0";

    if (aHasBackground != null)
    {
        background =  aHasBackground ? "1" : "0";
    }

    doCmd("showSpinner", active, pos, background);
    return mCmdResponseData;
}

function showOkDialog(aTitle, aMessage)
{
    var title   = aTitle   == null ? "" : aTitle;
    var message = aMessage == null ? "" : aMessage;

    doCmd("showOkDialog", title, message);
    return mCmdResponseData;
}

function setAppState(aStatus)
{
    doCmd("setAppState", aStatus);
    return mCmdResponseData;    
}

function getRequestHostUrl()
{
    doCmd("getRequestHostUrl");    
    return mCmdResponseData;
}

function setValue(aKey, aValue)
{
    //itrace("setValue, Key: " +aKey+ ", Value: " +aValue);
    
    doCmd("setValue", aKey, aValue);    
    return mCmdResponseData;    
}

function getValue(aKey)
{
    doCmd("getValue", aKey);   

    if (mCmdResponseData == "")
    {
        return null;
    }

    var value = atob(mCmdResponseData); // Convert from BASE64

    return value;    
}

function signValue(aKey)
{
    doCmd("signValue", aKey);   

    if (mCmdResponseData == "")
    {
        return null;
    }

    var value = mCmdResponseData; // Keep in BASE64

    return value;    
}

function getParam(aKey)
{
    doCmd("getParam", aKey);   

    if (mCmdResponseData == "")
    {
        return null;
    }

    var value = atob(mCmdResponseData); // Convert from BASE64

    return value;    
}

function readPage(aPage)
{
    doCmd("readPage", aPage);   

    if (mCmdResponseData == "")
    {
        return null;
    }

    var pageText = atob(mCmdResponseData); // Convert from BASE64

    return pageText;    
}

function writePage(aPage, aData)
{
    doCmd("writePage", aPage, aData);    
    return mCmdResponseData;    
}

function deletePage(aPage)
{

}

function getUrl(aPage, aHeaders, aCallback)
{
    //itrace("getUrl, Page:" +aPage);   
    return DEVICEWEB.getUrl(aPage, aHeaders, aCallback);
}

function postUrl(aUrl, aHeaders, aBody, aCallback)
{
    //itrace("postUrl, Url: " +aUrl);
    return DEVICEWEB.postUrl(aUrl, aHeaders, aBody, aCallback);
}

var mCellularRequestCount = 0;
var mCellularRequestById  = {};

function getCellularUrl(aUrl, aHeaders, aCallback)
{
    //itrace("getCellularUrl, Url:" +aUrl);   
    
    var headers = "";

    if (aHeaders != null)
    {
        headers = JSON.stringify(aHeaders);
    }

    //trace("getCellularUrl, Headers:" +headers);   

    mCellularRequestCount++;

    var requestId = "RQST-HTTP-CELL-" +mCellularRequestCount;

    mCellularRequestById[requestId] = {callback: aCallback, url: aUrl};

    doCmd("getCellularUrl", requestId, aUrl, headers);    

    var submitted = mCmdResponseStatus == "1";

    return submitted;   
}

function postCellularUrl(aUrl, aHeaders, aBody, aCallback)
{
    //itrace("postCellularUrl, Url:" +aUrl);   
    
    var headers = "";

    if (aHeaders != null)
    {
        headers = JSON.stringify(aHeaders);
    }

    var body = "";

    if (aBody != null)
    {
        body = aBody;
    }

    //trace("postCellularUrl, Headers:" +headers);   

    mCellularRequestCount++;

    var requestId = "RQST-HTTP-CELL-" +mCellularRequestCount;

    mCellularRequestById[requestId] = {callback: aCallback, url: aUrl};

    doCmd("postCellularUrl", requestId, aUrl, headers, body);    

    var submitted = mCmdResponseStatus == "1";

    return submitted;   
}

function onCellularResponse(aRequestId, aStatus, aHeader, aBody)
{
    //trace("onCellularResponse, ID:" +aRequestId+ ", Status: " +aStatus);   

    var success = aStatus  == "1";
    var header  = aHeader == "" ? "" : atob(aHeader);
    var body    = aBody   == "" ? "" : atob(aBody);

    //trace("onCellularResponse, Header: " +header+ ", Body: " +body);       

    var request = mCellularRequestById[aRequestId];

    if (request == null)
    {
        warn("onCellularResponse, No Callback for Request: " +aRequestId);
        return;
    }

    mCellularRequestById[aRequestId] = null;

    var callback = request.callback;
    var url      = request.url;

    setTimeout(callback, 0, url, success, header, body);
}

function setHeaderStyle(aShowStatusBar, aShowNavBar)
{
    var statusBar = aShowStatusBar ? "1" : "0";
    var navBar    = aShowNavBar    ? "1" : "0";

    doCmd("setHeaderStyle", statusBar, navBar);    
}

var mCommandRequestCount = 0;
var mCommandRequestById  = {};

function sendGetCommand(aUrl, aCallback)
{
    itrace("sendGetCommand, Url: " +aUrl);

    mCommandRequestCount++;

    var requestId = "RQST-GET-CMD-" +mCommandRequestCount;

    mCommandRequestById[requestId] = {callback: aCallback, url: aUrl};

    doCmd("sendGetCommand", requestId, aUrl); 

    return requestId;   
}

function sendSetCommand(aUrl, aPayload, aCallback)
{
    itrace("sendSetCommand, Url: " +aUrl+ ", Payload: " +aPayload);

    mCommandRequestCount++;

    var requestId = "RQST-SET-CMD-" +mCommandRequestCount;

    mCommandRequestById[requestId] = {callback: aCallback, url: aUrl};

    doCmd("sendSetCommand", requestId, aUrl, aPayload); 

    return requestId;   
}

function onGetCommandResponse(aRequestId, aSuccess, aPayload, aDecode)
{
    itrace(">> BRIDGE::onGetCommandResponse");

    var request = mCommandRequestById[aRequestId];

    if (request == null)
    {
        warn("onGetCommandResponse, No Callback for Request: " +aRequestId);
        return;
    }

    mCommandRequestById[aRequestId] = null;

    var callback = request.callback;

    var success = aSuccess == "1";
    var decode  = aDecode  == "1";

    var payload = aPayload;

    if (decode)
    {
        payload = atob(aPayload);
    }

    setTimeout(callback, 0, aRequestId, success, payload);    
}

function onSetCommandResponse(aRequestId, aSuccess, aPayload, aDecode)
{
    itrace(">> BRIDGE::onSetCommandResponse, Success: " +aSuccess+ ", Decode: " +aDecode);

    var request = mCommandRequestById[aRequestId];

    if (request == null)
    {
        warn("onSetCommandResponse, No Callback for Request: " +aRequestId);
        return;
    }

    mCommandRequestById[aRequestId] = null;

    var callback = request.callback;

    var success = aSuccess == "1";
    var decode  = aDecode  == "1";

    var payload = aPayload;

    if (decode)
    {
        payload = atob(aPayload);
    }

    setTimeout(callback, 0, aRequestId, success, payload);
}

function doPageEvent(aEvent)
{
    //itrace("+++ BRIDGE::onPageEvent, Event: " +aEvent);
    
    if (typeof onPageEvent === "function")
    {
        setTimeout(onPageEvent, 0, aEvent);
    }
}

function popPage()
{
    doCmd("popPage");        
}

function popToHomePage()
{
    doCmd("popToHomePage");        
}

function setPageBackground(aBackground)
{
    doCmd("setPageBackground", aBackground);
}

var exports = {};

exports.popPage = function() { return popPage(); }

exports.popToHomePage = function() { return popToHomePage(); }

exports.setPageBackground = function(aBackground) { return setPageBackground(aBackground); }

exports.getPageQuery = function() { return getPageQuery(); }
exports.getPageUrl   = function() { return getPageUrl(); }

exports.showSpinner  = function(aActive, aPos, aBackground) { return showSpinner(aActive, aPos, aBackground); }
exports.showOkDialog  = function(aTitle, aMessage)          { return showOkDialog(aTitle, aMessage); }

exports.setAppState  = function(aStatus) { return setAppState(aStatus); }

exports.getRequestHostUrl = function() { return getRequestHostUrl(); }

exports.getValue       = function(aKey)            { return getValue(aKey); }
exports.setValue       = function(aKey, aValue)    { return setValue(aKey, aValue); }

exports.signValue      = function(aValue)          { return signValue(aValue); }

exports.getParam       = function(aKey)            { return getParam(aKey); }

exports.readPage       = function(aPage)           { return readPage(aPage); }
exports.writePage      = function(aPage, aData)    { return writePage(aPage, aData); }
exports.deletePage     = function(aPage)           { return deletePage(aPage); }

exports.fetchUrl        = function(aUrl, aCallback, aHeaders)        { return getUrl(aUrl, aHeaders, aCallback); }
exports.getUrl          = function(aUrl, aHeaders, aCallback)        { return getUrl(aUrl, aHeaders, aCallback); }
exports.postUrl         = function(aUrl, aHeaders, aBody, aCallback) { return postUrl(aUrl, aHeaders, aBody, aCallback); }


exports.getCellularUrl    = function(aUrl, aHeaders, aCallback)        { return getCellularUrl(aUrl, aHeaders, aCallback); }
exports.postCellularUrl   = function(aUrl, aHeaders, aBody, aCallback) { return postCellularUrl(aUrl, aHeaders, aBody, aCallback); }

exports.onCellularResponse  = function(aRequestId, aStatus, aHeader, aBody) { onCellularResponse(aRequestId, aStatus, aHeader, aBody); }

exports.setHeaderStyle   = function(aShowStatusBar, aShowNavBar) { return setHeaderStyle(aShowStatusBar, aShowNavBar); }

exports.sendGetCommand   = function(aUrl, aCallback) { return sendGetCommand(aUrl, aCallback); }
exports.sendSetCommand   = function(aUrl, aPayload, aCallback) { return sendSetCommand(aUrl, aPayload, aCallback); }

exports.onSetCommandResponse = function(aRequestId, aSuccess, aPayload, aDecode) { return onSetCommandResponse(aRequestId, aSuccess, aPayload, aDecode); }
exports.onGetCommandResponse = function(aRequestId, aSuccess, aPayload, aDecode) { return onGetCommandResponse(aRequestId, aSuccess, aPayload, aDecode); }

exports.doPageEvent = function(aEvent) { return doPageEvent(aEvent); }

return exports;

}());


function onCellularResponse(aRequestId, aStatus, aHeader, aBody)
{
    BRIDGE.onCellularResponse(aRequestId, aStatus, aHeader, aBody);
}


var MMRSNATIVE=BRIDGE;


