var CANVASTEST = (function() {
"use strict"

var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }


var mIsNative = MMRSUTILS.isNativeRun();

var mTheApp = angular.module('app', []);

mTheApp.controller('controller', function($scope)
{
    var mContext = null;

    var mPrevX = null;
    var mPrevY = null;

    var mMouseDown = false;

    $scope.onMenuButtonClick = function(aAction)
    {
        trace("onMenuButtonClick, Action: " +aAction);

        DEVICEBRIDGE.onMenuButtonClick(aAction);
    }

    function onMouseMove(aEvent)
    {
        trace(">> onMouseMove");  

        aEvent.preventDefault();

        //DEVICEBRIDGE.showOkDialog("Mouse", "onMouseMove");

        if (!mMouseDown)
        {
            return;
        }

        var event = aEvent;

        if (mIsNative)
        {
            var touches = aEvent.touches;

            var event = touches[0];       
        }

        var currentX = event.pageX;
        var currentY = event.pageY; 

        drawLine(mPrevX, mPrevY, currentX, currentY);

        mPrevX = currentX;
        mPrevY = currentY;
    }

    function onMouseDown(aEvent)
    {
        trace(">> onMouseDown"); 

        aEvent.preventDefault();

        //DEVICEBRIDGE.showOkDialog("Mouse", "onMouseDown");

        var event = aEvent;

        if (mIsNative)
        {
            var touches = aEvent.touches;

            event = touches[0]; 
        }

        mPrevX = event.pageX;
        mPrevY = event.pageY;      

        mMouseDown = true;

        mContext.beginPath();             
    }

    function onMouseUp(aEvent)
    {
        trace(">> onMouseUp");  

        aEvent.preventDefault();        

        //DEVICEBRIDGE.showOkDialog("Mouse", "onMouseUp");

        mMouseDown = false;

        mContext.closePath();
    }

    function onCancel(aEvent)
    {
        trace(">> onCancel");  

        aEvent.preventDefault();        

        //DEVICEBRIDGE.showOkDialog("Mouse", "onCancel");

        mMouseDown = false;

        mContext.closePath();
    }    

    function drawLine(aX1, aY1, aX2, aY2)
    {
        //trace("drawLine, (" +aX1+ "," +aY1+ ") -> (" +aX2+ "," +aY2+ ")");

        mContext.moveTo(aX1, aY1);
        mContext.lineTo(aX2, aY2);
            
        mContext.stroke();      
    }


    function doLoad()
    {   
        trace(">> doLoad");

        var container = document.getElementById("touchContainer");;

        var canvas = document.getElementById("touch");;

        if (canvas == null || container == null)
        {
            error("doLoad, Cannot find the canvas");
            return;
        }

        trace("doLoad, Found the canvas");

        var screenSize = MMRSUTILS.getScreenDimensions();

        canvas.width  = "" + screenSize.w;
        canvas.height = "" + screenSize.h;

        var bubbleDown = true;

        var startEvent = mIsNative  ? "touchstart"  : "mousedown";
        var stopEvent  = mIsNative  ? "touchend"    : "mouseup";
        var moveEvent  = mIsNative  ? "touchmove"   : "mousemove";
        var cancelEvent = mIsNative ? "touchcancel" : "mouseup";

        canvas.addEventListener(startEvent,  onMouseDown, bubbleDown);
        canvas.addEventListener(stopEvent,   onMouseUp,   bubbleDown);
        canvas.addEventListener(moveEvent,   onMouseMove, bubbleDown);        
        //canvas.addEventListener(cancelEvent, onCancel,   bubbleDown);

        mContext = canvas.getContext('2d');

        if (mContext == null)
        {
            trace("doLoad, Could not get context");
            return;
        }

        mContext.strokeStyle = "black";        
        mContext.lineWidth   = 3;

        //mContext.beginPath();mContext.moveTo(0, 0); mContext.lineTo(200, 200); mContext.closePath(); mContext.stroke();             

        $scope.$applyAsync(function()
        {
        });

        trace("<< doLoad");
    }

    doLoad();
});
    
var mOptions = null;

var mMenuItems = [];


var exports = {};

exports.name    = "canvastest";


return exports;

}());
