/*jslint browser: true*/
/*global $, jQuery, alert, angular, MMRSUTILS, MMRSCLIENTGATEWAY, MMRSCONFIG, MMRSCHARTUTILS*/
var MMRSACTIONSLOG = (function () {

    var mTag = "mmrsactionslog";

    function trace(aText) { MMRSUTILS.doTrace(mTag, aText);  }
    function note(aText)  { MMRSUTILS.doTrace(mTag, aText);  }
    function warn(aText)  { MMRSUTILS.doWarn(mTag, aText);   }
    function error(aText) { MMRSUTILS.doError(mTag, aText);  }
    
    angular.module('mmrsApp', []).controller('actionslogcontrol', function ($scope)
    {
        var self = this;    
        
        $scope.headings = ["Type", "Status", "Date Created", "Assigned To", "Details",  "Created By", "Update"];

        $scope.actionsById = {};

        self.dateTime = function(aTime)
        {
            //trace("dateTime, Time: " +aTime);
            
            if (aTime == null || aTime == "")
            {
                return "";
            }

            var date = new Date(aTime*1000);

            var day   = date.getDate();
            var month = date.getMonth() + 1;
            var year  = date.getFullYear();

            month = "0" + month;
            year  = ""  + year;

            var dateStr = day+ "/" +month.substr(-2)+ "/" +year.substr(-2);

            var hours   = "0" +date.getHours();
            var minutes = "0" +date.getMinutes();
            var seconds = "0" +date.getSeconds();

            var timeStr = hours.substr(-2) + ':' + minutes.substr(-2) + ':' + seconds.substr(-2);

            var dateTimeStr = timeStr+ " " +dateStr;
            
            return dateTimeStr;
        }

        self.getActorName = function(aActor)
        {
            if (aActor == "REMOTE") return "Call Center";
            if (aActor == "GENIUS") return "In Store";
            if (aActor == "OWNER")  return "Device";
            if (aActor == "LOCAL")  return "Device";
            return aActor;
        }

        self.getStatusName = function(aStatus)
        {
            if (aStatus == "open")     return "Open";
            if (aStatus == "resolved") return "Resolved";
            if (aStatus == "closed")   return "Closed";
            return aStatus;
        }        

        self.onCreateAction = function()
        {
            trace("onCreateAction");      

            $scope.newActionText     = "";
            $scope.newActionAssignee = "UNASSIGNED";

            $('#actionPrompt').modal({keyboard: true}).on("hidden.bs.modal", function (aEvent)
               {
                    trace("onCreateNote, Dialog Closed");            
               });
        }

        self.onCreateNote = function()
        {
            trace("onCreateNote");            
        }

        self.onRefreshActions = function()
        {
            trace("onRefreshActions");    

            //$('#actionsWait').modal('show') 

            getActionsLog();
        }

        self.doActionSaveClick = function()
        {   
            var actionText = $scope.newActionText;
            var assignee   = $scope.newActionAssignee;

            trace("doActionSaveClick, Text: " +actionText+ ", Assignee: " +assignee);      

            if (actionText == null || actionText == "")
            {
                return;
            }

            if (assignee == "UNASSIGNED")
            {
                return;
            }

            $('#actionPrompt').modal('hide') 

            sendNewAction(actionText, assignee);
        }

        self.onCloseActionClick = function(aActionId)
        {
            trace("onCloseActionClick, Action: " +aActionId);     

            var thisAction = $scope.actionsById[aActionId];

            if (thisAction == null)
            {
                error("onCloseActionClick, Cannot find Action: " +aActionId);
                return;
            }

            $scope.closeActionId      = aActionId;
            $scope.closeActionSummary = thisAction.text;
            $scope.closeActionText    = "";

            $('#closeActionPrompt').modal({keyboard: true}).on("hidden.bs.modal", function (aEvent)
               {
                    trace("onCloseActionClick, Dialog Closed");            
               });    
        }

        self.doActionClose = function()
        {   
            var closeText = $scope.closeActionText;
            var actionId  = $scope.closeActionId;

            trace("doActionClose, Reason: " +closeText+ ", Action: " +actionId);      

            if (closeText == null || closeText == "")
            {
                return;
            }

            $('#closeActionPrompt').modal('hide') 

            sendUpdateActionStatus(actionId, closeText, "closed");            
        }


        self.onReOpenActionClick = function(aActionId)
        {
            trace("onReOpenActionClick, Action: " +aActionId);            

            var thisAction = $scope.actionsById[aActionId];

            if (thisAction == null)
            {
                error("onReOpenActionClick, Cannot find Action: " +aActionId);
                return;
            }

            $scope.reOpenActionId      = aActionId;
            $scope.reOpenActionSummary = thisAction.text;
            $scope.reOpenActionText    = "";

            $('#reOpenActionPrompt').modal({keyboard: true}).on("hidden.bs.modal", function (aEvent)
               {
                    trace("onReOpenActionClick, Dialog Closed");            
               });              
        }        


        self.doActionReOpen = function()
        {   
            var reOpenText = $scope.reOpenActionText;
            var actionId   = $scope.reOpenActionId;

            trace("doActionReOpen, Reason: " +reOpenText+ ", Action: " +actionId);      

            if (reOpenText == null || reOpenText == "")
            {
                return;
            }

            $('#reOpenActionPrompt').modal('hide') 

            sendUpdateActionStatus(actionId, reOpenText, "open");            
        }

        getActionsLog = function()
        {
            trace("getActionsLog");            

            //var batcher = MMRSCLIENTGATEWAY.makeBatcher();
            //batcher.addGetRequest("root.logger.actions", onLogResponse);            
            //batcher.dispatchRequests();

            if (MMRSUTILS.isNativeRun())
            {
                MMRSCLIENTGATEWAY.sendGetCommandToDevice("root.logger.actions", function(aRequestId, aSuccess, aResponse)
                {
                    $scope.$apply(function() { doLogResponse(aResponse); });            
                });       
            }            
        }

        function readActions(aActionsList)
        {
            var actionsById = {};

            for (var index in aActionsList)
            {
                var thisAction = aActionsList[index];
                
                var entryId        = thisAction.entryId;
                var entryType      = thisAction.entryType;
                var actionType     = thisAction.entrySubType;
                var actionTime     = thisAction.entryTime;
                var actionText     = thisAction.text;
                var actionAuthor   = thisAction.actor;
                var actionAssignee = thisAction.assignee;
                var actionStatus   = thisAction.status;

                trace("readActions, Index: " +index+ ", Id: " +entryId+ ", Entry: " +entryType+ ", Type: " +actionType+ ", Text: " +actionText+ ", Status: " +actionStatus);

                if (actionsById[entryId] == null)
                {
                    actionsById[entryId] = thisAction;
                    actionsById[entryId].updates = [];
                }
                else
                {
                    actionsById[entryId].updates.push(thisAction);            
                }

                actionsById[entryId].status = thisAction.status;                

            }

            $scope.actionsById = actionsById;
        }

        function doLogResponse(aResponse)
        {
            try
            {
                trace("doLogResponse: " +JSON.stringify(aResponse));

                //$('#actionsWait').modal('hide') 
                var response = aResponse[0];

                if (response == null || response == "") return;

                trace("doLogResponse, Node Title: " +response.nodetitle);

                var fields = response.fields[0];

                if (fields == null || fields == "") return;

                trace("doLogResponse, Field Title: " +fields.fieldtitle);

                var fieldValue = fields.fieldvalue;

                if (fieldValue == null || fieldValue == "") return;

                trace("doLogResponse, Field Value: " +fieldValue);

                var actions = JSON.parse(fieldValue);

                readActions(actions);

                //$scope.actions = actions;
            }
            catch (err)
            {
                error("doLogResponse, Exception: " +err);
            }
        }
        
        function onLogResponse(aRequestId, aSuccess, aResponse)
        {
            trace("onLogResponse: " +aResponse);

            $scope.$apply(doLogResponse(aResponse));
        }

        function sendNewAction(aActionText, aActionAssignee)
        {
            trace("sendNewAction, Text: " +aActionText+ ", Assignee: " +aActionAssignee);

            var update =
            {
                action: "update",

                fields: [
                    {
                        action: "upate",
                        field:  "actionAddText",
                        value:  aActionText
                    },
                    {
                        action: "update",
                        field:  "actionAddAssignee",
                        value:  aActionAssignee
                    }
                ]
            };
            
            var actionUrl = "root.logger.actionadd";

            trace("sendNewAction, Sending action to device");
            
            //$('#actionsWait').modal('show');            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(actionUrl, update, function (aRequestId, aSuccess, aResponse)
            {            
                $scope.$apply(function ()
                {
                    trace("sendNewAction, sendSetCommandToDevice: Success: " + aSuccess);

                    //$('#actionsWait').modal('hide');            
                    if (aSuccess)
                    {
                        getActionsLog();
                    }
                });
            });
        }


        function sendUpdateActionStatus(aActionId, aText, aStatus)
        {
            trace("sendUpdateActionStatus, Id: " +aActionId+ "Text: " +aText+ ", Assignee: " +aStatus);

            var update =
            {
                action: "update",
                fields: [
                            { action: "upate",  field:  "actionUpdateId",     value:  aActionId },
                            { action: "update", field:  "actionUpdateStatus", value:  aStatus   },
                            { action: "upate",  field:  "actionUpdateText",   value:  aText     }
                        ]
            };
            
            var actionUrl = "root.logger.actionupdate";

            trace("sendUpdateActionStatus, Sending action update to device");
            
            //$('#actionsWait').modal('show');            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(actionUrl, update, function (aRequestId, aSuccess, aResponse)
            {            
                $scope.$apply(function ()
                {
                    trace("sendUpdateActionStatus, sendSetCommandToDevice: Success: " + aSuccess);

                    //$('#actionsWait').modal('hide');            
                    if (aSuccess)
                    {
                        getActionsLog();
                    }
                });
            });
        }

        /*
        MMRSCLIENTGATEWAY.waitForClientCommandConnectionEvent(function (aChannelStatus)
        {
            trace("actionslogcontrol, Got connection event");            

            if (aChannelStatus === MMRSCLIENTGATEWAY.CHANNEL_STATUS_CONNECTED)
            {
                trace("Got connected event");
                $scope.$apply(getActionsLog());
            }
        });
        */

        getActionsLog();

    });

}());
