/*jslint browser: true*/
/*global $, jQuery, alert, angular, MMRSUTILS, MMRSCLIENTGATEWAY, MMRSCONFIG*/
var MMRSASSIST = (function () {
    "use strict";
    
    function trace(aText) {
        MMRSUTILS.doTrace("mmrsassist", aText);
    }

    function note(aText) {
        MMRSUTILS.doTrace("mmrsassist", aText);
    }

    function warn(aText) {
        MMRSUTILS.doWarn("mmrsassist", aText);
    }

    function error(aText) {
        MMRSUTILS.doError("mmrsassist", aText);
    }
    
    angular.module('mmrsApp').factory('mmrsAssist', ['$rootScope', '$timeout', 'mmrsQuickCheckFactory', 'mmrsAssistPages', function ($rootScope, $timeout, aQuickCheckFactory, aAssistPages) {
        var mmrsAssistInstance = null,
            LAUNCHING_SCREEN = "LAUNCHING_SCREEN",
            TAKING_FIRST_SCREENSHOT = "TAKING_FIRST_SCREENSHOT",
            REFRESHING_SCREENSHOT = "REFRESHING_SCREENSHOT",
            USER_AUTH_ERROR = "USER_AUTH_ERROR",
            GENERAL_ERROR = "GENERAL_ERROR",
            NO_SCREEN_SHOT = "NO_SCREEN_SHOT",
            GOT_SCREEN_SHOT = "GOT_SCREEN_SHOT";
        
        mmrsAssistInstance = {
            displayDialog : null, // Function for launching the MMRS Assist Dialog
            setDialogContent : null, // Function for setting the MMRS dialog content
            screenShotStatus : null,
            screenShotSrc : null,
            pageData : null,
            showBackButton : false,
            previousPageIds : [],
            thisPageId : null,
            LAUNCHING_SCREEN : LAUNCHING_SCREEN,
            TAKING_FIRST_SCREENSHOT : TAKING_FIRST_SCREENSHOT,
            REFRESHING_SCREENSHOT : REFRESHING_SCREENSHOT,
            USER_AUTH_ERROR : USER_AUTH_ERROR,
            GENERAL_ERROR : GENERAL_ERROR,
            NO_SCREEN_SHOT : NO_SCREEN_SHOT,
            GOT_SCREEN_SHOT : GOT_SCREEN_SHOT
        };
        
        function doStatusUpdateCallback() {
            trace(">> doStatusUpdateCallback");
            /*
            if (statusUpdateCallback) {
                trace("Doing callback");
                statusUpdateCallback(screenShotStatus, screenShotSrc);
            }
            */
            trace("<< doStatusUpdateCallback");
        }
        
        function clearScreenShot() {
            trace("Got clearScreenShot");
            mmrsAssistInstance.screenShotSrc = null;
            mmrsAssistInstance.screenShotStatus = NO_SCREEN_SHOT;
            doStatusUpdateCallback();
        }

        // TODO: Move the screen shotter to a service
            // TODO: how to manage the callback? via a promise?
        
        // Screen shot stuff
        // TODO: a lot of this is shared with the client tree screen shot controller
        // TODO: consider the best way to share it

        function onScreenShotDone(aRequestId, aSuccess, aData) {
            $rootScope.$apply(function () { // Doing an $apply on the root scope isn't very nice. We might be able to get rid of this if/when device protocol requests are handled by an Angular service
                if (!aSuccess) {
                    error("onScreenShotDone, Request Failed: " + aRequestId);
                    mmrsAssistInstance.screenShotStatus = GENERAL_ERROR;
                    doStatusUpdateCallback();
                    return;
                }

                if (!aData) {
                    mmrsAssistInstance.screenShotStatus = USER_AUTH_ERROR;
                    doStatusUpdateCallback();
                    //--                trace("<< onScreenShotDone");   
                    return;
                }

                mmrsAssistInstance.screenShotSrc = "data:image/jpeg;base64," + aData;
                mmrsAssistInstance.screenShotStatus = GOT_SCREEN_SHOT;
                doStatusUpdateCallback();
            });

            trace("<< onScreenShotDone");
        }
        
        function takeScreenShot() {
            var requestUrl = MMRSCONFIG.GAE_HOST + "root.screenshot.capture",
                requestId = null;

            mmrsAssistInstance.screenShotSrc = null;
            doStatusUpdateCallback();
            requestId = MMRSCLIENTGATEWAY.sendGetCommandToDevice(requestUrl, onScreenShotDone);
        }
        
        mmrsAssistInstance.takeFirstScreenShot = function () {
            mmrsAssistInstance.screenShotStatus = TAKING_FIRST_SCREENSHOT;
            takeScreenShot();
        };

        mmrsAssistInstance.onScreenShotClick = function () {
            mmrsAssistInstance.screenShotStatus = REFRESHING_SCREENSHOT;
            takeScreenShot();
        };
        
        mmrsAssistInstance.onBackClick = function () {
            var previousPageId = mmrsAssistInstance.previousPageIds.pop();
            if (previousPageId) {
                mmrsAssistInstance.setDialogContent(previousPageId, true);
            } else {
                window.history.back();
            }
            
            mmrsAssistInstance.showBackButton = (mmrsAssistInstance.previousPageIds.length > 0);
        };
        
        mmrsAssistInstance.setDialogContent = function (aPageId, aBackClicked) {
            var testGroup,
                pageItemsArray,
                pageItemIndex,
                pageItem,
                screenId,
                screenAction,
                testItemIndex,
                testItem,
                // batcher = MMRSCLIENTGATEWAY.makeBatcher(),
                batcher = {
                    addGetRequest : function (aRequestUrl, aCallback) {
                        MMRSCLIENTGATEWAY.sendGetCommandToDevice(aRequestUrl, function (aRequestId, aSuccess, aResponseData) {
                            var firstResponseItem = aResponseData[0]; // For local requests we only support one request at a time, so we only expect one item in the response
                            if (firstResponseItem) {
                                trace("*** firstResponseItem: " + JSON.stringify(firstResponseItem));
                                aCallback(aRequestId, aSuccess, firstResponseItem);
                            } else {
                                trace("*** No firstResponseItem");
                            }
                        });
                    }, // This is a fake batcher, the local requirest in made immediately
                        // MMRSCLIENTGATEWAY.sendGetCommandToDevice(aRequestUrl, aCallback);
                    
                    dispatchRequests : function () {
                        // Do nothing - the requests have already been sent
                    }
                },
                testStatus,
                assistPages = aAssistPages,
                originalPageData,
                pageData;
            
            if ((mmrsAssistInstance.thisPageId) && (!aBackClicked)) {
                mmrsAssistInstance.previousPageIds.push(mmrsAssistInstance.thisPageId);
            }
            
            mmrsAssistInstance.thisPageId = aPageId;
            
            mmrsAssistInstance.showBackButton = (mmrsAssistInstance.previousPageIds.length > 0);
            
            clearScreenShot();

            originalPageData = assistPages.pages[aPageId];
            pageData = jQuery.extend(true, {}, originalPageData);
            
            // Deep copy the page data so we don't modify the original when we app properties later            
            mmrsAssistInstance.pageData = pageData;

            pageItemsArray = mmrsAssistInstance.pageData.items;
            
            // TODO: add handling for case when there are "items" does not exit in the page data - bail with error
            
            for (pageItemIndex = 0; pageItemIndex < pageItemsArray.length; pageItemIndex = pageItemIndex + 1) {
                pageItem = pageItemsArray[pageItemIndex];
                if (pageItem.type === "TEST_GROUP") {
                    pageItem.tests = [];
                    for (testItemIndex = 0; testItemIndex < pageItem.items.length; testItemIndex = testItemIndex + 1) {
                        testItem = pageItem.items[testItemIndex];

                        // Get the test status for each item by testid + create the test (pass in batcher, testid)
                        testStatus = aQuickCheckFactory.createQuickCheck(testItem.testid, batcher, mmrsAssistInstance);
                        
                        if (testItem.criticalTest) {
                            // TODO: this seems to be the wrong place to be hard coding the replace device / mmrs.deviceReplacement link
                                // TODO: could this be defined in mmrsassistpages somewhere?
                            aQuickCheckFactory.setFailureAssistPageLink(testStatus, "Click here to replace device", "", "mmrs.deviceReplacement");
                        }
                        
                        if (testStatus) {
                            pageItem.tests.push(testStatus);
                        } else {
                            error("No testStatus returned for testid: " + testItem.testid);
                        }
                    }
                }
            }
            
            if (pageItem.condition) {
                pageItem.visible = false; // If a page item is conditional on some state change then make it invisible to start with
            } else {
                pageItem.visible = true;
            }

            batcher.dispatchRequests(); // Note that the batcher will only send a network request if requests have actually been added
            
            if (pageData.launchscreen) {
                trace(">> mmrsAssistInstance.displayDialog");
                screenId = pageData.launchscreen;
                screenAction = {
                    action: "update",
                    fields: [{
                        action: "update",
                        field: screenId,
                        value: "screen-start"
                    }]
                };

                // mmrsAssistInstance.statusUpdateCallback = aStatusUpdateCallback;

                clearScreenShot();

                mmrsAssistInstance.screenShotStatus = LAUNCHING_SCREEN;
                doStatusUpdateCallback();

                MMRSCLIENTGATEWAY.sendSetCommandToDevice(pageData.launchScreenProvider, screenAction, function (aRequestId, aResult, aSetCommandStatus) {
                    trace("sendSetCommandToDevice complete");
                    // Maybe have a combined action that performs the action and returns a screenshot? This could improve performance by avoiding two seperate messages
                    // TODO: speak to Billy about the above when he's back
                    if (pageData.screenshot) {
                        $timeout(function () {
                            mmrsAssistInstance.takeFirstScreenShot();
                        }, 1000); // Wait to give the activity time to open
                    }
                });
            }

            mmrsAssistInstance.title = pageData.title;
            
            trace("*** Go to document top");
            $("#topDiv").scrollTop(0); // This scrolls to the top of the screen - it might be nicer if called from the HTML rather than the controller (the controller shouldn't know about the HTML)
        };
        
        mmrsAssistInstance.displayDialog = function (aPageId) {
            mmrsAssistInstance.previousPageIds = [];
            mmrsAssistInstance.showBackButton = false;
            mmrsAssistInstance.thisPageId = null;
            
            mmrsAssistInstance.setDialogContent(aPageId);

            // $('#mmrsAssistModal').modal('show'); // TODO: there must be a more Angular way to make the modal visible - rather than invoking it with jQuery

            trace("<< mmrsAssistInstance.displayDialog");
        };
        
        mmrsAssistInstance.startScreen = function (aScreenUrl, aScreenId) {
            var screenAction = {
                action: "update",
                fields: [{
                    action: "update",
                    field: aScreenId,
                    value: "screen-start"
                }]
            };

            MMRSCLIENTGATEWAY.sendSetCommandToDevice(aScreenUrl, screenAction, function (aRequestId, aSuccess, aSetCommandStatus) {
                // Ignore result
            });
        }
                        
        return mmrsAssistInstance;
    }]);
}());
