/*jslint browser: true*/
/*global angular*/
var MMRSASSISTPAGES = (function () {
    "use strict";
    
    // item types:
        // TEXT : text
        // HELP_TOPIC : title, text, items (optional array of text + image items)
        // TEST_GROUP : title, items
        // PAGE_LINK : text, linkedPageId
        // "ACTION_CREATOR" : <none>
    
    // action types:
        // LAUNCH_SCREEN
    
    angular.module('mmrsApp').factory('mmrsAssistPages', [function () {
        var pagesData = {
            pages : {
                "mmrs.batteryQuickChecks" : {
                    title : "Battery Checks",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "The following tests identify issues causing high battery drain:"
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Battery Drain",
                            items : [
                                {
                                    testid : "tests.battery.health"
                                },
                                {
                                    testid : "tests.battery.screen-auto-brightness"
                                },
                                {
                                    testid : "tests.battery.screen-timeout"
                                },
                                {
                                    testid : "tests.battery.screen-brightness-level"
                                }
                            ]
                        },
                        {
                            type : "TEXT",
                            text : "If this does not resolve the issue then launch the Android battery settings screen for more information:"
                        },
                        {
                            type : "SCREEN_LINK",
                            launchScreenProvider: "root.intents.screens",
                            launchscreen : "android/intent/action/POWER_USAGE_SUMMARY",
                            text : "Click here to launch the Android battery settings screen"
                        }
                    ]
                },
                
                "mmrs.audioQuickChecks" : {
                    title : "Audio Checks",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "The following tests identify any problems with the audio settings:"
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Audio Checks",
                            items : [
                                {
                                    testid : "tests.audio.callVolume"
                                },
                                {
                                    testid : "tests.audio.mediaVolume"
                                },
                                {
                                    testid : "tests.audio.ringVolume"
                                }
                            ]
                        },
                        {
                            type : "TEXT",
                            text : "If this does not resolve the issue then test the device hardware:"
                        },
                        {
                            type : "PAGE_LINK",
                            text : "Click here to launch the audio hardware test",
                            linkedPageId : "mmrs.audioHardwareTest"
                        }
                    ]
                },
                
                "mmrs.audioHardwareTest" : {
                    title : "Audio Hardware Tests",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "This test identifies any problems with the device speakers."
                        },
                        {
                            type : "TEXT",
                            text : "Start the test on the device by clicking the “Start test” link."
                        },
                        {
                            type : "TEXT",
                            text : "There are two speaker tests “Speaker” and “Earpiece”. Both tests must be run by tapping the appropriate button on the device."
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Speaker Test",
                            items : [
                                {
                                    testid : "tests.hardware.speaker" // ,
                                    // criticalTest : true
                                }
                            ]
                        }
                    ]
                },
                
                "mmrs.mobileNetwork.calls" : {
                    title : "Making or Receiving Calls",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "The following tests identify problems that could prevent the device making of receiving calls:"
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Mobile Network and SIM",
                            items : [
                                {
                                    testid : "tests.mobilenetwork.sim"
                                },
                                {
                                    testid : "tests.mobilenetwork.flightmode"
                                },
                                {
                                    testid : "tests.mobilenetwork.service"
                                },
                                {
                                    testid : "tests.mobilenetwork.signalstrength"
                                }
                            ]
                        }
                    ]
                },
                
                
                "mmrs.mobileNetwork.data" : {
                    title : "Mobile Internet",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "The following tests identify problems that could prevent the device from using mobile data:"
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Mobile Network and SIM",
                            items : [
                                {
                                    testid : "tests.mobilenetwork.sim"
                                },
                                {
                                    testid : "tests.mobilenetwork.flightmode"
                                },
                                {
                                    testid : "tests.mobilenetwork.service"
                                },
                                {
                                    testid : "tests.mobilenetwork.signalstrength"
                                }
                            ]
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Mobile Data",
                            items : [
                                {
                                    testid : "tests.mobilenetwork.data"
                                },
                                {
                                    testid : "tests.mobilenetwork.dataroaming"
                                }
                            ]
                        }
                    ]
                },
                
                "mmrs.screenQuickChecks" : {
                    title : "Screen",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "The following tests identify problems with the display settings:"
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Screen",
                            items : [
                                {
                                    testid : "tests.screen.lowbrightness"
                                },
                                {
                                    testid : "tests.battery.screen-auto-brightness"
                                },
                                {
                                    testid : "tests.screen.autorotate"
                                }
                            ]
                        },
                        {
                            type : "PAGE_LINK",
                            text : "If this does not resolve the issue, click here to test the phone hardware",
                            linkedPageId : "mmrs.screenHardwareTest"
                        }
                    ]
                },
                
                
                "mmrs.screenHardwareTest" : {
                    title : "Screen Hardware Tests",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "These tests identify any problems with the device screen."
                        },
                        {
                            type : "TEXT",
                            text : "Start each test on the device by clicking the appropriate “Start test” link."
                        },
                        {
                            type : "TEST_GROUP",
                            title : "Screen Tests",
                            items : [
                                {
                                    testid : "tests.hardware.screen.displayandtouch" // ,
                                    // criticalTest : true
                                },
                                {
                                    testid : "tests.hardware.lightsensor" // ,
                                    // criticalTest : true
                                },
                                {
                                    testid : "tests.hardware.backlight" // ,
                                    // criticalTest : true
                                }
                            ]
                        },
                        {
                            type : "PAGE_LINK",
                            text : "Click here to follow up in-store",
                            linkedPageId : "mmrs.inStoreReferral"
                        }
                    ]
                },
                
                
                "mmrs.deviceReplacement" : {
                    title : "Replace Device",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "Device replacement"
                        },
                        {
                            type : "TEXT",
                            text : "Authorization code: 2673-7235"
                        },
                        {
                            type : "TEXT",
                            text : "Note: This page is simulated - no replacement has been initiated"
                        }
                    ]
                },
                
                
                "mmrs.inStoreReferral" : {
                    title : "In-Store Follow Up",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "In-Store Follow Up"
                        },
                        {
                            type : "ACTION_CREATOR"
                        },
                        {
                            type : "TEXT",
                            text : "Note: This page is simulated - no store appointement has been booked"
                        }
                    ]
                    
                    // TODO: add action for in-store
                },
                
                
                "mmrs.troubleshootingWizardMenu" : {
                    title : "Troubleshooting Wizard",
                    screenshot : false,
                    items : [
                        {
                            type : "TEXT",
                            text : "This Troubleshooting Wizard can help identify problems with the device."
                        },
                        {
                            type : "TEXT",
                            text : "Select one of the following problem areas to continue:"
                        },
                        {
                            type : "PAGE_LINK",
                            text : "High battery drain",
                            linkedPageId : "mmrs.batteryQuickChecks"
                        },
                        {
                            type : "PAGE_LINK",
                            text : "The phone does not ring for incoming calls",
                            linkedPageId : "mmrs.audioQuickChecks"
                        },
                        {
                            type : "PAGE_LINK",
                            text : "Making or receiving calls",
                            linkedPageId : "mmrs.mobileNetwork.calls"
                        },
                        {
                            type : "PAGE_LINK",
                            text : "Mobile internet",
                            linkedPageId : "mmrs.mobileNetwork.data"
                        },
                        {
                            type : "PAGE_LINK",
                            text : "Screen",
                            linkedPageId : "mmrs.screenQuickChecks"
                        }
                    ]
                }
            }
        };
                
        return pagesData;
    }]);
}());