var MMRSAUTOCHECK = (function() {
	"use strict"

var mAngularScope = null;

var STATUS_ICON_INFO     = "images/statusInfo.png";
var STATUS_ICON_PASS     = "images/statusPass.png";
var STATUS_ICON_WARNING  = "images/statusWarning.png";
var STATUS_ICON_CRITICAL = "images/statusCritical.png"; 
var STATUS_ICON_FIXED    = "images/statusFixed.png";

var FIX_LINK_ARROW_INFO     = "images/fixLinkArrowInfo.png";
var FIX_LINK_ARROW_WARNING  = "images/fixLinkArrowWarning.png";
var FIX_LINK_ARROW_CRITICAL = "images/fixLinkArrowCritical.png";

var FIX_LINK_CELL_INFO     = "testCaseFixInfo";
var FIX_LINK_CELL_WARNING  = "testCaseFixWarning";
var FIX_LINK_CELL_CRITICAL = "testCaseFixCritical";

var mIsNativeRun = MMRSUTILS.isNativeRun();

var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();    

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }

window.onerror = function(aText) { error(aText); }

if (!String.format)
{
    String.format = function(format) {
    var args = Array.prototype.slice.call(arguments, 1);
    return format.replace(/{(\d+)}/g, function(match, number) { 
      return typeof args[number] != 'undefined'
        ? args[number] 
        : match
      ;
    });
  };
}

// Start a settings screen
function startScreen(aScreenUrl, aScreenId, aTestStatus) {
    var screenAction = {
        action: "update",
        fields: [{
            action: "update",
            field: aScreenId,
            value: "screen-start"
        }]
    };

    MMRSCLIENTGATEWAY.sendSetCommandToDevice(aScreenUrl, screenAction, function (aRequestId, aSuccess, aSetCommandStatus) {
        // $scope.$applyAsync(function () {
            // Do nothing - no need to update status
        // });
    });
}

var mTheApp = angular.module('mmrsApp', ['ngSanitize']);

//mTheApp.controller('mmrsController', ['$scope', 'mmrsAssist', 'mmrsAssistPages', function($scope)

mTheApp.controller('mmrsController', ['$scope', '$sce', function($scope, $sce)
{
    $scope.quickCheckGroups = [];

    /*
    Create a new (empty) test group and add it to aTestGroups
    */
    function addNewTestStatusToGroup(aTestGroup, aTitle)
    {
      	trace(">> addNewTestStatusToGroup");
	
        var newTestStatus = {
            "title"   : aTitle,
            "status"  : "CHECKING",
            "details" : "Please wait",
            "fixes"   : []
        };

        aTestGroup.tests.push(newTestStatus);

      	trace("<< addNewTestStatusToGroup");

        return newTestStatus;
    }
    
    /*
    Create a new info item to a group
    An info item is similar to a test, except it doesn't check any values (it always displays the same content)
    */
    function addNewInfoItemToGroup(aTestGroup, aTitle, aDetails)
    {
      	trace(">> addNewInfoItemToGroup");

        var newInfoStatus = {
            "title" : aTitle,
            "status" : "INFO",
            "details" : aDetails,
            "fixes" : []
        };

        aTestGroup.tests.push(newInfoStatus);

        return newInfoStatus;
    }

    /*
    Create a new (empty) test group and add it to aTestGroups
    Returns the created test group
    */
    function addNewTestGroup(aTestGroups, aTitle)
    {
      	trace(">> addNewTestGroup");

        var newTestGroup = {"title" : aTitle,
                            tests : []};

        aTestGroups.push(newTestGroup);

        return newTestGroup;
    }

    function setTestStatusOk(aTestStatus, aPassText)
    {
        trace(">> setTestStatusOk: " + aTestStatus.details);
    
        var passText = aPassText == null ? "No issues detected" : aPassText;

        aTestStatus.icon   = STATUS_ICON_PASS;
        aTestStatus.status = "OK";
        aTestStatus.details = passText;
//--        trace("<< setTestStatusOk");
    }
    
    // Add a possible fix to the test status
    // Used when a test gives a warning (or worse)
    // The fix could be something like "Change screen timeout to 30 seconds"
    // aFixFunction: callback function to apply the fix (called when the user selects the option to apply the fix). The callback function takes no parameters (assume that all data is already available to callback?)
    // aFixButtonText: the text displayed in the button (or link, or similar) that applies the fix
    // aFixTextDetails: extra info on what the fix will do (may be displayed as a hover item, or is a confirmation / info dialog)
    function addFixToTestStatus(aFixButtonText, aFixTextDetails, aTestStatus, aFixFunction)
    {
        trace(">> addFixToTestStatus, Pushing new fix to aTestStatus");

        var newFix = {
            "buttonText"   : $sce.trustAsHtml(aFixButtonText),
            "textDetails"  : aFixTextDetails,
            "fixFunction"  : aFixFunction,
            "hideLinkArrow" : true
        };

        if (aFixFunction) {
            // If there's no fix function then we shouldn't display the chevron
            // This is because the whole text isn't a link, there's probably an embedded <a href>
            // in the fix text itself.
            newFix.hideLinkArrow = false;
        }
        
        trace("addFixToTestStatus, Text: " +aFixButtonText);
        aTestStatus.fixes.push(newFix);

        trace("<< addFixToTestStatus");

    }      

    function setTestStatusInfo(aTestStatus, aDetails)
    {
        aTestStatus.icon         = STATUS_ICON_INFO;
        aTestStatus.fixLinkArrow = FIX_LINK_ARROW_INFO;
        aTestStatus.fixLinkClass = FIX_LINK_CELL_INFO;
        aTestStatus.status       = "INFO";
        aTestStatus.details      = aDetails;

        trace("setTestStatusInfo, +++ Title: " +aTestStatus.title+ ", Details: " +aDetails);

    }

    function setTestStatusWarning(aTestStatus, aDetails)
    {
        aTestStatus.icon         = STATUS_ICON_WARNING;   
        aTestStatus.fixLinkArrow = FIX_LINK_ARROW_WARNING;
        aTestStatus.fixLinkClass = FIX_LINK_CELL_WARNING;        
        aTestStatus.status       = "WARNING";
        aTestStatus.details      = aDetails;

        trace("setTestStatusWarning, +++ Title: " +aTestStatus.title+ ", Details: " +aDetails);

        sendCommentToLogger("warning", aTestStatus.title, aDetails);
    }

    function setTestStatusCriticalWarning(aTestStatus, aDetails)
    {
        aTestStatus.icon         = STATUS_ICON_CRITICAL;   
        aTestStatus.fixLinkArrow = FIX_LINK_ARROW_CRITICAL;
        aTestStatus.fixLinkClass = FIX_LINK_CELL_CRITICAL;        
        aTestStatus.status       = "CRITICAL_WARNING";
        aTestStatus.details      = aDetails;

        trace("setTestStatusCriticalWarning, +++ Title: " +aTestStatus.title+ ", Details: " +aDetails);

        sendCommentToLogger("critical warning", aTestStatus.title, aDetails);
    }

    function getDataFromDevice(aUrl, aResponseHandler)
    {
    	trace(">> getDataFromDevice");

        if (! mIsNativeRun) //>>> TEST
        {
            aResponseHandler("123", true, "");
            return; 
        }

        MMRSCLIENTGATEWAY.sendGetCommandToDevice(aUrl, function (aRequestId, aSuccess, aResponseData) {
            trace("getDataFromDevice, Callback");

            $scope.$applyAsync(function ()
            {
                var response = null;

                if (aResponseData != null) {
                    response = aResponseData[0];
                }

                aResponseHandler(aRequestId, aSuccess, response);
            });
        });

    	trace("<< getDataFromDevice");
    }
        
    // Utility function to set data on the device as part of an automated fix
    function applyFixToDevice(aUrl, aFixedData, aTestStatus) {
            trace(">> applyFixToDevice: aUrl: " + aUrl + " aFixedData: " + JSON.stringify(aFixedData));
            trace("<< applyFixToDevice");
    }

    function sendCommentToLogger(aLevel, aTitle, aDetail)
    {
        trace("sendCommentToLogger, Level: " +aLevel+ ", Title: " +aTitle+ ", Detail: " +aDetail);

        var update =
        {
            action: "update",
            fields: [
                        { action: "upate",  field:  "commentAddLevel",  value:  aLevel  },
                        { action: "update", field:  "commentAddTitle",  value:  aTitle  },
                        { action: "upate",  field:  "commentAddDetail", value:  aDetail }
                    ]
        };
        
        var actionUrl = "root.logger.commentadd";

        trace("sendUpdateActionStatus, Sending Comment to device");
        
        MMRSCLIENTGATEWAY.sendSetCommandToDevice(actionUrl, update, function (aRequestId, aSuccess, aResponse)
        {            
            $scope.$applyAsync(function ()
            {
                trace("sendCommentToLogger, sendSetCommandToDevice: Success: " + aSuccess);
            });
        });
    }
        
    function setFixFailed(aTestStatus) {
        aTestStatus.status  = "FIX_FAILED";
        aTestStatus.details = "Fix failed";
        aTestStatus.fixes   = [];
    }
        
    function fixFieldValueOnDevice(aFetchedData, aUrl, aFieldName, aNewValue, aTestStatus)
    {
            trace(">> fixFieldValueOnDevice");

        var field = MMRSUTILS.getFieldUsingSettingItem(aFetchedData, aFieldName),
            fieldNumber,
            updateField,
            updateAction;
        
        if ((field === null) || (field === undefined))
        {
            error("Unable to get field for: " + aFieldName);
            setFixFailed(aTestStatus);
        }
        else
        {
            fieldNumber = field.fieldid; // TODO: review comment: "fieldid" here vs. "field" in update action
            updateField = [{"action": "update",
                            "field": fieldNumber,
                            "value": aNewValue}];

            updateAction = {"action": "update",
                            "fields": updateField};
            
            aTestStatus.status = "FIXING";
            aTestStatus.details = "Applying fix - Please wait";
            aTestStatus.fixes = []; // clear fixes

            MMRSCLIENTGATEWAY.sendSetCommandToDevice(aUrl, updateAction, function (aRequestId, aSuccess, aSetCommandStatus) {
                $scope.$applyAsync(function () {
//--                        trace("Callback from: MMRSCLIENTGATEWAY.sendSetCommandToDevice: Success: " + aSuccess);
                    if (aSuccess) {
                        aTestStatus.status  = "FIXED";
                        aTestStatus.details = "Fix applied";
                        aTestStatus.fixes   = [];
                        aTestStatus.icon    = STATUS_ICON_FIXED;
                    } else {
                        setFixFailed(aTestStatus);
                    }
                });
            });
        }
        
//--            trace("<< fixFieldValueOnDevice");
    }
        
    
    ///////////////////////////////////////////////////////////////////////////////////////

    function mkSetValueCallback(aResponseData, aProperty, aField, aValue, aStatus)
    {
        return function()
        {
            trace("mkSetValueCallback - Value: " +aValue);

            fixFieldValueOnDevice(aResponseData, aProperty, aField, aValue, aStatus);
        }
    }

    function mkStartSreenCallback(aScreenUrl, aScreenId, aStatus)
    {
        return function()
        {
            trace("mkStartSreenCallback - Screen: " +aScreenId);

            startScreen(aScreenUrl, aScreenId, aStatus);
        }
    }

    function setTestResult(aStatusElement, aTestItem, aResponseData, aResult)
    {
        trace(">> setTestResult");

        if (aTestItem.isPass != null && aTestItem.isPass(aResult))
        {
            var passText = null;

            if (aTestItem.passText != null)
            {
                passText = aTestItem.passText(aResult);
            }

            trace("setTestResult - Passed");
            setTestStatusOk(aStatusElement, passText);
            return;
        }

        if (aTestItem.isInfo)
        {
            trace("setTestResult - Info");
            setTestStatusInfo(aStatusElement, aTestItem.infoText);
        }
        else if (aTestItem.isCritical(aResult))
        {
            trace("setTestResult - Critical");
            setTestStatusCriticalWarning(aStatusElement, aTestItem.criticalText(aResult));
        }
        else
        {
            trace("setTestResult - Warning");
            setTestStatusWarning(aStatusElement, aTestItem.warningText(aResult));
        }

        for (var index in aTestItem.fixes)
        {
            var thisFix = aTestItem.fixes[index];

            if (thisFix.value != null)
            {
                trace("setTestResult, Adding - Set Value Callback: " +thisFix.value);

                var callback = mkSetValueCallback(aResponseData, aTestItem.property, aTestItem.field, thisFix.value, aStatusElement);

                addFixToTestStatus(thisFix.title, "", aStatusElement, callback);
                continue;
            }

            if (thisFix.screen != null)
            {
                trace("setTestResult, Adding - Launch Screen Callback: " +thisFix.screen);

                var callback = mkStartSreenCallback("root.intents.screens", thisFix.screen, aStatusElement);

                addFixToTestStatus(thisFix.title, "", aStatusElement, callback);
                continue;
            }

            // If there's no test action, then assume there's something embedded in the fix, like an
            // HTML link that runs some Javascript directly
            addFixToTestStatus(thisFix.title, "", aStatusElement, null);
        }
    }

  
    ///////////////////////////////////////////////////////////////////////////////////////

    function addTestGroup(aTestGroup, aProperties)
    {
        trace(">> addTestGroup, Group: " +aTestGroup.testGroup);

        var testGroup = addNewTestGroup($scope.quickCheckGroups, aTestGroup.testGroup);

        var testCases = aTestGroup.testCases;

        for (var caseIndex in testCases)
        {
            var thisCase = testCases[caseIndex];

            trace("addTestGroup, Test Case: " +thisCase.testCase); 

            thisCase.resultElement = addNewTestStatusToGroup(testGroup, thisCase.testCase);

            var testItems = thisCase.testItems;

            if (testItems != null)
            {
                for (var itemIndex in testItems)
                {
                    var thisItem = testItems[itemIndex];

                    var testProperty = thisItem.property;

                    if (testProperty != null)
                    {
                        trace("addTestGroup, Case: " +thisCase.testCase+ ", Property: " +testProperty)
                        aProperties[testProperty] = true;
                    }
                }
            }

        }

        trace("<< addTestGroup");        
    }



    function fetchProperty(aProperty)
    {
        getDataFromDevice(aProperty, function (aRequestId, aSuccess, aResponsData)
        {
            var data = null;

            if (aResponsData != null) data = JSON.stringify(aResponsData);

            trace("fetchProperty, Got Data For Property: " +aProperty+ ", Success: " +aSuccess+ ", Data: " +data);

            mResponseData[aProperty] = aResponsData;

            setTimeout(fetchNextProperty);
        });
    }


    function fetchNextProperty()
    {
        if (mPropertyFetchCount >= mPropertiesList.length)
        {
            trace("fetchNextProperty, Got all properties");
            checkTestResults();
            return;
        }

        var property = mPropertiesList[mPropertyFetchCount];
        mPropertyFetchCount++;

        trace("fetchNextProperty, Fetching: " +property);

        fetchProperty(property);
    }


    function checkGroupResults(aTestGroup)
    {
        trace(">> checkGroupResults, Group: " +aTestGroup.testGroup);

        var testCases = aTestGroup.testCases;

        for (var caseIndex in testCases)
        {
            var thisCase = testCases[caseIndex];

            var resultElement = thisCase.resultElement;

            if (resultElement == null)
            {
                error("checkGroupResults, No Result Element for Case: " +thisCase.testCase); 

            }

            trace("checkGroupResults, Case: " +thisCase.testCase);

            var testItems = thisCase.testItems;

            if (testItems != null)
            {
                var numTestItems   = testItems.length;
                var finalItemIndex = numTestItems - 1;

                trace("checkGroupResults, Case: " +thisCase.testCase+ ", Test Items: " +numTestItems);
                for (var itemIndex in testItems)
                {
                    var thisItem     = testItems[itemIndex];

                    var isInfo       = thisItem.isInfo;

                    if (isInfo)
                    {
                        $scope.$applyAsync(setTestResult(resultElement, thisItem, null, null));
                        continue;
                    }

                    var testProperty = thisItem.property;
                    var testField    = thisItem.field;

                    if (testField == null)
                    {
                        continue;
                    }

                    var responseData = mResponseData[testProperty];

                    var value = MMRSUTILS.getValueFromResponseData(responseData, testField);

                    if (value == null)
                    {
                        error("checkGroupResults, No Result For Case: " +thisCase.testCase+ ", Property: " +testProperty+ ", Field: " +testField);
                        continue;
                    }

                    trace("checkGroupResults, Case: " +thisCase.testCase+ ", Property: " +testProperty+ ", Field: " +testField+ ", Value: " +value);
                    var testPassed = thisItem.isPass != null && thisItem.isPass(value);

                    if (testPassed && itemIndex < finalItemIndex)
                    {
                        trace("checkGroupResults, Test Item Passed, Moving to Next. Item: " +itemIndex+ ", Final: " +finalItemIndex);                        
                        continue;
                    }

                    $scope.$applyAsync(setTestResult(resultElement, thisItem, responseData, value));

                    break; // Break - it is either the last item or the test has failed.
                }
            }
        }

        trace("<< checkGroupResults"); 
    }

    function checkTestResults()
    {
        for (var index in mTestGroups)
        {
            var thisGroup = mTestGroups[index];

            trace("checkTestResults, Group: " +thisGroup.testGroup);

            checkGroupResults(thisGroup);
        }
    }


    var mResponseData       = null;
    var mPropertiesList     = null;
    var mPropertyFetchCount = 0;

    var mTestGroups = [];

    function createTestGroups(aGroups)
    {
        var propertiesById = {};

        if (aGroups.doCellular)
        {
            var cellular = MMRSAUTOCHECKCELLULAR.getTestGroup();

            mTestGroups.push(cellular);
            addTestGroup(cellular, propertiesById);
        }

        if (aGroups.doPower)
        {
            var power = MMRSAUTOCHECKPOWER.getTestGroup();

            addTestGroup(power, propertiesById);
            mTestGroups.push(power);            
        }

        if (aGroups.doAudio)
        {
            var audio = MMRSAUTOCHECKAUDIO.getTestGroup();

            addTestGroup(audio, propertiesById);
            mTestGroups.push(audio);

        }

        mPropertiesList = [];
        mResponseData   = {};

        for (var thisProperty in propertiesById)
        {
            trace("createTestGroups, Property: " +thisProperty);

            mPropertiesList.push(thisProperty)
        }

        fetchNextProperty();
    }

    ///////////////////////////////////////////////////////////////////////////////////////

    function getTestGroups(aOptions)
    {
        var groups = {};

        var checks = aOptions.autoCheck;

        trace("getTestGroups, Checks: " +checks); 

        if (checks == "cellular" || checks == "all")
        {
            groups.doCellular = true;
        }

        if (checks == "power" || checks == "all")
        {
            groups.doPower = true;
        }

        if (checks == "audio" || checks == "all")
        {
            groups.doAudio = true;
        }

        return groups;
    }

 	$scope.doLoad = function(aOptions)
    {   
	    trace(">> doLoad");

        var testGroups = getTestGroups(aOptions);

		$scope.$applyAsync(function()
		{
            createTestGroups(testGroups);
		});

	    trace("<< doLoad");
	}
}]);
//}]);


function onLoad()
{
    trace(">> onLoad");

    mAngularScope = MMRSUTILS.getAngularScope("angularArea");

    if (mAngularScope == null)
    {
        error("<< onLoad, Could not find angular scope area");
        return;
    }

    var query = null;

    if (mIsNativeRun)
    {
        var query = DEVICEBRIDGE.getPageQuery();
    }

    trace("onLoad, Query: " +query);

    var options = MMRSUTILS.getOptions(query);

    mAngularScope.doLoad(options);

    trace("<< onLoad");
}

var exports = {};

exports.name = "mmrsautocheck";

exports.onLoad = function(aOptions) { onLoad(aOptions); }

// This is exported so that specific test groups can launch screens from embedded links
exports.startScreen = startScreen;

return exports;

}());
