var MMRSAUTOCHECKAUDIO = (function() {
	"use strict"

    function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
    function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
    function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
    function error(aText) { MMRSUTILS.doError(exports.name, aText); }

    if (!String.format)
    {
        String.format = function(format) {
        var args = Array.prototype.slice.call(arguments, 1);
        return format.replace(/{(\d+)}/g, function(match, number) { 
          return typeof args[number] != 'undefined'
            ? args[number] 
            : match
          ;
        });
      };
    }


    //=============================================================================
    //  Sound and Volume
    //=============================================================================

    //-------------------------------------------------------------------------
    // Call Volume
    //-------------------------------------------------------------------------    
    //            
    var mCallVolumeMuteErrorText =
        "Call volume is set to mute (0%). Calls will be silent.";

    var mCallVolumeLowErrorText =
        "Call volume is set very low ({0}%). Calls may be difficult to hear at this volume.";  

    var mCallVolumeTestItem =
    {
        property: "root.sound.voice",
        field:    "voice",
        fixes:   
        [
            { title: "Change call volume to 100%", value: "100" },
            { title: "Change call volume to 75%",  value: "75"  },
            { title: "Change call volume to 50%",  value: "50"  }                
        ],

        warningText:  function(aResult) { return String.format(mCallVolumeLowErrorText,  aResult); },
        criticalText: function(aResult) { return String.format(mCallVolumeMuteErrorText, aResult); },

        isPass:     function(aResult) { return aResult >= 25; },
        isWarning:  function(aResult) { return aResult > 0 && aResult < 25; },
        isCritical: function(aResult) { return aResult == 0; }              
    };

    var mCallVolumeTestCase =
    {
        testCase  : "Call Volume",
        testItems : [mCallVolumeTestItem]
    };    

    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Ring Volume
    //-------------------------------------------------------------------------    
    //
    var mRingVolumeMuteErrorText =
        "Ring volume is set to mute (0%). Incoming calls will not ring.";

    var mRingVolumeLowErrorText =
        "Ring volume is set very low ({0}%). The ring for incoming calls may be difficult to hear at this volume.";    


    var mRingVolumeTestItem =
    {
        property: "root.sound.ring",
        field:    "ring",
        fixes:   
        [
            { title: "Change ring volume to 100%", value: "100" },
            { title: "Change ring volume to 75%",  value: "75"  },
            { title: "Change ring volume to 50%",  value: "50"  }                
        ],

        warningText:  function(aResult) { return String.format(mRingVolumeLowErrorText,  aResult); },
        criticalText: function(aResult) { return String.format(mRingVolumeMuteErrorText, aResult); },

        isPass:     function(aResult) { return aResult >= 25; },
        isWarning:  function(aResult) { return aResult > 0 && aResult < 25; },
        isCritical: function(aResult) { return aResult == 0; }               
    };   

    var mRingVolumeTestCase =
    {
        testCase  : "Ring Volume",
        testItems : [mRingVolumeTestItem]
    }; 
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Music Volume
    //-------------------------------------------------------------------------    
    //    
    var mMusicVolumeMuteErrorText =
        "Music and media volume is set to mute (0%). Music and other media audio will not play.";

    var mMusicVolumeLowErrorText =
        "Music and media volume is set very low ({0}%). Music and other media may be difficult to hear at this volume.";

    var mMusicVolumeTestItem =
    {
        warning:  "Music volme is too low.",
        property: "root.sound.music",
        field:    "music",
        fixes:   
        [
            { title: "Change music volume to 100%", value: "100" },
            { title: "Change music volume to 75%",  value: "75"  },
            { title: "Change music volume to 50%",  value: "50"  }                
        ],

        warningText:  function(aResult) { return String.format(mMusicVolumeLowErrorText,  aResult); },
        criticalText: function(aResult) { return String.format(mMusicVolumeMuteErrorText, aResult); },

        isPass:     function(aResult) { return aResult >= 25; },
        isWarning:  function(aResult) { return aResult > 0 && aResult < 25; },
        isCritical: function(aResult) { return aResult == 0; }            
    };   

    var mMusicVolumeTestCase =
    {
        testCase  : "Music and Media Volume",
        testItems : [mMusicVolumeTestItem]
    }; 


    //-------------------------------------------------------------------------
    // Test Group
    //-------------------------------------------------------------------------    
    //    
    var mTestGroup =
    {
        testGroup: "Audio and Sound",
        testCases: 
        [
            mCallVolumeTestCase,
            mRingVolumeTestCase,
            mMusicVolumeTestCase
        ]
    };

var exports = {};

exports.name = "mmrsautocheckaudio";

exports.getTestGroup = function() { return mTestGroup; }

return exports;

}());
