var MMRSAUTOCHECKCELLULAR = (function() {
	"use strict"

    var SIGNAL_STRENGTH_POOR_THRESHOLD = 20;

    function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
    function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
    function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
    function error(aText) { MMRSUTILS.doError(exports.name, aText); }

    if (!String.format)
    {
        String.format = function(format) {
        var args = Array.prototype.slice.call(arguments, 1);
        return format.replace(/{(\d+)}/g, function(match, number) { 
          return typeof args[number] != 'undefined'
            ? args[number] 
            : match
          ;
        });
      };
    }

    //=============================================================================
    // MOBILE NETWORKS
    //=============================================================================
    //

    //-------------------------------------------------------------------------
    // SIM
    //-------------------------------------------------------------------------
    //
    var mSimNotFoundErrorText = 
       "No SIM card was detected.<BR>" + 
       "Without a SIM card this device will not be able to make or receive calls or use mobile data.<BR>" +
       "<P>If a SIM card is inserted then try the following:</P>" +
       "<UL class='plain'>" +
       "<LI>Remove the SIM card then reinsert it</LI>"  + 
       "<LI>Turn the device off and on again</LI>"      +
       "<LI>Use another SIM card</LI>"                  + 
       "</UL>";

    var mSimNetworkPinErrorText =
       "This phone is locked by the network operator and cannot be used with this SIM. " +
       "It may be possible to unlock this device with a code from the network operator.";

    var mSimUserPinErrorText = 
       "This SIM is locked. You must enter your PIN before you can make calls, receive calls or use mobile data.";

    var mSimPUKErrorText = 
        "This SIM is locked. You must enter your PUK before you can make calls, receive calls or use mobile data. " +
       "Contact your network operator to get your PUK code.";    


    var mNetworkSimTestItem =
    {
        property: "root.mobile_network.sim",
        field:    "sim_status",
        fixes:   [],

        passes:
        {
            "OK" : true
        },

        criticals:
        {
            "No SIM Detected" : mSimNotFoundErrorText,
            "Unknown"         : mSimNotFoundErrorText
        },

        warnings:
        {
            "Network PIN Required" : mSimNetworkPinErrorText,
            "User PIN Required"    : mSimUserPinErrorText,
            "PUK Required"         : mSimPUKErrorText
        },

        isPass:     function(aResult) { return this.passes[aResult]    == true; },
        isWarning:  function(aResult) { return (!this.isPass(aResult) && !this.isCritical(aResult)); },
        isCritical: function(aResult) { return this.criticals[aResult] != null; },

        warningText:  function(aResult) 
        {
            var text = this.warnings[aResult];

            if (text == null) text = "SIM status is: " +aResult;  

            return text;
        },

        criticalText: function(aResult) { return this.criticals[aResult]; }
    };    


    var mNetworkSimTestCase =
    {
        testCase  : "SIM",
        testItems : [mNetworkSimTestItem]
    };
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Flight Mode
    //-------------------------------------------------------------------------    
    //
    var mFlightModeErrorText =
        "Flight mode is switched on. " +
        "This will prevent you from using the phone for calls, text messages and mobile data.";


    var mFlightModeTestItem =
    {
        property: "root.mobile_network.current_mobile_network",
        field:    "flight_mode",
        fixes:   
        [
            { title: "Go to the flight mode setting screen", screen: "android/settings/AIRPLANE_MODE_SETTINGS" }
        ],

        isPass:     function(aResult) { return aResult.toLowerCase() == "no"; },
        isWarning:  function(aResult) { return false; },
        isCritical: function(aResult) { return aResult.toLowerCase() != "no"; },

        warningText:  function(aResult) { return null; },
        criticalText: function(aResult) { return mFlightModeErrorText; }        
    };   

    var mFlightModeTestCase =
    {
        testCase  : "Flight Mode Setting",
        testItems : [mFlightModeTestItem]
    }; 
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Signal Strength
    //-------------------------------------------------------------------------    
    // 
    var mSignalStrengthErrorText =
        "Signal strength is poor at this location. ";
        //"Consider using Wi-Fi for data, and Wi-Fi calling for voice calls.";

    var mSignalStrenghtSIMTestItem =
    {
        property: "root.mobile_network.sim",
        field:    "sim_status",
        fixes:    [],

        isPass:     function(aResult) { return aResult.toLowerCase() == "ok"; },
        isWarning:  function(aResult) { return false; },
        isCritical: function(aResult) { return aResult.toLowerCase() != "ok";  },

        warningText:  function(aResult) { return null; },
        criticalText: function(aResult) { return "No SIM detected"; }
    };    

    var mSignalStrenghtFlightModeTestItem =
    {
        property: "root.mobile_network.current_mobile_network",
        field:    "flight_mode",
        fixes:    [],

        isPass:     function(aResult) { return aResult.toLowerCase() == "no"; },
        isWarning:  function(aResult) { return false; },
        isCritical: function(aResult) { return aResult.toLowerCase() != "no"; },

        warningText:  function(aResult) { return null; },
        criticalText: function(aResult) { return "Device is in flight mode"; }        
    };     

    var mSignalStrenghtTestItem =
    {
        property: "root.mobile_network.current_mobile_network",
        field:    "signal_strength",
        fixes:    [],

        isPass:     function(aResult) { return aResult >= SIGNAL_STRENGTH_POOR_THRESHOLD; },
        isWarning:  function(aResult) { return aResult <  SIGNAL_STRENGTH_POOR_THRESHOLD; },
        isCritical: function(aResult) { return false; },

        warningText:  function(aResult) { return mSignalStrengthErrorText; },
        criticalText: function(aResult) { return null; }
    };   


    var mSignalStrengthTestCase =
    {
        testCase  : "Mobile Signal Strength",
        testItems : [mSignalStrenghtSIMTestItem, mSignalStrenghtFlightModeTestItem, mSignalStrenghtTestItem]
    }; 
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Mobile Data
    //-------------------------------------------------------------------------    
    // 
    var mMobileDataErrorText =
        "Mobile data is switched off. This will prevent you from browsing the Internet, " +
        "checking email and using apps on your mobile network.";


    var mMobileDataTestItem =
    {
        property: "root.mobile_network.mobile_data",
        field:    "data_enabled",
        fixes:   
        [
            {
                title:  "Go to the settings screen where you can enable mobile data",
                screen: "com/android/settings"
            }
        ],

        isPass:     function(aResult) { return MMRSUTILS.contains(aResult, "enabled"); },
        isWarning:  function(aResult) { return MMRSUTILS.contains(aResult, "disabled"); },
        isCritical: function(aResult) { return false; },

        warningText:  function(aResult) { return mMobileDataErrorText; },
        criticalText: function(aResult) { return null; }        
    };  

    var mMobileDataTestCase =
    {
        testCase  : "Mobile Data Settings",
        testItems : [mMobileDataTestItem]
    };     
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Roaming Data
    //-------------------------------------------------------------------------    
    // 
    var mRoamingDataHelpText = 
        "Go to the settings screen where you can enable data roaming. " +
        "Note that if you do this, data roaming charges may apply.";

    var mRoamingDataErrorText =
        "This device is roaming, but the data roaming setting is switched off. " +
        "This will prevent you from browsing the Internet, checking email and using apps on your mobile network.";


    var mRoamingDataTestItem =
    {
        property: "root.mobile_network.mobile_data",
        field:    "data_roaming_enabled",
        fixes:   
        [
            {
                title:  mRoamingDataHelpText,
                screen: "android/settings/DATA_ROAMING_SETTINGS"
            }
        ],

        isPass:     function(aResult) { return  MMRSUTILS.contains(aResult, "enabled"); },
        isWarning:  function(aResult) { return !MMRSUTILS.contains(aResult, "enabled"); },
        isCritical: function(aResult) { return false; },

        warningText:  function(aResult) { return mRoamingDataErrorText; },
        criticalText: function(aResult) { return null; }        
    };  

    var mRoamingDataTestCase =
    {
        testCase  : "Romaing Data Settings",
        testItems : [mRoamingDataTestItem]
    };    
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Network Service Availability
    //-------------------------------------------------------------------------    
    // 

    // TODO: create test case contains 2 test items

    var mNoNetworkManualModeErrorText = 
        "<p>No mobile service is available.</p>" +
        "<p>This device is set to manual network selection (as opposed to automatic network selection)." +
        "Check that the correct network is selected for this phone and SIM.</p>" +
        "<p>Alternatively, the device may be out of signal coverage for your network, " +
        "or there may be an issue with your account or SIM.</p>";

    var mNoNetworkCoverageErrorText =
        "No mobile service is available. " +
        "The device may be out of signal coverage for your network, or there may be an issue with your account or SIM.";


    var mNoNetworkAvailableTestItem =
    {
        property: "root.mobile_network.current_mobile_network",
        field:    "selection_mode",
        fixes:   
        [
            // We're launching the data roaming settings because this usually has the network selection settings
            // We're not using ACTION_NETWORK_OPERATOR_SETTINGS because it just hangs searching for devices on at least some devices
            // See: https://code.google.com/p/android/issues/detail?id=205030
            {
                title:  "Go to the settings screen where you can check that the correct mobile network is selected.",
                screen: "android/settings/DATA_ROAMING_SETTINGS"
            }
        ],

        isPass:     function(aResult) { return false; },
        isCritical: function(aResult) { return false; },

        // We are only used when there is no network available. Therefore, there always a warning.
        // If we are in manual network selection mode, we want to keep the option of going to that settings screen
        // Otherwise, we remove that option as there is nothing we can do about no network availability in automatic mode
        // 
        isWarning:  function(aResult)
        { 
            var isManual = MMRSUTILS.contains(aResult, "manual");

            if (!isManual) this.fixes = [];
            return true;
        },

        criticalText: function(aResult) { return null; },
        warningText:  function(aResult)
        {
            if (MMRSUTILS.contains(aResult, "manual")) 
            {
                return mNoNetworkManualModeErrorText;
            }
            else
            {
                return mNoNetworkCoverageErrorText;
            }
        },
    };  


    // 
    //-------------------------------------------------------------------------

    //-------------------------------------------------------------------------
    // 


    ///********** FOLLOWING CODE IS UNUSED AND SHOULD BE REMOVED ONCE ALL TESTS HAVE BEEN IMPLEMENTED *************// 

    //
    function addMobileNetworkTestGroup()
    {
        var testGroup = addNewTestGroup($scope.quickCheckGroups, "Mobile Network and SIM");
    
        var simStatusTestStatus            = addNewTestStatusToGroup(testGroup, "SIM");
        var flightModeStatusTestStatus     = addNewTestStatusToGroup(testGroup, "Flight Mode Setting");
        var mobileNetworkServiceAvailabilityTestStatus = addNewTestStatusToGroup(testGroup, "Mobile Service Availability");
        var mobileSignalStrengthTestStatus = addNewTestStatusToGroup(testGroup, "Mobile Signal Strength");
        var mobileDataEnbaledTestStatus    = addNewTestStatusToGroup(testGroup, "Mobile Data Settings");
        var mobileDataRoamingTestStatus    = addNewTestStatusToGroup(testGroup, "Mobile Data Roaming");
        // var mobileDataNetworkTypeTestStatus = addNewTestStatusToGroup(testGroup, "Mobile Data Network Type"); // TODO: Add this later?
        // var mobileDataStatusTestStatus = addNewTestStatusToGroup(testGroup, "Mobile Data Status"); // TODO: Add this later
   
        getDataFromDevice(mNetworkSimFix.property, function (aRequestId, aSuccess, aSimResponseData)
        {
            var value = MMRSUTILS.getValueFromResponseData(aSimResponseData, mNetworkSimFix.field);

            trace("addMobileNetworkTestGroup, SIM Status: " + value);

            setTestResult(simStatusTestStatus, mNetworkSimFix, aSimResponseData, value);
        });
            
        // TODO: [later] check that MCC / MNC values are in the supported list
        
        // Get mobile network info
        getDataFromDevice(mFlightModeFix.property, function (aRequestId, aSuccess, aCurrentMobileNetworkResponseData)
        {
            var flightModeValue,
                serviceAvailabilityValue,
                signalStrengthStringValue,
                signalStrengthNumberValue,
                selectionModeValue;
            
            var value = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, mFlightModeFix.field);

            trace("Flight Mode: " + value);

            setTestResult(flightModeStatusTestStatus, mFlightModeFix, aCurrentMobileNetworkResponseData, value);
        
            var signalStrengthStringValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, mSignalStrenghtFix.field);
            var value = parseInt(signalStrengthStringValue, 10);

            setTestResult(mobileSignalStrengthTestStatus, mSignalStrenghtFix, aCurrentMobileNetworkResponseData, value);


            var serviceAvailabilityValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, "service_state");

            if (serviceAvailabilityValue.toLowerCase().includes("available"))
            {
                setTestStatusOk(mobileNetworkServiceAvailabilityTestStatus);
            }
            else
            {
                var mode = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, mNoNetworkAvailableFix.field);

                setTestResult(mobileNetworkServiceAvailabilityTestStatus, mNoNetworkAvailableFix, aCurrentMobileNetworkResponseData, mode);
            }

        
            // Get mobile data info
            getDataFromDevice(mMobileDataFix.property, function (aRequestId, aSuccess, aMobileDataResponseData)
            {
                var value = MMRSUTILS.getValueFromResponseData(aMobileDataResponseData, mMobileDataFix.field);
                
                trace("mobileDataEnabledValue: " + value);
                
                setTestResult(mobileDataEnbaledTestStatus, mMobileDataFix, aMobileDataResponseData, value);

                var currentlyRoamingValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, "currently_roaming");
                
                if (! currentlyRoamingValue.toLowerCase().includes("roaming"))
                {
                    setTestStatusOk(mobileDataRoamingTestStatus);                            
                }
                else
                {
                    var roamingData = MMRSUTILS.getValueFromResponseData(aMobileDataResponseData, mRoamingDataFix.field);
                    setTestResult(mobileDataRoamingTestStatus, mRoamingDataFix, aMobileDataResponseData, roamingData);                            
                }
            });
        });            
    }

    //-------------------------------------------------------------------------
    // Test Group
    //-------------------------------------------------------------------------    
    //   
    var mTestGroup =
    {
        testGroup: "Calls and Internet",
        testCases: 
        [
            mNetworkSimTestCase,
            mFlightModeTestCase,
            //mSignalStrengthTestCase,
            mMobileDataTestCase  
            // TODO
            // Roaming Data
            // Service Availability
        ]
    };

var exports = {};

exports.name = "mmrsautocheckcellular";

exports.getTestGroup = function() { return mTestGroup; }

return exports;

}());
