var MMRSAUTOCHECKPOWER = (function() {
	"use strict"

    function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
    function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
    function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
    function error(aText) { MMRSUTILS.doError(exports.name, aText); }

    if (!String.format)
    {
        String.format = function(format) {
        var args = Array.prototype.slice.call(arguments, 1);
        return format.replace(/{(\d+)}/g, function(match, number) { 
          return typeof args[number] != 'undefined'
            ? args[number] 
            : match
          ;
        });
      };
    }

    //=============================================================================
    //  Battery and Power Test Cases
    //=============================================================================
    //   

    //-------------------------------------------------------------------------
    // Screen Sleep Timeout
    //-------------------------------------------------------------------------    
    //  
    var mSleepErrorText =
        "The screen timeout is set to {0} seconds. Long screen timeouts may cause excessive battery drain."

    var SCREEN_TIMEOUT_FIX_TEXT = "You can change the screen timeout by going to the "
                                    + "<a href=\"javascript:MMRSAUTOCHECK.startScreen('root.intents.screens', 'android/settings/DISPLAY_SETTINGS')\">Display settings</a>"
                                    + " then:"
                                    + "<br>"
                                    + "<br>"
                                    + "- Tap on <b>Sleep</b> or <b>Screen timeout</b>"
                                    + "<br>"
                                    + "<br>"
                                    + "- Select a shorter timeout option";


    var mScreenSleepTestItem =
    {
        property: "root.screen.sleep",
        field:    "screenofftime",
        fixes:   
        [
            { title: SCREEN_TIMEOUT_FIX_TEXT}
        ],

        warningText:  function(aResult) { return String.format(mSleepErrorText, aResult); },
        criticalText: function(aResult) { return String.format(mSleepErrorText, aResult); },

        isPass:     function(aResult) { return aResult <= 90; },
        isWarning:  function(aResult) { return aResult > 90 && aResult <= 120 },
        isCritical: function(aResult) { return aResult > 120; }
    };

    var mScreenTimeoutTestCase =
    {
        testCase  : "Screen Timeout",
        testItems : [mScreenSleepTestItem]
    };
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Brightness Level
    //-------------------------------------------------------------------------    
    //  
   var mBrightnessLevelErrorText =
        "Screen brightness is set to {0}%. This may cause excessive battery drain.<br><br>Reduce your screen brightness to 75%<br><br>You can reduce the brightness in the Quick Settings by a <b>long swipe down from the top of the screen</b> or in <b>Settings > Display > Brightness.</b>";

    var mBrightnessLevelModeErrorText = 
        "Screen brightness is set to \"automatic\".";

    var mBrightnessLevelModeTestItem =
    {
        property: "root.screen.brightness",
        field:    "brightnessmode",
        fixes:   [],

        passText: function(aResult)
        {
            return aResult == "1" ? "Screen brightness is set automatically" : null;
        },

        warningText:  function(aResult) { return null; },
        criticalText: function(aResult) { return null; },

        isPass:     function(aResult) { return true; },
        isWarning:  function(aResult) { return false; },
        isCritical: function(aResult) { return false; }       
    };

    var mBrightnessLevelTestItem =
    {
        property: "root.screen.brightness",
        field:    "brightnesslevel",
        fixes:   [],
        
        warningText:  function(aResult) { return String.format(mBrightnessLevelErrorText, aResult); },
        criticalText: function(aResult) { return String.format(mBrightnessLevelErrorText, aResult); },

        isPass:     function(aResult) { return aResult <= 75; },
        isWarning:  function(aResult) { return aResult >  75 && aResult <= 90; },
        isCritical: function(aResult) { return aResult >  90; }        
    };    

    var mBrightnessLevelTestCase =
    {
        testCase: "Screen Brightness Level",
        testItems: [mBrightnessLevelTestItem, mBrightnessLevelModeTestItem]
    };

    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Brightness Mode
    //-------------------------------------------------------------------------    
    //  
    var mBrightnessModeErrorText = 
        "Screen brightness is set to \"manual\". " +
        "This may cause excessive battery drain and make the screen harder to see in some conditions."
        +"<br><br>You can you can enable automatic brightness in <b>Settings > Display > Brightness.</b>";

    var mBrightnessModeTestItem =
    {
        property: "root.screen.brightness",
        field:    "brightnessmode",
        fixes:   
        [],

        warningText:  function(aResult) { return mBrightnessModeErrorText; },
        criticalText: function(aResult) { return null; },

        isPass:     function(aResult) { return aResult == "1"; },
        isWarning:  function(aResult) { return aResult == "0"; },
        isCritical: function(aResult) { return false; }       
    };       

    var mBrightnessModeTestCase  =
    {
        testCase:  "Automatic Screen Brightness",
        testItems: [mBrightnessModeTestItem]
    };

    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Battery Health
    //-------------------------------------------------------------------------    
    //  
    var mBatteryErrorText = 
        "Battery health warning: {0}";        

    var mBatteryHealthTestItem =
    {
        property: "root.device.battery",
        field:    "health",
        fixes:   [],

        warningText:  function(aResult) { return String.format(mBatteryErrorText, aResult); },
        criticalText: function(aResult) { return null; },

        isPass:     function(aResult) { return  MMRSUTILS.contains(aResult, "good"); },
        isWarning:  function(aResult) { return !MMRSUTILS.contains(aResult, "good"); },
        isCritical: function(aResult) { return false; }       
    };   

    var mBatteryHealthTestCase = 
    {
        testCase:  "Battery Health",
        testItems: [mBatteryHealthTestItem]
    };
    // 
    //-------------------------------------------------------------------------


    //-------------------------------------------------------------------------
    // Battery Usage Info
    //-------------------------------------------------------------------------    
    //  
    var mBatteryUsageInfoTestItem =
    {
        property: null,
        field:    null,
        fixes:  
        [
            {
                title:  "The Android settings app can show what is using the most battery power.",
                screen: "android/intent/action/POWER_USAGE_SUMMARY"
            }
        ],

        infoText: "Launch the Android battery settings screen",
        isInfo:   true
    }; 

    var mBatteryUsageInfoTestCase = 
    {
        testCase:  "Battery Usage Troubleshooting Tip",
        testItems: [mBatteryUsageInfoTestItem]
    };

    // 
    //-------------------------------------------------------------------------


   ///********** FOLLOWING CODE IS UNUSED AND SHOULD BE REMOVED ONCE ALL TESTS HAVE BEEN IMPLEMENTED *************// 


    function addBatteryTestGroup()
    {
    	trace(">> addBatteryTestGroup");

        var testGroup = addNewTestGroup($scope.quickCheckGroups, "Battery and Power");

        // TODO: make URL, field name and "30" constant
        //

        var batteryUsageInfo = addNewInfoItemToGroup  (testGroup, "Battery Usage Troubleshooting Tip", "The Android settings app can show what is using the most battery power.");


        addFixToTestStatus("Launch the Android battery settings screen", "", batteryUsageInfo, function ()
        {
            trace("Launch the Android battery usage screen");

            startScreen("root.intents.screens", "android/intent/action/POWER_USAGE_SUMMARY", batteryUsageInfo);
        });
    }


    //-------------------------------------------------------------------------
    // Test Group
    //-------------------------------------------------------------------------    
    //  
    var mTestGroup =
    {
        testGroup: "Battery and Power",
        testCases: 
        [
            mScreenTimeoutTestCase,
            mBrightnessLevelTestCase,
            mBrightnessModeTestCase,
            mBatteryHealthTestCase,
            mBatteryUsageInfoTestCase    
        ]
    };

var exports = {};

exports.name = "mmrsautocheckpower";

exports.getTestGroup = function() { return mTestGroup; }

return exports;

}());
