/*jslint browser: true*/
/*global $, jQuery, alert, angular, MMRSUTILS, MMRSCLIENTGATEWAY, MMRSCONFIG, MMRSCHARTUTILS*/
var MMRSCHANGESLOG = (function () {

    function trace(aText) { MMRSUTILS.doTrace("mmrsactionslog", aText);  }
    function note(aText)  { MMRSUTILS.doTrace("mmrsactionslog", aText);  }
    function warn(aText)  { MMRSUTILS.doWarn("mmrsactionslog", aText);   }
    function error(aText) { MMRSUTILS.doError("mmrsactionslog", aText);  }
    
    angular.module("mmrsChangesLogApp", ['ngSanitize']).controller("ChangesLogController", [function($scope)
    {
        var self = this;    
        
        self.headings = ["Type", "Time", "Who", "Area", "Setting", "Change"];

        self.dateTime = function(aTime)
        {
            trace("dateTime, Time: " +aTime);
            
            var date = new Date(aTime*1000);

            var day   = date.getDate();
            var month = date.getMonth() + 1;
            var year  = date.getFullYear();

            month = "0" + month;
            year  = ""  + year;

            var dateStr = day+ "/" +month.substr(-2)+ "/" +year.substr(-2);

            var hours   = "0" +date.getHours();
            var minutes = "0" +date.getMinutes();
            var seconds = "0" +date.getSeconds();

            var timeStr = hours.substr(-2) + ':' + minutes.substr(-2) + ':' + seconds.substr(-2);

            var dateTimeStr = timeStr+ " " +dateStr;
            
            return dateTimeStr;
        }

        self.getActorName = function(aActor)
        {
            if (aActor == "REMOTE") return "Call Center";
            if (aActor == "GENIUS") return "In Store";
            if (aActor == "OWNER")  return "Owner";
            if (aActor == "LOCAL")  return "Owner";
            return aActor;
        }
	
	self.getLevelName = function(aLevel)
        {
            if (aLevel == null || aLevel == "") return "";

            var level = aLevel.charAt(0).toUpperCase() + aLevel.slice(1);

            return level;
        }

        self.getLevelStyle = function(aLevel)
        {
            if (aLevel == "warning") return "{color: orange}";
            if (aLevel == "login")   return "{color: blue}";

            return "{color: black}";
        }

        self.onRefreshChanges = function()
        {
            trace("actionslogcontrol, onRefreshChanges");    

            $('#changesWait').modal('show');            

            getChangesLog();
        }

        function getChangesLog()
        {
            trace("actionslogcontrol, getChangesLog");            

            var batcher = MMRSCLIENTGATEWAY.makeBatcher();
       
            batcher.addGetRequest("root.logger.changes", onLogResponse);
            
            batcher.dispatchRequests();
        }


        doGetChangesLog2 = function()
        {
            trace("actionslogcontrol, getChangesLog");            

            var response = [{"nodetype":"leaf","nodetitle":"Actions","nodeid":"actions","nodeidpath":"root.logger.actions","mode":"rw","fields":[{"fieldtype":"text","fieldid":"logbytime","fieldtitle":"Actions By Time","fieldvalue":"[{\"entryType\":\"update\",\"entryTime\":1477473330,\"actor\":\"GENIUS\",\"node\":\"Sleep\",\"field\":\"Screen Off Time (seconds)\",\"newValue\":\"60\"}]"}]}];

            var response1 = [
                                {
                                    "nodetype":"leaf",
                                    "nodetitle":"Actions",
                                    "nodeid":"actions",
                                    "nodeidpath":"root.logger.actions",
                                    "mode":"rw",
                                    "fields":
                                        [
                                            {
                                                "fieldtype":"text",
                                                "fieldid":"logbytime",
                                                "fieldtitle":"Actions By Time",
                                                "fieldvalue": "[{\"entryType\":\"update\", \"entryTime\":1477473330, \"actor\":\"GENIUS\", \"node\":\"Sleep\", \"field\":\"Screen Off Time (seconds)\", \"newValue\":\"60\"}]"
                                            }
                                        ]
                                }
                            ];

            //var response = JSON.parse(responseStr);

            doLogResponse(response);
        }

        doLogResponse = function(aResponse)
        {
            try
            {
                trace("doLogResponse: " +JSON.stringify(aResponse));

                var response = aResponse[0];

                if (response == null || response == "") return;

                trace("doLogResponse, Node Title: " +response.nodetitle);

                var fields = response.fields[0];

                if (fields == null || fields == "") return;

                trace("doLogResponse, Field Title: " +fields.fieldtitle);

                var fieldValue = fields.fieldvalue;

                if (fieldValue == null || fieldValue == "") return;

                trace("doLogResponse, Field Value: " +fieldValue);

                var changes = JSON.parse(fieldValue);

                self.changes = changes;
            }
            catch (err)
            {
                error("doLogResponse, Exception: " +err);
            }
        }
        
        
        function onLogResponse(aRequestId, aSuccess, aResponse)
        {
            trace("changeslogcontrol, onLogResponse: " +aResponse);

            self.$apply(doLogResponse(aResponse));
        }
    }]);

    function getAngularScope(aAngularArea)
    {
    //--    trace(">> getAngularScope");

        var angularElement = document.getElementById(aAngularArea);

        if (angularElement == null)
        {
            error("getAngularScope, Cannot find angular area element: " +aAngularArea);
            return;
        }

        var scope = angular.element(angularElement).scope();

        if (scope == null)
        {
            error("getAngularScope, annot get angular scope in area: " +aAngularArea);
            return;
        }

    //--    trace("<< getAngularScope");    

        return scope;
    }

    var mAngularScope = null;


    var mTestResponse = [
                        {
                            "nodetype":"leaf",
                            "nodetitle":"Actions",
                            "nodeid":"actions",
                            "nodeidpath":"root.logger.actions",
                            "mode":"rw",
                            "fields":
                                [
                                    {
                                        "fieldtype":"text",
                                        "fieldid":"logbytime",
                                        "fieldtitle":"Actions By Time",
                                        "fieldvalue": "[{\"entryType\":\"update\", \"entryTime\":1477473330, \"actor\":\"GENIUS\", \"node\":\"Sleep\", \"field\":\"Screen Off Time (seconds)\", \"newValue\":\"60\"}]"
                                    }
                                ]
                        }
                    ];

    function injectTestResponse()
    {
        trace("injectTestResponse");
        mAngularScope.$apply(function() { doLogResponse(mTestResponse); });                    
    }                    

    function getChangesLog()
    {
        trace("getChangesLog");            

        if (MMRSUTILS.isNativeRun())
        {
            MMRSCLIENTGATEWAY.sendGetCommandToDevice("root.logger.changes", function(aRequestId, aSuccess, aResponse)
            {
                mAngularScope.$apply(function() { doLogResponse(aResponse); });            
            });       
        }
        else
        {
            setTimeout(injectTestResponse, 1000);
        }
    }


    function onLoad()
    {
        note(">> onLoad");

        //if (! mIsNative) localStorage.clear();

        mAngularScope = getAngularScope("angularArea");

        getChangesLog();

        note("<< onLoad");
    }

    var exports = {};

    exports.name = "changeslog";

    exports.onLoad = function()   { onLoad(); }

    return exports;



}());
