var MMRSCLIENTGATEWAY = (function () {
    "use strict";

    var mIsNativeRun = MMRSUTILS.isNativeRun();

    var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();    

    var mCommandCallbackByLocalId = {},
        mLocalRequestSequenceNum = 0,
        exports = {};

    //=============================================================================
    // Logging
    //=============================================================================
    //
    function trace(aText) {
        MMRSUTILS.doTrace(exports.name, aText);
    }
    
    function note(aText) {
        MMRSUTILS.doTrace(exports.name, aText);
    }
    
    function warn(aText) {
        MMRSUTILS.doWarn(exports.name, aText);
    }
    
    function error(aText) {
        MMRSUTILS.doError(exports.name, aText);
    }

    //=============================================================================
    // Sending a POST Command to Device
    //=============================================================================
    //
    
    function onSetCommandResponse(aRequestId, aSuccess, aPayload) {
        note("onSetCommandResponse, Success: " +aSuccess+ ", Payload: " +aPayload);
        
        var callback = mCommandCallbackByLocalId[aRequestId],
            resultString;

        if (callback == null) {
            error("onSetCommandResponse, No Callback stored for Local Request ID: " + aRequestId);
            return;
        }

        callback(aRequestId, aSuccess, aPayload);
    }


    function sendSetCommandResponse(aRequestId, aSuccess, aPayload, aDecode)
    {
        trace(">>sendSetCommandResponse, ID: " +aRequestId+ ", Success: " +aSuccess);
        trace("  sendSetCommandResponse, Payload: " +aPayload+ ", Decode: " +aDecode);

        var success = aSuccess == "1";
        var decode  = aDecode  == "1";

        if (decode)
        {
            aPayload = atob(aPayload);
        }    

        onSetCommandResponse(aRequestId, success, aPayload);

        trace("<< sendSetCommandResponse");        
    }


    function sendSetCommandToDevice(aCommandUrl, aPayload, aOnCommandDone) {
        note("sendSetCommandToDevice, Command: " + aCommandUrl + ", Payload: " + aPayload);

        if (aCommandUrl == null || aPayload == null || aOnCommandDone == null) {
            warn("sendSetCommandToDevice, Not enough parameters");
            return null;
        }

        var payloadStr = null,
            localRequestId,
            responseData;

        try {
            payloadStr = JSON.stringify(aPayload);
        } catch (err) {
            warn("sendSetCommandToDevice, Could not convert payload to JSON string: " + err);
            payloadStr = null;
        }

        trace("sendSetCommandToDevice, Sending: " + aCommandUrl + ", Payload: " + payloadStr);

        mLocalRequestSequenceNum++;

        localRequestId = "CMD-SET-" + mLocalRequestSequenceNum;

        trace("sendSetCommandToDevice, Sending: " + aCommandUrl);

        mCommandCallbackByLocalId[localRequestId] = aOnCommandDone;


        if (mIsNativeRun)
        {
            responseData = DEVICEBRIDGE.onSetCommand(aCommandUrl, localRequestId, payloadStr);

            if (responseData == "ASYNC")
            {
                trace("sendSetCommandToDevice, Command will complete asynchronously");
            }
            else
            {
                error("sendSetCommandToDevice, Command tried to complete SYNCHRONOUSLY - No longer supported");
            }
        }
                
        return localRequestId;
    }

    //=============================================================================
    // Sending a GET Command to Device
    //=============================================================================
    //

    function onGetCommandResponse(aRequestId, aSuccess, aData) {
        trace("onGetCommandResponse, Id: " + aRequestId + ", Success: " + aSuccess);
        trace("onGetCommandResponse, aData: " + aData);
        //trace("onGetCommandResultsRequestSent, Data: " +aData);

        var callback = mCommandCallbackByLocalId[aRequestId],
            data = null;

        if (callback == null) {
            error("onGetCommandResponse, No Callback stored for Local Request ID: " + aRequestId);
            return;
        }

        try {
            data = JSON.parse(aData);
            trace("onGetCommandResponse, Data: " + aData);
        } catch (err) {
            data = null;
            trace("onGetCommandResponse, JSON Error: " + err);
        }

        if (data == null) {
            data = aData;
        }

        callback(aRequestId, aSuccess, data);
    }

        function sendGetCommandResponse(aRequestId, aSuccess, aPayload, aDecode) {
            trace(">> sendGetCommandResponse");

            var decodedData;

            if (aDecode) {
                trace("decoding data");
                decodedData = window.atob(aPayload);
                trace("decoded data: " + decodedData);
                onGetCommandResponse(aRequestId, aSuccess, decodedData);
            } else {
                onGetCommandResponse(aRequestId, aSuccess, aPayload);
            }

            trace("<< sendGetCommandResponse");
        }

    function sendGetCommandToDevice(aCommandUrl, aCallback) {
        note("sendGetCommandToDevice, Command: " + aCommandUrl);

        if (aCommandUrl == null || aCallback == null) {
            warn("sendGetCommandToDevice, Not enough parameters");
            return null;
        }

        trace("sendGetCommandToDevice, Sending: " + aCommandUrl);

        mLocalRequestSequenceNum++;

        var localRequestId = "CMD-GET-" + mLocalRequestSequenceNum,
            responseData;

        trace("sendGetCommandToDevice, Sending: " + aCommandUrl + " (" + localRequestId + ")");

        mCommandCallbackByLocalId[localRequestId] = aCallback;

        if (mIsNativeRun)
        {
            responseData = DEVICEBRIDGE.onGetCommand(aCommandUrl, localRequestId);
        }
        
        // TODO: temp code - we are doing this synchronously, it should be asynchronour (called back into from Android Java)
        // onGetCommandResponse(localRequestId, "200", responseData);

        return localRequestId;
    }

    exports.name = "mmrsclientgateway_local";

    exports.sendGetCommandToDevice  = function (aCommandUrl, aCallback) {
        return sendGetCommandToDevice(aCommandUrl, aCallback);
    };
    
    exports.sendSetCommandToDevice  = function (aCommandUrl, aPayload, aCallback) {
        return sendSetCommandToDevice(aCommandUrl, aPayload, aCallback);
    };

    exports.sendGetCommandResponse = function (aRequestId, aSuccess, aPayload, aDecode) {
        return sendGetCommandResponse(aRequestId, aSuccess, aPayload, aDecode);
    };

    exports.sendSetCommandResponse  = function(aRequestId, aSuccess, aPayload, aDecode) {
        return sendSetCommandResponse(aRequestId, aSuccess, aPayload, aDecode); 
    }


    return exports;

}());
