/*jslint browser: true*/
/*global $, jQuery, alert, angular, MMRSUTILS, MMRSCLIENTGATEWAY, MMRSCONFIG*/
var MMRSHARDWARETESTS = (function () {
    "use strict";

    function trace(aText) {
        MMRSUTILS.doTrace("mmrshardwaretests", aText);
    }

    function note(aText) {
        MMRSUTILS.doTrace("mmrshardwaretests", aText);
    }

    function warn(aText) {
        MMRSUTILS.doWarn("mmrshardwaretests", aText);
    }

    function error(aText) {
        MMRSUTILS.doError("mmrshardwaretests", aText);
    }

    // angular.module('quickCheckApp').controller('hardwareTestsControl', ['$scope', function ($scope) {
    var app = angular.module('quickCheckApp', ['ngSanitize']);
    app.controller('hardwareTestsControl', function ($scope) {
        /*
    Create a new test status to a group
    */
        function addNewTestStatusToGroup(aTestGroup, aTitle) {
            var newTestStatus = {
                "title" : aTitle,
                "status" : $scope.NOT_RUN,
                "details" : "Not Run",
                "fixes" : []
            };

            aTestGroup.tests.push(newTestStatus);

            return newTestStatus;
        }

        /*
    Create a new info item to a group
    An info item is similar to a test, except it doesn't check any values (it always displays the same content)
    */
        function addNewInfoItemToGroup(aTestGroup, aTitle, aDetails) {
            var newInfoStatus = {
                "title" : aTitle,
                "status" : "INFO",
                "details" : aDetails,
                "fixes" : []
            };

            aTestGroup.tests.push(newInfoStatus);

            return newInfoStatus;
        }

        /*
    Create a new (empty) test group and add it to aTestGroups
    Returns the created test group
    */
        function addNewTestGroup(aTestGroups, aTitle) {
            var newTestGroup = {"title" : aTitle,
                                tests : []};

            aTestGroups.push(newTestGroup);

            return newTestGroup;
        }

        // Add a possible fix to the test status
        // Used when a test gives a warning (or worse)
        // The fix could be something like "Change screen timeout to 30 seconds"
        // aFixFunction: callback function to apply the fix (called when the user selects the option to apply the fix). The callback function takes no parameters (assume that all data is already available to callback?)
        // aFixButtonText: the text displayed in the button (or link, or similar) that applies the fix
        // aFixTextDetails: extra info on what the fix will do (may be displayed as a hover item, or is a confirmation / info dialog)
        function addFixToTestStatus(aFixButtonText, aFixTextDetails, aTestStatus, aFixFunction) {
            var newFix = {
                "buttonText" : aFixButtonText,
                "textDetails" : aFixTextDetails,
                "fixFunction" : aFixFunction
            };

            //--        trace("Pushing new fix to aTestStatus");
            aTestStatus.fixes.push(newFix);
        }
        
        function setTestStatusOk(aTestStatus) {
            trace(">> setTestStatusOk: " + aTestStatus.details);
            aTestStatus.status = "OK";
            aTestStatus.details = "No issues detected";
            trace("<< setTestStatusOk");
        }

        function setTestStatusWarning(aTestStatus, aWarningText) {
            aTestStatus.status = "WARNING";
            aTestStatus.details = aWarningText;

            trace("setTestStatusWarning, +++ Title: " + aTestStatus.title  + ", Warning: " + aWarningText);

            sendCommentToLogger("warning", aTestStatus.title, aWarningText);
        }

        function setTestStatusCriticalWarning(aTestStatus, aWarningText) {
            aTestStatus.status = "CRITICAL_WARNING";
            aTestStatus.details = aWarningText;
        }
        
        $scope.hardwareTestGroups = [];
        
        $scope.NOT_STARTED = "NOT_RUN";
        $scope.RUNNING = "RUNNING";
        $scope.FAILED = "FAILED";
        $scope.PASSED = "PASSED";
        
        // Utility function to set data on the device as part of an automated fix
        function applyFixToDevice(aUrl, aFixedData, aTestStatus) {
//--            trace(">> applyFixToDevice: aUrl: " + aUrl + " aFixedData: " + JSON.stringify(aFixedData));


//--            trace("<< applyFixToDevice");
        }
        

        function sendCommentToLogger(aLevel, aTitle, aDetail) {
            trace("sendCommentToLogger, Level: " + aLevel + ", Title: " + aTitle + ", Detail: " + aDetail);

            var update = {
                action: "update",
                fields: [
                    { action: "upate",  field:  "commentAddLevel",  value:  aLevel  },
                    { action: "update", field:  "commentAddTitle",  value:  aTitle  },
                    { action: "upate",  field:  "commentAddDetail", value:  aDetail }
                ]
            },
                actionUrl = "root.logger.commentadd";

            trace("sendUpdateActionStatus, Sending Comment to device");
            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(actionUrl, update, function (aRequestId, aSuccess, aResponse) {
                $scope.$apply(function () {
                    trace("sendCommentToLogger, sendSetCommandToDevice: Success: " + aSuccess);
                });
            });
        }

        // Start a settings screen and update the test status
        function startScreen(aScreenUrl, aScreenId, aTestStatus) {
            var screenAction = {
                action: "update",
                fields: [{
                    action: "update",
                    field: aScreenId,
                    value: "screen-start"
                }]
            };
            
            aTestStatus.status = "FIXING";
            aTestStatus.details = "Launching screen - Please wait";
            aTestStatus.fixes = [];
            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(aScreenUrl, screenAction, function (aRequestId, aSuccess, aSetCommandStatus) {
                $scope.$apply(function () {
                    trace("startScreen: sendSetCommandToDevice: Success: " + aSuccess);

                    if (aSuccess) {
                        trace("Screen launched");
                        aTestStatus.status = "FIXED";
                        aTestStatus.details = "Launched screen";
                        aTestStatus.fixes = [];
                    } else {
                        trace("Failed to launch screen");
                        aTestStatus.status = "FIX_FAILED";
                        aTestStatus.details = "Failed to launch screen";
                        aTestStatus.fixes = [];
                    }
                });
            });
        }
        
        function setFixFailed(aTestStatus) {
            aTestStatus.status = "FIX_FAILED";
            aTestStatus.details = "Fix failed";
            aTestStatus.fixes = [];
        }

        function initTests() {
            // TODO: this function is getting big - split it into individual tests
                // TODO: pass the batcher into each sub-test? or pre-fetch and then pass data in?
                // TODO: what if multiple tests request the same data? can we just fetch it once and then complete
                    // TODO: BUT we need to ensure that the ordering of responses to requests is preserved
            var hardwareTestGroup;

            ///////////////////////////////////////////////////////////////////////////////////////
            // Hardware tests
            //

            hardwareTestGroup = addNewTestGroup($scope.hardwareTestGroups, "Hardware Tests");
            
            function addHardwareTest(aTestTitle, aTestId) {
                var screenTestStatus;
                screenTestStatus = addNewTestStatusToGroup(hardwareTestGroup, aTestTitle);
                addFixToTestStatus("Start test", "", screenTestStatus, function () {

                    var screenAction = {
                            action: "update",
                            fields: [{
                                action: "update",
                                field: aTestId,
                                value: "screen-start"
                            }]
                        };

                    screenTestStatus.status = $scope.RUNNING;
                    screenTestStatus.details = "Test Running";

                    trace("About to start test");
                    MMRSCLIENTGATEWAY.sendSetCommandToDevice("root.hardware-tests.screens", screenAction, function (aRequestId, aResult, aSetCommandStatus) {
                        trace("sendSetCommandToDevice complete");

                        $scope.$evalAsync(function (evalAsyncScope) {
                            trace("Updating test results");
                            trace("aSetCommandStatus: " + JSON.stringify(aSetCommandStatus));
                            var parsedSetCommandStatus = JSON.parse(aSetCommandStatus),
                                changedNodes,
                                fields,
                                firstField,
                                firstChangedNode,
                                testResult;

                            changedNodes = parsedSetCommandStatus.changedNodes;

                            if (changedNodes) {
                                firstChangedNode = changedNodes[0];
                                if (firstChangedNode) {
                                    fields = firstChangedNode.fields;
                                    if (fields) {
                                        firstField = fields[0];
                                        if (firstField) {
                                            testResult = firstField.value;
                                            trace("testResult : " + testResult);

                                            if (testResult === "ACTION_FAILED") {
                                                screenTestStatus.status = $scope.FAILED;
                                                screenTestStatus.details = "Test Failed";
                                                sendCommentToLogger("critical warning", "Hardware test failed", aTestTitle);
                                            } else if (testResult === "ACTION_COMPLETE") {
                                                screenTestStatus.status = $scope.PASSED;
                                                screenTestStatus.details = "Test Passed";
                                                sendCommentToLogger("info", "Hardware test passed", aTestTitle);
                                            } else if (testResult === "ACTION_CANCELED") {
                                                screenTestStatus.status = $scope.NOT_STARTED;
                                                screenTestStatus.details = "Test Canceled";
                                                sendCommentToLogger("info", "Hardware test canceled", aTestTitle);
                                            } else if (testResult === "ACTION_ERROR") {
                                                screenTestStatus.status = $scope.NOT_STARTED;
                                                screenTestStatus.details = "Test General Error";
                                                sendCommentToLogger("info", "Hardware test general error", aTestTitle);
                                            }
                                        }
                                    }
                                }
                            }
                        });
                    });
                });
                
                
                                   

                /*
                var assistPages = aAssistPages,
                    hardwareTestPageData = jQuery.extend(true, {}, assistPages.pages["mmrs.hardwareTestAssist"]);
                
                $scope.assistService = aAssistService;
                hardwareTestPageData.title = "Screen Test";
                hardwareTestPageData.launchscreen = "screen-test";
                // root|blackbelt-analyst|screens, Field: screen-test

                aAssistService.displayDialog('#mmrsAssistModalHardware', hardwareTestPageData);
                */
            }
            addHardwareTest("Version", "version-test");
            addHardwareTest("Battery Charge Test", "battery-charge-test");
            addHardwareTest("Screen Dead Pixels", "screen-pixel-test");
            addHardwareTest("Screen Touch Sensitivity", "screen-drawing-test");
            addHardwareTest("Speaker Test", "speaker-test");
            addHardwareTest("Earpiece Test", "earpiece-test");
            addHardwareTest("Front Camera Test",  "camera-front-viewfinder-test");
            addHardwareTest("Rear Camera Test",  "camera-rear-viewfinder-test");
            addHardwareTest("Camera Flash Test",  "camera-flash-test");
            addHardwareTest("Microphone Live Test",  "microphone-live-test");
            addHardwareTest("Microphone Playback Test",  "microphone-playback-test");
            addHardwareTest("Light Sensor Test",  "light-sensor-test");
            addHardwareTest("Proximity Sensor Test",  "proximity-sensor-test");
            addHardwareTest("NFC Sensor Test",  "nfc-sensor-test");
            addHardwareTest("Backlight Test", "backlight-test");
            addHardwareTest("Call Test", "call-test");
            addHardwareTest("Sensors Test","sensors-test");
            addHardwareTest("GPS Test", "gps-test");
            addHardwareTest("Bluetooth Test", "bluetooth-test");
            addHardwareTest("Wi-Fi Test", "wifi-test");
            addHardwareTest("Vibrating Test", "vibrating-test");


//            addHardwareTest("Speaker", "speaker-test");
//            addHardwareTest("Camera", "camera-test");
//            addHardwareTest("Microphone", "microphone-test");
//            // addHardwareTest("Dimming", "dimming-test");
//            addHardwareTest("Light Sensor", "light-sensor-test");
//            addHardwareTest("NFC", "nfc-test");
//            addHardwareTest("Proximity Sensor", "proximity-test");
//            addHardwareTest("Camera Flash", "camera-flash-test");
//            addHardwareTest("Vibra", "vibra-test");
//            addHardwareTest("GPS", "gps-test");
//            addHardwareTest("Backlight", "backlight-test");
//            addHardwareTest("Sensors", "sensors-test");
//            addHardwareTest("Call", "call-test");
//            addHardwareTest("Wi-Fi", "wifi-test");
//            addHardwareTest("Bluetooth", "bluetooth-test");
        }
        
        initTests();
    });
}());
