var MMRSHOME = (function() {
	"use strict"

var mIsNativeRun = false; //MMRSUTILS.isNativeRun();

var BRIDGE = MMRSUTILS.getDeviceBridge();

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }

var EOR = String.fromCharCode(10);

window.onerror = function(aMessage) { error(aMessage); }

var mTheApp = angular.module('aApp', []);

mTheApp.controller('aController', function($scope)
{
    //var WORKFLOW_PAGE_LINK = "mmrsswap.html?page="; // The old swap check page

    var WORKFLOW_PAGE_LINK = "workflow/workflow.html?workflow=flows/swap/index.js&title=Device%20upgrade&page=";

    var ASSESSMENT_VALIDITY_DAYS = 7;

    var ACCOUNT_START_DATE_PROPERTY  = "accountContractStartDate";
    var ACCOUNT_LEASE_PLAN_PROPERTY  = "accountOnLeasePlan";
    var ACCOUNT_ASSURE_PLAN_PROPERTY = "accountOnAssurePlan";
    var ACCOUNT_PRODUCTS             = "accountProducts";
    var OPTION_ALLOW_SWAP            = "optionAllowSwap";

    var mAccountOnLeasePlan  = false;
    var mAccountOnAssurePlan = false;
    var mAccountStartTime    = null;

    $scope.menuItems =
    [
        { // 0
            action:  "backup.html",
            icon:    "images/icontransfer.png",
            title:   "Transfer, backup or restore",
            comment: "Manage your content",
        },    
        { // 1
            action:  "localcheck.html",
            icon:    "images/iconoptimise.png",
            title:   "Self-Care",
            comment: "Optimise & test your device"
        },
        { // 2
            action:  "remotecheck.html",
            icon:    "images/iconagent.png",
            title:   "Assisted care",
            comment: "Get support from a Telstra agent"
        },
        { // 3
            action:  "hardwaretests.html",
            hide:    true,
            icon:    "images/iconoptimise.png",
            title:   "Hardware Tests",
            comment: "Internal - Hardware diagnostics"
        },        
        { // 4
            action:  "doswap",
            hide:    true,
            icon:    "images/iconplan.png",
            title:   "Your lease plan",
            comment: "Your plan and upgrade options"
        }

    ];
/*

            { // 4
            action:  "doswap",
            hide:    true,
            icon1:    "images/icondevice.png",
            title:   "Upgrade your device",
            comment: "You started leasing this device on "
        }
*/

    $scope.mSwapMenuItemIndex = 4;

    $scope.onMenuItemClick = function(aItem)
    {
        trace("onMenuItemClick, Item: " +aItem.title);

        if (aItem.action == "doswap")
        {
            startSwapFlow();
            return;
        }

        window.location.href = aItem.action;
    }

    function startSwapFlow()
    {
        var accountDate    = new Date(mAccountStartTime*1000);
        var year1EndDate   = new Date(mAccountStartTime*1000);
        var month30EndDate = new Date(mAccountStartTime*1000);

        year1EndDate.setFullYear(year1EndDate.getFullYear() + 1);

        month30EndDate.setMonth(month30EndDate.getMonth() + 30);

        //trace("startSwapFlow, Account Date: " +accountDate.toISOString());
        //trace("startSwapFlow, Year 1 End:   " +year1EndDate.toISOString());
        //trace("startSwapFlow, Month 30 End: " +month30EndDate.toISOString());

        var timeNow        = getTimeNow();
        var year1EndTime   = year1EndDate.getTime();
        var month30EndTime = month30EndDate.getTime();

        // Without assure - Year 1
        //
        if (timeNow < year1EndTime)
        {
            var appState = mAccountOnAssurePlan ? MMRSAPPSTATE.EVERGREEN_INSIDE_PERIOD : 
                                                  MMRSAPPSTATE.EVERGREEN_OUTSIDE_PERIOD;

            BRIDGE.setAppState(appState);
            var pageId = mAccountOnAssurePlan ? "assure-year1" : "noassure-year1";

            trace("startSwapFlow, Page: " +pageId);

            window.location.href = WORKFLOW_PAGE_LINK + pageId;
            return;
        }

        if (timeNow >= month30EndTime)
        {
            BRIDGE.setAppState(MMRSAPPSTATE.EVERGREEN_EXPIRED_PERIOD);        

            var pageId = "swap-month30";

            trace("startSwapFlow, Page: " +pageId);

            window.location.href = WORKFLOW_PAGE_LINK + pageId;
            return;
        }
        
        BRIDGE.setAppState(MMRSAPPSTATE.EVERGREEN_INSIDE_PERIOD);        

        if (haveValidAssessment())
        {
            trace("startSwapFlow, Page: " +pageId);

            window.location.href = WORKFLOW_PAGE_LINK + "valid-preassessment&finishOnPush=true";

            return;
        }

        var pageId = mAccountOnAssurePlan ? "assure-year2" : "noassure-year2";

        var pageId = "swap-year2";

        trace("startSwapFlow, Page: " +pageId);

        window.location.href = WORKFLOW_PAGE_LINK + pageId;
    }

    function haveValidAssessment()
    {
        var assessmentStr = BRIDGE.readPage("UserReportSwapAssessment");

        trace("haveValidAssessment, Assessment: " +assessmentStr);

        if (assessmentStr == null || assessmentStr == "")
        {
            trace("haveValidAssessment, No user Assessment stored");
            return false;
        }

        var assessment = null;

        try 
        {
            assessment = JSON.parse(assessmentStr); 
        }
        catch (err)
        {
            assessment = null;
        }

        if (assessment == null)
        {
            return false;
        }

        var reportIMEI = assessment.imei;

        if (reportIMEI == null)
        {
            trace("retrieveAssessment, Report has no IMEI");

            return false;
        }

        // Check if the swap assessment report has come from a different device
        // Possibly due to the user backing up their old device and restoring it onto
        // this (new) device.
        //
        var deviceIMEI = BRIDGE.getValue("imei");

        if (reportIMEI != deviceIMEI)
        {
            trace("haveValidAssessment, Report is not from this device");
            return false;
        }

        var reportDate = new Date(assessment.created);
        
        if (reportDate == null)
        {
            return false;
        }

        var reportDateStr = makeDayDateString(reportDate);

        trace("haveValidAssessment, Report Date: " +reportDateStr);

        reportDate.setDate(reportDate.getDate() + ASSESSMENT_VALIDITY_DAYS);

        var reportValidEndStr = makeDayDateString(reportDate);

        trace("haveValidAssessment, Validity End: " +reportValidEndStr);

        var reportValidEndTime = reportDate.getTime();

        var timeNow = getTimeNow();

        var reportValid = timeNow < reportValidEndTime;

        trace("haveValidAssessment, Report Valid: " +reportValid);

        return reportValid;
    }


    var MONTH_NAMES = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
    var MONTH_NAMES = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

    function makeDayDateString(aDate)
    {
        var dayOfWeek  = aDate.getDay();
        var dayOfMonth = aDate.getDate();

        var month = aDate.getMonth();
        var year  = aDate.getFullYear();

        var dateStr = dayOfMonth+ " " +MONTH_NAMES[month]+ " " +year;

        return dateStr;
    }

    function getTimeNow()
    {
        var dateNow = new Date();
        var timeNow = dateNow.getTime();

        return timeNow;
    }


    ///////////////////////////////////////////////////////////////////////////////////////

 	$scope.doLoad = function()
    {
	    //trace(">> doLoad");

        BRIDGE.setAppState(MMRSAPPSTATE.HOME);

        mAccountOnLeasePlan  = BRIDGE.getValue(ACCOUNT_LEASE_PLAN_PROPERTY)  == "true";
        mAccountOnAssurePlan = BRIDGE.getValue(ACCOUNT_ASSURE_PLAN_PROPERTY) == "true";
        mAccountStartTime    = BRIDGE.getValue(ACCOUNT_START_DATE_PROPERTY);

        var doSwap = BRIDGE.getValue(OPTION_ALLOW_SWAP)  == "true";

        trace("doLoad, Do Swap Option: " +doSwap);
        trace("doLoad, On Lease Plan:  " +mAccountOnLeasePlan);
        trace("doLoad, On Assure Plan: " +mAccountOnAssurePlan);
        trace("doLoad, Contract Time:  " +mAccountStartTime);

        //var haveAssessment = haveValidAssessment();

        if (doSwap)
        {
            $scope.$applyAsync(function()
            {
                var swapTile = $scope.menuItems[$scope.mSwapMenuItemIndex];

                swapTile.hide = false;

                //var accountStartDate = new Date(mAccountStartTime*1000);

                //swapTile.comment += makeDayDateString(accountStartDate);
            });
        }

        /*
        $scope.$applyAsync(function()
        {
            var deviceHandle = BRIDGE.getValue("device-handle");

            var pageHeading = "TELSTRA DEVICE CARE";

            if (deviceHandle != null && deviceHandle != "")
            {
                pageHeading = deviceHandle; 
            }

            $scope.pageHeading = pageHeading;

            var imei = BRIDGE.getValue("imei");  

            if (imei != null && imei != "")
            {
                $scope.pageSubHeading = "IMEI: " +imei;
            }            
        });
        */

	    //trace("<< doLoad");
	}

    $scope.doLoad();
});

function onLoad()
{
}

var exports = {};

exports.name = "mmrshome";

exports.onLoad = function(aOptions) { onLoad(aOptions); }

return exports;

}());
