/*jslint browser: true*/
/*global $, jQuery, alert, angular, MMRSUTILS, MMRSCLIENTGATEWAY, MMRSCONFIG, MMRSCHARTUTILS*/
var MMRSLOGIN = (function () {

    function trace(aText) { MMRSUTILS.doTrace("mmrslogin", aText);  }
    function note(aText)  { MMRSUTILS.doTrace("mmrslogin", aText);  }
    function warn(aText)  { MMRSUTILS.doWarn("mmrslogin", aText);   }
    function error(aText) { MMRSUTILS.doError("mmrslogin", aText);  }


    var mNumOpenActions = -1;

    function getActionsLog()
    {
        trace("getActionsLog");

        if (MMRSUTILS.isNativeRun())
        {
            MMRSCLIENTGATEWAY.sendGetCommandToDevice("root.logger.actions", function(aRequestId, aSuccess, aResponse)
            {
                doLogResponse(aResponse);
            });
        }
    }

    function countOpenActions(aActions)
    {
        var numOpen    = 0;
        var numActions = 0;

        for (var actionId in aActions)
        {
            numActions++;

            var thisAction = aActions[actionId];

            var status = thisAction.status;

            if (status == "open") numOpen++;
        }

        trace("countOpenActions, Total Actions: " +numActions+ ", Open: " +numOpen);

        return numOpen;
    }

    function readActions(aActionsList)
    {
        var actionsById = {};

        for (var index in aActionsList)
        {
            var thisAction = aActionsList[index];

            var entryId        = thisAction.entryId;
            var entryType      = thisAction.entryType;
            var actionType     = thisAction.entrySubType;
            var actionTime     = thisAction.entryTime;
            var actionText     = thisAction.text;
            var actionAuthor   = thisAction.actor;
            var actionAssignee = thisAction.assignee;
            var actionStatus   = thisAction.status;

            trace("readActions, Index: " +index+ ", Id: " +entryId+ ", Entry: " +entryType+ ", Type: " +actionType+ ", Text: " +actionText+ ", Status: " +actionStatus);

            if (actionsById[entryId] == null)
            {
                actionsById[entryId] = thisAction;
                actionsById[entryId].updates = [];
            }
            else
            {
                actionsById[entryId].updates.push(thisAction);
            }

            actionsById[entryId].status = thisAction.status;
        }

        if (mNumOpenActions == -1)
        {
            mNumOpenActions = countOpenActions(actionsById);

            if (mNumOpenActions > 0)
            {
                var actionText = mNumOpenActions == 1 ? "There is 1 open action" : "There are " +mNumOpenActions+ " open actions";

                //window.alert(actionText+ " for this device.\n\nSelect the Actions Tab to review them.");

                if (MMRSUTILS.isNativeRun())
                {
                    responseData = MMRSNATIVE.onOpenActionsCount(mNumOpenActions);
                }                
            }
        }
    }

    function doLogResponse(aResponse)
    {
        try
        {
            trace("doLogResponse: " +JSON.stringify(aResponse));

            //$('#actionsWait').modal('hide')
            var response = aResponse[0];

            if (response == null || response == "") return;

            trace("doLogResponse, Node Title: " +response.nodetitle);

            var fields = response.fields[0];

            if (fields == null || fields == "") return;

            trace("doLogResponse, Field Title: " +fields.fieldtitle);

            var fieldValue = fields.fieldvalue;

            if (fieldValue == null || fieldValue == "") return;

            trace("doLogResponse, Field Value: " +fieldValue);

            var actions = JSON.parse(fieldValue);

            readActions(actions);
        }
        catch (err)
        {
            error("doLogResponse, Exception: " +err);
        }
    }

    function onLoad()
    {
        note(">> onLoad");

        //if (! mIsNative) localStorage.clear();

        getActionsLog();

        note("<< onLoad");
    }

    var exports = {};

    exports.name = "login";

    exports.onLoad = function()   { onLoad(); }

    return exports;



}());
