/*jslint browser: true*/
/*global $, jQuery, alert, angular, MMRSUTILS, MMRSCLIENTGATEWAY*/
var MMRSQUICKCHECK = (function () {
    "use strict";

    var app = angular.module('mmrsApp', ['ngSanitize', 'ui.bootstrap']),
        SIGNAL_STRENGTH_POOR_THRESHOLD = 20;

    function trace(aText) {
        MMRSUTILS.doTrace("mmrsquickcheck", aText);
    }

    function note(aText) {
        MMRSUTILS.doTrace("mmrsquickcheck", aText);
    }

    function warn(aText) {
        MMRSUTILS.doWarn("mmrsquickcheck", aText);
    }

    function error(aText) {
        MMRSUTILS.doError("mmrsquickcheck", aText);
    }

    /*
    Create a new (empty) test group and add it to aTestGroups
    */
    function addNewTestStatusToGroup(aTestGroup, aTitle) {
        var newTestStatus = {
            "title" : aTitle,
            "status" : "CHECKING",
            "details" : "Please wait",
            "fixes" : []
        };

        aTestGroup.tests.push(newTestStatus);

        return newTestStatus;
    }
    
    /*
    Create a new info item to a group
    An info item is similar to a test, except it doesn't check any values (it always displays the same content)
    */
    function addNewInfoItemToGroup(aTestGroup, aTitle, aDetails) {
        var newInfoStatus = {
            "title" : aTitle,
            "status" : "INFO",
            "details" : aDetails,
            "fixes" : []
        };

        aTestGroup.tests.push(newInfoStatus);

        return newInfoStatus;
    }

    /*
    Create a new (empty) test group and add it to aTestGroups
    Returns the created test group
    */
    function addNewTestGroup(aTestGroups, aTitle) {
        var newTestGroup = {"title" : aTitle,
                            tests : []};

        aTestGroups.push(newTestGroup);

        return newTestGroup;
    }

    function setTestStatusOk(aTestStatus) {
//--        trace(">> setTestStatusOk: " + aTestStatus.details);
        aTestStatus.status = "OK";
        aTestStatus.details = "No issues detected";
//--        trace("<< setTestStatusOk");
    }
    
    // Add a possible fix to the test status
    // Used when a test gives a warning (or worse)
    // The fix could be something like "Change screen timeout to 30 seconds"
    // aFixFunction: callback function to apply the fix (called when the user selects the option to apply the fix). The callback function takes no parameters (assume that all data is already available to callback?)
    // aFixButtonText: the text displayed in the button (or link, or similar) that applies the fix
    // aFixTextDetails: extra info on what the fix will do (may be displayed as a hover item, or is a confirmation / info dialog)
    function addFixToTestStatus(aFixButtonText, aFixTextDetails, aTestStatus, aFixFunction) {
        var newFix = {
            "buttonText" : aFixButtonText,
            "textDetails" : aFixTextDetails,
            "fixFunction" : aFixFunction
        };
        
//--        trace("Pushing new fix to aTestStatus");
        aTestStatus.fixes.push(newFix);
    }

    app.controller('myCtrl', ['$scope', 'mmrsAssist', 'mmrsAssistPages', function ($scope, aAssistService, aAssistPages) {
        // initMMRS();

        $scope.quickCheckGroups = [];

        function setTestStatusWarning(aTestStatus, aWarningText) {
            aTestStatus.status = "WARNING";
            aTestStatus.details = aWarningText;

            trace("setTestStatusWarning, +++ Title: " +aTestStatus.title+ ", Warning: " +aWarningText);

            sendCommentToLogger("warning", aTestStatus.title, aWarningText);
        }

            function setTestStatusCriticalWarning(aTestStatus, aWarningText) {
                aTestStatus.status = "CRITICAL_WARNING";
                aTestStatus.details = aWarningText;
                sendCommentToLogger("critical warning", aTestStatus.title, aWarningText);
            }

        function getDataFromDevice(aUrl, aResponseHandler) {
            MMRSCLIENTGATEWAY.sendGetCommandToDevice(aUrl, function (aRequestId, aSuccess, aResponseData) {
                $scope.$apply(function () {

                    var response = null;

                    if (aResponseData != null) {
                        response = aResponseData[0];
                    }

                    aResponseHandler(aRequestId, aSuccess, response);
                });
            });
        }
        
        // Utility function to set data on the device as part of an automated fix
        function applyFixToDevice(aUrl, aFixedData, aTestStatus) {
//--            trace(">> applyFixToDevice: aUrl: " + aUrl + " aFixedData: " + JSON.stringify(aFixedData));


//--            trace("<< applyFixToDevice");
        }

        function sendCommentToLogger(aLevel, aTitle, aDetail)
        {
            trace("sendCommentToLogger, Level: " +aLevel+ ", Title: " +aTitle+ ", Detail: " +aDetail);

            var update =
            {
                action: "update",
                fields: [
                            { action: "upate",  field:  "commentAddLevel",  value:  aLevel  },
                            { action: "update", field:  "commentAddTitle",  value:  aTitle  },
                            { action: "upate",  field:  "commentAddDetail", value:  aDetail }
                        ]
            };
            
            var actionUrl = "root.logger.commentadd";

            trace("sendUpdateActionStatus, Sending Comment to device");
            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(actionUrl, update, function (aRequestId, aSuccess, aResponse)
            {            
                $scope.$apply(function ()
                {
                    trace("sendCommentToLogger, sendSetCommandToDevice: Success: " + aSuccess);
                });
            });
        }


        // Start a settings screen
        function startScreen(aScreenUrl, aScreenId, aTestStatus) {
            var screenAction = {
                action: "update",
                fields: [{
                    action: "update",
                    field: aScreenId,
                    value: "screen-start"
                }]
            };
            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(aScreenUrl, screenAction, function (aRequestId, aSuccess, aSetCommandStatus) {
                // $scope.$apply(function () {
                    // Do nothing - no need to update status
                // });
            });
        }
        
        function setFixFailed(aTestStatus) {
            aTestStatus.status = "FIX_FAILED";
            aTestStatus.details = "Fix failed";
            aTestStatus.fixes = [];
        }
        
        function fixFieldValueOnDevice(aFetchedData, aUrl, aFieldName, aNewValue, aTestStatus) {
//--            trace(">> fixFieldValueOnDevice");
            var field = MMRSUTILS.getFieldUsingSettingItem(aFetchedData, aFieldName),
                fieldNumber,
                updateField,
                updateAction;
            
            if ((field === null) || (field === undefined)) {
                error("Unable to get field for: " + aFieldName);
                setFixFailed(aTestStatus);
            } else {
                fieldNumber = field.fieldid; // TODO: review comment: "fieldid" here vs. "field" in update action
                updateField = [{"action": "update",
                                "field": fieldNumber,
                                "value": aNewValue}];

                updateAction = {"action": "update",
                                "fields": updateField};
                
                aTestStatus.status = "FIXING";
                aTestStatus.details = "Applying fix - Please wait";
                aTestStatus.fixes = []; // clear fixes

                MMRSCLIENTGATEWAY.sendSetCommandToDevice(aUrl, updateAction, function (aRequestId, aSuccess, aSetCommandStatus) {
                    $scope.$apply(function () {
//--                        trace("Callback from: MMRSCLIENTGATEWAY.sendSetCommandToDevice: Success: " + aSuccess);
                        if (aSuccess) {
                            aTestStatus.status = "FIXED";
                            aTestStatus.details = "Fix applied";
                            aTestStatus.fixes = [];
                        } else {
                            setFixFailed(aTestStatus);
                        }
                    });
                });
            }
            
//--            trace("<< fixFieldValueOnDevice");
        }
        
        function fixFieldValueOnDeviceAsPercentage(aFetchedData, aUrl, aFieldName, aNewValue, aTestStatus) {
            var field = MMRSUTILS.getFieldUsingSettingItem(aFetchedData, aFieldName),
                absoluteValueNumber = NaN,
                matchingMaximumNumber = NaN,
                percentValueNumber = NaN,
                matchingField;
            
            if ((field === null) || (field === undefined)) {
                error("Unable to get field for: " + aFieldName);
            } else {
                percentValueNumber = Number(aNewValue);

                matchingField = MMRSUTILS.getFieldUsingSettingItem(aFetchedData, aFieldName);

                if ((matchingField !== null) && (matchingField !== undefined)) {
                    matchingMaximumNumber = Number(matchingField.fieldmaxvalue);
                    trace("Got maximum number: " + matchingMaximumNumber);
                }

                if ((isNaN(matchingMaximumNumber)) || (isNaN(percentValueNumber))) {
                    error("Can't set percentage value because there's no percent value, or no max value");
                    setFixFailed(aTestStatus);
                } else {
                    absoluteValueNumber = (percentValueNumber / 100) * matchingMaximumNumber;
                    absoluteValueNumber = Math.ceil(absoluteValueNumber);
                    if (absoluteValueNumber > matchingMaximumNumber) {
                        absoluteValueNumber = matchingMaximumNumber; // Just in case
                    }
                }
            }
            
            if (isNaN(absoluteValueNumber)) {
                setFixFailed(aTestStatus);
            } else {
                trace("Above to apply absolute value: " + absoluteValueNumber);
                fixFieldValueOnDevice(aFetchedData, aUrl, aFieldName, absoluteValueNumber, aTestStatus);
            }
        }

        var batteryTestGroup,
            mobileNetworkTestGroup,
            batteryHealthTestStatus,
            callVolumeTestStatus,
            ringVolumeTestStatus,
            musicVolumeTestStatus,
            alertVolumeTestStatus,
            alarmVolumeTestStatus,
            screenTimeoutPowerUsageTestStatus,
            screenBrightnessLevelTestStatus,
            screenAutomaticBrightnessTestStatus,
            simStatusTestStatus,
            flightModeStatusTestStatus,
            mobileNetworkServiceAvailabilityTestStatus,
            mobileSignalStrengthTestStatus,
            mobileDataEnbaledTestStatus,
            mobileDataRoamingTestStatus,
            mobileDataNetworkTypeTestStatus,
            mobileDataStatusTestStatus,
            generalGroup,
            firmwareInfo,
            batteryUsageInfo,
            wizardInfo;

        ///////////////////////////////////////////////////////////////////////////////////////
        // Battery and Power tests
        //

        batteryTestGroup = addNewTestGroup($scope.quickCheckGroups, "Battery and Power");

        // TODO: make URL, field name and "30" constant
        screenTimeoutPowerUsageTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Screen Timeout");
        getDataFromDevice("root.screen.sleep", function (aRequestId, aSuccess, aResponseData) {
            var screenTimeoutValue = MMRSUTILS.getValueFromResponseData(aResponseData, "screenofftime");
            if (screenTimeoutValue > 30) {
                setTestStatusWarning(screenTimeoutPowerUsageTestStatus, "The screen timeout is set to more than than 30 seconds. Long screen timeouts may cause excessive battery drain.");
                
                addFixToTestStatus("Change screen timeout to 30 seconds", "", screenTimeoutPowerUsageTestStatus, function () {
                    fixFieldValueOnDevice(aResponseData, "root.screen.sleep", "screenofftime", "30", screenTimeoutPowerUsageTestStatus);
                });
                
                addFixToTestStatus("Change screen timeout to 60 seconds", "", screenTimeoutPowerUsageTestStatus, function () {
                    fixFieldValueOnDevice(aResponseData, "root.screen.sleep", "screenofftime", "60", screenTimeoutPowerUsageTestStatus);
                });
                
                addFixToTestStatus("Change screen timeout to 120 seconds", "", screenTimeoutPowerUsageTestStatus, function () {
                    fixFieldValueOnDevice(aResponseData, "root.screen.sleep", "screenofftime", "120", screenTimeoutPowerUsageTestStatus);
                });
            } else {
                setTestStatusOk(screenTimeoutPowerUsageTestStatus);
            }
        });

        batteryHealthTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Battery Health");
        getDataFromDevice("root.device.battery", function (aRequestId, aSuccess, aResponseData) {
            var batteryHealth = MMRSUTILS.getValueFromResponseData(aResponseData, "health");
            if (batteryHealth !== "Good") {
                setTestStatusWarning(batteryHealthTestStatus, "Battery health warning: " + batteryHealth);
                // TODO: add "fix" that launches other window to process device return (potentially pre-populated with IMEI + diagnostics data)
            } else {
                setTestStatusOk(batteryHealthTestStatus);
            }
        });
        
        screenBrightnessLevelTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Screen Brightness Level");
        screenAutomaticBrightnessTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Automatic Screen Brightness");
        getDataFromDevice("root.screen.brightness", function (aRequestId, aSuccess, aResponseData) {
            var screenBrightnessLevel = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnesslevel"),
                screenBrightnessPercent = (screenBrightnessLevel / 255) * 100, // TODO: use max value rather than hard coded 255
                automaticScreenBrightnessSetting = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnessmode");
            
            if (screenBrightnessPercent > 75) {
                setTestStatusWarning(screenBrightnessLevelTestStatus, "Screen brightness is set to " + Math.ceil(screenBrightnessPercent) + "%. This may cause excessive battery drain.");
                addFixToTestStatus("Change screen brightness to 75%", "", screenBrightnessLevelTestStatus, function () {
                    fixFieldValueOnDevice(aResponseData, "root.screen.brightness", "brightnesslevel", "190", screenBrightnessLevelTestStatus); // TODO: use contant for 75% + use max value rather than 255
                });
            } else {
                setTestStatusOk(screenBrightnessLevelTestStatus);
            }
            
            if (automaticScreenBrightnessSetting == 0) { // 1 == automatic, 0 == manual
                setTestStatusWarning(screenAutomaticBrightnessTestStatus, "Screen brightness is set to \"manual\". This may cause excessive battery drain and make the screen harder to see in some conditions.");
                addFixToTestStatus("Change screen brightness to \"Automatic\"", "", screenAutomaticBrightnessTestStatus, function () {
                    fixFieldValueOnDevice(aResponseData, "root.screen.brightness", "brightnessmode", 1, screenAutomaticBrightnessTestStatus); // TODO: use contant for 1 (automatic)
                });
            } else {
                setTestStatusOk(screenAutomaticBrightnessTestStatus);
            }
        });

            batteryUsageInfo = addNewInfoItemToGroup(batteryTestGroup, "Battery Usage Troubleshooting Tip", "The Android settings app can show what is using the most battery power.");
            addFixToTestStatus("Launch the Android battery settings screen", "", batteryUsageInfo, function () {
                trace("Launch the Android battery usage screen");

                startScreen("root.intents.screens", "android/intent/action/POWER_USAGE_SUMMARY", batteryUsageInfo);

                // TODO: xxx - launch battery usage screen

                /*
                var assistPages = aAssistPages,
                    batteryAssistPageData;
                
                batteryAssistPageData = assistPages.pages["mmrs.batteryScreenAssist"];
                $scope.assistService = aAssistService;
                
                aAssistService.displayDialog('#mmrsAssistModal', batteryAssistPageData);
                */
            });
        // screenAutomaticBrightnessTestStatus
        
        // TODO: general review comment: should values all be strings like they are now? or should some be integers?
        // TODO: general review comment: is there any value in moving to the Angular JSON format?
        // TODO: general review comment: use constants for URL, field and status strings (especially as some are used more than once)
        // TODO: general review comment: how to support i8n
        // TODO: review comment: we don't seem to get errors if we fail to apply a fix (e.g. due to badly formatted field)
        
        ///////////////////////////////////////////////////////////////////////////////////////
        // Sound and Volume
        //

        batteryTestGroup = addNewTestGroup($scope.quickCheckGroups, "Sound and Volume");

        callVolumeTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Call Volume");
        getDataFromDevice("root.sound.voice", function (aRequestId, aSuccess, aResponseData) {
            var callVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "voice"),
                testFailed = false;
            
            if (callVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                setTestStatusWarning(callVolumeTestStatus, "Call volume is set to 0% (mute). Calls will be silent.");
                testFailed = true;
            } else if (callVolumePercent < 25) {
                setTestStatusWarning(callVolumeTestStatus, "Call volume is set very low (" + callVolumePercent + "%). Calls may be difficult to hear at this volume.");
                testFailed = true;
            }
            else {
                setTestStatusOk(callVolumeTestStatus);
            }
            
            if (testFailed) {
                addFixToTestStatus("Change call volume to 100%", "", callVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.voice", "voice", "100", callVolumeTestStatus);
                });
                addFixToTestStatus("Change call volume to 75%", "", callVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.voice", "voice", "75", callVolumeTestStatus);
                });
                addFixToTestStatus("Change call volume to 50%", "", callVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.voice", "voice", "50", callVolumeTestStatus);
                });
            } 
        });

        ringVolumeTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Ring Volume");
        getDataFromDevice("root.sound.ring", function (aRequestId, aSuccess, aResponseData) {
            var ringVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "ring"),
                testFailed = false;
            
            if (ringVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                setTestStatusWarning(ringVolumeTestStatus, "Ring volume is set to 0% (mute). Incoming calls will not ring.");
                testFailed = true;
            } else if (ringVolumePercent < 25) {
                setTestStatusWarning(ringVolumeTestStatus, "Ring volume is set very low (" + ringVolumePercent + "%). The ring for incoming calls may be difficult to hear at this volume.");
                testFailed = true;
            }
            else {
                setTestStatusOk(ringVolumeTestStatus);
            }
            
            if (testFailed) {
                addFixToTestStatus("Change ring volume to 100%", "", ringVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.ring", "ring", "100", ringVolumeTestStatus);
                });
                addFixToTestStatus("Change ring volume to 75%", "", ringVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.ring", "ring", "75", ringVolumeTestStatus);
                });
                addFixToTestStatus("Change ring volume to 50%", "", ringVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.ring", "ring", "50", ringVolumeTestStatus);
                });
            } 
        });
        
        musicVolumeTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Music and Media Volume");
        getDataFromDevice("root.sound.music", function (aRequestId, aSuccess, aResponseData) {
            var musicVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "music"),
                testFailed = false;
            
            if (musicVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                setTestStatusWarning(musicVolumeTestStatus, "Music and media volume is set to 0% (mute). Music and other media audio will not play.");
                testFailed = true;
            } else if (musicVolumePercent < 25) {
                setTestStatusWarning(musicVolumeTestStatus, "Music and media volume is set very low (" + musicVolumePercent + "%). Music and other media may be difficult to hear at this volume.");
                testFailed = true;
            }
            else {
                setTestStatusOk(musicVolumeTestStatus);
            }
            
            if (testFailed) {
                addFixToTestStatus("Change music and media volume to 100%", "", musicVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.music", "music", "100", musicVolumeTestStatus);
                });
                addFixToTestStatus("Change music and media volume to 75%", "", musicVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.music", "music", "75", musicVolumeTestStatus);
                });
                addFixToTestStatus("Change music and media volume to 50%", "", musicVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.music", "music", "50", musicVolumeTestStatus);
                });
            } 
        });
        
        // Commented out due to the S7 seemingly overriding the alert volume
        /*
        alertVolumeTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Notification Volume");
        getDataFromDevice("root.sound.alert", function (aRequestId, aSuccess, aResponseData) {
            var alertVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "alert"),
                testFailed = false;
            
            if (alertVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                setTestStatusWarning(alertVolumeTestStatus, "Notification volume is set to 0% (mute). You will not hear notification tones.");
                testFailed = true;
            } else if (alertVolumePercent < 25) {
                setTestStatusWarning(alertVolumeTestStatus, "Notification volume is set very low (" + alertVolumePercent + "%). Notification tones may be difficult to hear at this volume.");
                testFailed = true;
            }
            else {
                setTestStatusOk(alertVolumeTestStatus);
            }
            
            if (testFailed) {
                addFixToTestStatus("Change notification volume to 100%", "", alertVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.alert", "alert", "100", alertVolumeTestStatus);
                });
                addFixToTestStatus("Change notification volume to 75%", "", alertVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.alert", "alert", "75", alertVolumeTestStatus);
                });
                addFixToTestStatus("Change notification volume to 50%", "", alertVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.alert", "alert", "50", alertVolumeTestStatus);
                });
            } 
        });
        */
        
        // Commented out due to the S7 seemingly overriding the alarm volume
        /*
        alarmVolumeTestStatus = addNewTestStatusToGroup(batteryTestGroup, "Alarm Volume");
        getDataFromDevice("root.sound.alarm", function (aRequestId, aSuccess, aResponseData) {
            var alarmVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "alarm"),
                testFailed = false;
            
            if (alarmVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                setTestStatusWarning(alarmVolumeTestStatus, "Alarm volume is set to 0% (mute). Alarms will be silent.");
                testFailed = true;
            } else if (alarmVolumePercent < 25) {
                setTestStatusWarning(alarmVolumeTestStatus, "Alarm volume is set very low (" + alarmVolumePercent + "%). Alarms may be difficult to hear at this volume.");
                testFailed = true;
            }
            else {
                setTestStatusOk(alarmVolumeTestStatus);
            }
            
            if (testFailed) {
                addFixToTestStatus("Change alarm volume to 100%", "", alarmVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.alarm", "alarm", "100", alarmVolumeTestStatus);
                });
                addFixToTestStatus("Change alarm volume to 75%", "", alarmVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.alarm", "alarm", "75", alarmVolumeTestStatus);
                });
                addFixToTestStatus("Change alarm volume to 50%", "", alarmVolumeTestStatus, function () {
                    fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.alarm", "alarm", "50", alarmVolumeTestStatus);
                });
            } 
        });
        */
        
        ///////////////////////////////////////////////////////////////////////////////////////
        // Mobile Network
        //

        mobileNetworkTestGroup = addNewTestGroup($scope.quickCheckGroups, "Mobile Network and SIM");
        
        simStatusTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "SIM");
        flightModeStatusTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "Flight Mode Setting");
        mobileNetworkServiceAvailabilityTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "Mobile Service Availability");
        mobileSignalStrengthTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "Mobile Signal Strength");
        mobileDataEnbaledTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "Mobile Data Settings");
        mobileDataRoamingTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "Mobile Data Roaming");
        // mobileDataNetworkTypeTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "Mobile Data Network Type"); // TODO: Add this later?
        // mobileDataStatusTestStatus = addNewTestStatusToGroup(mobileNetworkTestGroup, "Mobile Data Status"); // TODO: Add this later
        
        getDataFromDevice("root.mobile_network.sim", function (aRequestId, aSuccess, aSimResponseData) {
            // Test SIM status
            var simStatus = MMRSUTILS.getValueFromResponseData(aSimResponseData, "sim_status");
            if (simStatus === "OK") {
                setTestStatusOk(simStatusTestStatus);
            } else if ((simStatus === "No SIM Detected") || (simStatus === "Unknown")) {
                setTestStatusCriticalWarning(simStatusTestStatus, "<p>No SIM card was detected.</p><p>Without a SIM card this device will not be able to make calls, recive calls or use mobile data.</p><p>If a SIM card is inserted then try the following:</p><p>* Remove the SIM card then reinsert it</p><p>* Try rebooting the device</p><p>* Try a replacement SIM card</p>");
            } else if (simStatus === "Network PIN Required") {
                setTestStatusWarning(simStatusTestStatus, "This phone is locked by the network operator and cannot be used with this SIM. It may be possible to unlock this device with a code from the network operator.");
            } else if (simStatus === "User PIN Required") {
                setTestStatusWarning(simStatusTestStatus, "This SIM is locked. You must enter your PIN before you can make calls, receive calls or use mobile data."); // TODO: provide more info
            } else if (simStatus === "PUK Required") {
                setTestStatusWarning(simStatusTestStatus, "This SIM is locked. You must enter your PUK before you can make calls, receive calls or use mobile data. Contact your network operator to get your PUK code.");
            } else {
                setTestStatusWarning(simStatusTestStatus, "SIM status is: " + simStatus);
            }
            
            // TODO: [later] check that MCC / MNC values are in the supported list
            
            // Get mobile network info
            getDataFromDevice("root.mobile_network.current_mobile_network", function (aRequestId, aSuccess, aCurrentMobileNetworkResponseData) {
                var flightModeValue,
                    serviceAvailabilityValue,
                    signalStrengthStringValue,
                    signalStrengthNumberValue,
                    selectionModeValue;
                
                flightModeValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, "flight_mode");
                if (flightModeValue === "No") {
                    setTestStatusOk(flightModeStatusTestStatus);
                } else {
                    addFixToTestStatus("Go to the flight mode setting screen", "", flightModeStatusTestStatus, function () {
                        startScreen("root.intents.screens", "android/settings/AIRPLANE_MODE_SETTINGS", flightModeStatusTestStatus);
                    });
                    
                    setTestStatusCriticalWarning(flightModeStatusTestStatus, "Flight mode is switched on. This will prevent you from using the phone for calls, text messages and mobile data.");
                }
                
                selectionModeValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, "selection_mode");
                serviceAvailabilityValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, "service_state");
                if (serviceAvailabilityValue === "Service Available") {
                    setTestStatusOk(mobileNetworkServiceAvailabilityTestStatus);
                } else {
                    if (selectionModeValue === "Manual") {
                        addFixToTestStatus("Go to the settings screen where can then check that the correct mobile network is selected.", "", mobileNetworkServiceAvailabilityTestStatus, function () {
                            // We're launching the data roaming settings because this usually has the network selection settings
                            // We're not using ACTION_NETWORK_OPERATOR_SETTINGS because it just hangs searching for devices on at least some devices
                                // See: https://code.google.com/p/android/issues/detail?id=205030
                            startScreen("root.intents.screens", "android/settings/DATA_ROAMING_SETTINGS", mobileNetworkServiceAvailabilityTestStatus);
                        });
                        
                        setTestStatusWarning(mobileNetworkServiceAvailabilityTestStatus, "<p>No mobile service is available.</p><p>This device is set to manual network selection (as opposed to automatic network selection). Check that the correct network is selected for this phone and SIM.</p><p>Alternatively, the device may be out of signal coverage for your network, or there may be an issue with your account or SIM.</p>");
                    } else {
                        setTestStatusWarning(mobileNetworkServiceAvailabilityTestStatus, "No mobile service is available. The device may be out of signal coverage for your network, or there may be an issue with your account or SIM.");
                    }
                }

                signalStrengthStringValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, "signal_strength");
                signalStrengthNumberValue = parseInt(signalStrengthStringValue, 10);
                if ((signalStrengthNumberValue < SIGNAL_STRENGTH_POOR_THRESHOLD) && (signalStrengthNumberValue >= 0)) {
                    setTestStatusWarning(mobileSignalStrengthTestStatus, "Signal strength is poor at this location. Consider using Wi-Fi for data, and Wi-Fi calling for voice calls."); // TODO: improve this text + only  + link to Wi-Fi calling (if supported)
                } else {
                    setTestStatusOk(mobileSignalStrengthTestStatus);
                }
            
                // Get mobile data info
                getDataFromDevice("root.mobile_network.mobile_data", function (aRequestId, aSuccess, aMobileDataResponseData) {
                    var mobileDataEnabledValue,
                        currentlyRoamingValue,
                        dataRoamingEnabledValue;
                    
                    mobileDataEnabledValue = MMRSUTILS.getValueFromResponseData(aMobileDataResponseData, "data_enabled");
                    trace("mobileDataEnabledValue: " + mobileDataEnabledValue);
                    if (mobileDataEnabledValue === "Mobile Data Disabled") {
                        setTestStatusCriticalWarning(mobileDataEnbaledTestStatus, "Mobile data is switched off. This will prevent you from browsing the Internet, checking email and using apps on your mobile network.");
                        addFixToTestStatus("Go to the settings screen where you can enable mobile data.", "", mobileDataEnbaledTestStatus, function () {
                            startScreen("root.intents.screens", "com/android/settings", mobileDataEnbaledTestStatus);
                        });
                    } else {
                        setTestStatusOk(mobileDataEnbaledTestStatus);
                    }

                    currentlyRoamingValue = MMRSUTILS.getValueFromResponseData(aCurrentMobileNetworkResponseData, "currently_roaming");
                    dataRoamingEnabledValue = MMRSUTILS.getValueFromResponseData(aMobileDataResponseData, "data_roaming_enabled");
                    if (currentlyRoamingValue === "Roaming") {
                        if (dataRoamingEnabledValue === "Enabled") {
                            setTestStatusOk(mobileDataRoamingTestStatus);
                        } else {
                            addFixToTestStatus("Go to the settings screen where you can enable data roaming. Note that if you do this, data roaming charges may apply.", "", mobileDataRoamingTestStatus, function () {
                                startScreen("root.intents.screens", "android/settings/DATA_ROAMING_SETTINGS", mobileDataRoamingTestStatus);
                            });
                            
                            setTestStatusCriticalWarning(mobileDataRoamingTestStatus, "This device is roaming, but the data roaming setting is switched off. This will prevent you from browsing the Internet, checking email and using apps on your mobile network.");
                        }
                    } else {
                        setTestStatusOk(mobileDataRoamingTestStatus);
                    }

                    // TODO: [later] : Does current network type support packet data?
                        // TODO: Is packet data network slow (GPRS or below)?
                        // mobileDataNetworkTypeTestStatus

                    // TODO: [later] : Is data connected? (Consider if Wi-Fi is connected, we expect data to be disconnected if Wi-Fi is not available)
                        // mobileDataStatusTestStatus
                });
            });
        });
        

        ///////////////////////////////////////////////////////////////////////////////////////
        // General Troubleshooting
        //

        generalGroup = addNewTestGroup($scope.quickCheckGroups, "General Troubleshooting");

        firmwareInfo = addNewInfoItemToGroup(generalGroup, "System Updates", "Check if the device is running the latest available software.");
        addFixToTestStatus("Launch the settings page to check for any available system updates", "", firmwareInfo, function () {
            startScreen("root.intents.screens", "android/settings/DEVICE_INFO_SETTINGS", firmwareInfo);
            /*
                var assistPages = aAssistPages,
                    firmwareUpdatePageData;

                firmwareUpdatePageData = assistPages.pages["mmrs.firmwareUpdateScreenAssist"];
                $scope.assistService = aAssistService;

                aAssistService.displayDialog('#mmrsAssistModal', firmwareUpdatePageData);
                */
        });
        
        wizardInfo = addNewInfoItemToGroup(generalGroup, "Troubleshooting Wizard", "The troubleshooting wizard can help identify and fix device problems");
        addFixToTestStatus("Launch the Troubleshooting Wizard", "", wizardInfo, function () {
            trace("Click here to launch the troubleshooting wizard");

            window.location.href = 'mmrswizard.html';
        });
        
        $scope.assistService = aAssistService;
        $scope.assistService.displayDialog("mmrs.troubleshootingWizardMenu");
    }]);
}());
