/*jslint browser: true*/
/*global $, jQuery, alert, angular, MMRSUTILS, MMRSCLIENTGATEWAY, MMRSCONFIG*/
var MMRSQUICKCHECKFACTORY = (function () {
    "use strict";
    
    var SIGNAL_STRENGTH_POOR_THRESHOLD = 20;

    function trace(aText) {
        MMRSUTILS.doTrace("mmrsquickcheckfactory", aText);
    }

    function note(aText) {
        MMRSUTILS.doTrace("mmrsquickcheckfactory", aText);
    }

    function warn(aText) {
        MMRSUTILS.doWarn("mmrsquickcheckfactory", aText);
    }

    function error(aText) {
        MMRSUTILS.doError("mmrsquickcheckfactory", aText);
    }
    
    angular.module('mmrsApp').factory('mmrsQuickCheckFactory', ['$rootScope', '$timeout', function ($rootScope, $timeout) {
        var mmrsQuickCheckFactoryInstance = null;
        
        // Add a possible fix to the test status
        // Used when a test gives a warning (or worse)
        // The fix could be something like "Change screen timeout to 30 seconds"
        // aFixFunction: callback function to apply the fix (called when the user selects the option to apply the fix). The callback function takes no parameters (assume that all data is already available to callback?)
        // aFixButtonText: the text displayed in the button (or link, or similar) that applies the fix
        // aFixTextDetails: extra info on what the fix will do (may be displayed as a hover item, or is a confirmation / info dialog)
        function addFixToTestStatus(aFixButtonText, aFixTextDetails, aTestStatus, aFixFunction) {
            var newFix = {
                "buttonText" : aFixButtonText,
                "textDetails" : aFixTextDetails,
                "fixFunction" : aFixFunction
            };

            aTestStatus.fixes.push(newFix);
        }
        
        function clearFixes(aTestStatus) {
            aTestStatus.fixes = [];
        }
        
        function sendCommentToLogger(aLevel, aTitle, aDetail) {
            trace("sendCommentToLogger, Level: " + aLevel + ", Title: " + aTitle + ", Detail: " + aDetail);

            var update = {
                action: "update",
                fields: [
                    {
                        action: "upate",
                        field:  "commentAddLevel",
                        value:  aLevel
                    },
                    {
                        action: "update",
                        field:  "commentAddTitle",
                        value:  aTitle
                    },
                    {
                        action: "upate",
                        field:  "commentAddDetail",
                        value:  aDetail
                    }
                ]
            },
                actionUrl = "root.logger.commentadd";

            trace("sendUpdateActionStatus, Sending Comment to device");
            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(actionUrl, update, function (aRequestId, aSuccess, aResponse) {
//                $scope.$apply(function () {
                trace("sendCommentToLogger, sendSetCommandToDevice: Success: " + aSuccess);
//                });
            });
        }
        
        function setTestStatusOk(aTestStatus) {
            trace(">> setTestStatusOk: " + aTestStatus.details);
            aTestStatus.status = "OK";
            aTestStatus.details = "No issues detected";
            trace("<< setTestStatusOk");
        }

        function setTestStatusWarning(aTestStatus, aWarningText) {
            aTestStatus.status = "WARNING";
            aTestStatus.details = aWarningText;

            trace("setTestStatusWarning, +++ Title: " + aTestStatus.title + ", Warning: " + aWarningText);

            sendCommentToLogger("warning", aTestStatus.title, aWarningText);
        }

        function setTestStatusCriticalWarning(aTestStatus, aWarningText) {
            aTestStatus.status = "CRITICAL_WARNING";
            aTestStatus.details = aWarningText;
            sendCommentToLogger("critical warning", aTestStatus.title, aWarningText);
        }
         
        function getDataFromDevice(aUrl, aResponseHandler) {
            trace("getDataFromDevice, Url: " + aUrl);

            MMRSCLIENTGATEWAY.sendGetCommandToDevice(aUrl, function (aRequestId, aSuccess, aResponseData) {
                $rootScope.$apply(function () { // TODO: get rid of $rootScope later when MMRSCLIENTGATEWAY is an angular service

                    var response = null;

                    if (aResponseData) {
                        response = aResponseData[0];
                    }

                    aResponseHandler(aRequestId, aSuccess, response);
                });
            });
        }
        
        // Utility function to set data on the device as part of an automated fix
        function applyFixToDevice(aUrl, aFixedData, aTestStatus) {
//--            trace(">> applyFixToDevice: aUrl: " + aUrl + " aFixedData: " + JSON.stringify(aFixedData));


//--            trace("<< applyFixToDevice");
        }

        // Start a settings screen and update the test status
        function startScreen(aScreenUrl, aScreenId, aTestStatus) {
            var screenAction = {
                action: "update",
                fields: [{
                    action: "update",
                    field: aScreenId,
                    value: "screen-start"
                }]
            };
            
            /*
            aTestStatus.status = "FIXING";
            aTestStatus.details = "Launching screen - Please wait";
            aTestStatus.fixes = [];
            */
            
            MMRSCLIENTGATEWAY.sendSetCommandToDevice(aScreenUrl, screenAction, function (aRequestId, aSuccess, aSetCommandStatus) {
                /*
                $rootScope.$apply(function () {
                    trace("startScreen: sendSetCommandToDevice: Success: " + aSuccess);

                    if (aSuccess) {
                        trace("Screen launched");
                        aTestStatus.status = "FIXED";
                        aTestStatus.details = "<p>Launched screen</p><p>Take a screenshot at any time to see what the user sees.</p>";
                        aTestStatus.fixes = [];
                    } else {
                        trace("Failed to launch screen");
                        aTestStatus.status = "FIX_FAILED";
                        aTestStatus.details = "Failed to launch screen";
                        aTestStatus.fixes = [];
                    }
                });
                */
            });
        }
        
        function setFixFailed(aTestStatus) {
            aTestStatus.status = "FIX_FAILED";
            aTestStatus.details = "Fix failed";
            aTestStatus.fixes = [];
        }
        
        function fixFieldValueOnDevice(aFetchedData, aUrl, aFieldName, aNewValue, aTestStatus) {
//--            trace(">> fixFieldValueOnDevice");
            var field = MMRSUTILS.getFieldUsingSettingItem(aFetchedData, aFieldName),
                fieldNumber,
                updateField,
                updateAction;
            
            if ((field === null) || (field === undefined)) {
                error("Unable to get field for: " + aFieldName);
                setFixFailed(aTestStatus);
            } else {
                fieldNumber = field.fieldid; // TODO: review comment: "fieldid" here vs. "field" in update action
                updateField = [{"action": "update",
                                "field": fieldNumber,
                                "value": aNewValue}];

                updateAction = {"action": "update",
                                "fields": updateField};
                
                aTestStatus.status = "FIXING";
                aTestStatus.details = "Applying fix - Please wait";
                aTestStatus.fixes = []; // clear fixes

                MMRSCLIENTGATEWAY.sendSetCommandToDevice(aUrl, updateAction, function (aRequestId, aSuccess, aSetCommandStatus) {
                    $rootScope.$apply(function () {
//--                        trace("Callback from: MMRSCLIENTGATEWAY.sendSetCommandToDevice: Success: " + aSuccess);
                        if (aSuccess) {
                            aTestStatus.status = "FIXED";
                            aTestStatus.details = "Fix applied";
                            aTestStatus.fixes = [];
                        } else {
                            setFixFailed(aTestStatus);
                        }
                    });
                });
            }
            
//--            trace("<< fixFieldValueOnDevice");
        }
        
        function fixFieldValueOnDeviceAsPercentage(aFetchedData, aUrl, aFieldName, aNewValue, aTestStatus) {
            var field = MMRSUTILS.getFieldUsingSettingItem(aFetchedData, aFieldName),
                absoluteValueNumber = NaN,
                matchingMaximumNumber = NaN,
                percentValueNumber = NaN,
                matchingField;
            
            if ((field === null) || (field === undefined)) {
                error("Unable to get field for: " + aFieldName);
            } else {
                percentValueNumber = Number(aNewValue);

                matchingField = MMRSUTILS.getFieldUsingSettingItem(aFetchedData, aFieldName);

                if ((matchingField !== null) && (matchingField !== undefined)) {
                    matchingMaximumNumber = Number(matchingField.fieldmaxvalue);
                    trace("Got maximum number: " + matchingMaximumNumber);
                }

                if ((isNaN(matchingMaximumNumber)) || (isNaN(percentValueNumber))) {
                    error("Can't set percentage value because there's no percent value, or no max value");
                    setFixFailed(aTestStatus);
                } else {
                    absoluteValueNumber = (percentValueNumber / 100) * matchingMaximumNumber;
                    absoluteValueNumber = Math.ceil(absoluteValueNumber);
                    if (absoluteValueNumber > matchingMaximumNumber) {
                        absoluteValueNumber = matchingMaximumNumber; // Just in case
                    }
                }
            }
            
            if (isNaN(absoluteValueNumber)) {
                setFixFailed(aTestStatus);
            } else {
                trace("Above to apply absolute value: " + absoluteValueNumber);
                fixFieldValueOnDevice(aFetchedData, aUrl, aFieldName, absoluteValueNumber, aTestStatus);
            }
        }
        
        // aCallbackFunction : (aRequestId, aSuccess, aResponseData)
        // Use this function as opposed to adding to the batcher directly, in order to avoid the possibility of execution outside of the Angular digest
        function addGetRequestToBatcher(aRequestUrl, aBatcher, aCallbackFunction) {
            aBatcher.addGetRequest(aRequestUrl, function (aRequestId, aSuccess, aResponseData) {
                $rootScope.$evalAsync(function () {
                    aCallbackFunction(aRequestId, aSuccess, aResponseData);
                }); // TODO: is there a way of making the batcher kick off a digest, if there is not one going?
            });
        }
        
        function createTestStatus(aTitle, aCallbacks) {
            var newTestStatus = {
                title : aTitle,
                status : mmrsQuickCheckFactoryInstance.NOT_STARTED,
                details : "Not started",
                fixes : [],
                callbacks : aCallbacks,
                failureAssistPage : false,
                failureAssistPageLinkText : null,
                failureAssistPageDetailsText : null,
                failureAssistPageId : null
            };

            return newTestStatus;
        }

        function createCheckingTestStatus(aTitle, aCallbacks) {
            var newTestStatus = createTestStatus(aTitle, aCallbacks);

            newTestStatus.status = "CHECKING";
            newTestStatus.details = "Please wait";

            return newTestStatus;
        }
                
        function createBatteryHealthTest(aBatcher, aCallbacks) {
            var batteryHealthTestStatus = createCheckingTestStatus("Battery Health", aCallbacks);
            addGetRequestToBatcher("root.device.battery", aBatcher, function (aRequestId, aSuccess, aResponseData) {

                trace("Battery Health: " + JSON.stringify(aResponseData));

                var batteryHealth = MMRSUTILS.getValueFromResponseData(aResponseData, "health");
                if (batteryHealth !== "Good") {
                    setTestStatusWarning(batteryHealthTestStatus, "Battery health warning: " + batteryHealth);
                    // TODO: add "fix" that launches other window to process device return (potentially pre-populated with IMEI + diagnostics data)
                } else {
                    setTestStatusOk(batteryHealthTestStatus);
                }
            });
            
            return batteryHealthTestStatus;
        }
        
        function createBatteryScreenTimeoutTest(aBatcher, aCallbacks) {
            var screenTimeoutPowerUsageTestStatus = createCheckingTestStatus("Screen Timeout", aCallbacks);

            addGetRequestToBatcher("root.screen.sleep", aBatcher, function (aRequestId, aSuccess, aResponseData) {

                var screenTimeoutValue = MMRSUTILS.getValueFromResponseData(aResponseData, "screenofftime");

                trace("Screen Timeout Handler, Value: " + screenTimeoutValue);

                if (screenTimeoutValue > 120) {
                    setTestStatusWarning(screenTimeoutPowerUsageTestStatus, "The screen timeout is set to more than than 120 seconds. Long screen timeouts may cause excessive battery drain.");

                    addFixToTestStatus("Change screen timeout to 30 seconds", "", screenTimeoutPowerUsageTestStatus, function () {
                        fixFieldValueOnDevice(aResponseData, "root.screen.sleep", "screenofftime", "30", screenTimeoutPowerUsageTestStatus);
                    });

                    addFixToTestStatus("Change screen timeout to 60 seconds", "", screenTimeoutPowerUsageTestStatus, function () {
                        fixFieldValueOnDevice(aResponseData, "root.screen.sleep", "screenofftime", "60", screenTimeoutPowerUsageTestStatus);
                    });

                    addFixToTestStatus("Change screen timeout to 120 seconds", "", screenTimeoutPowerUsageTestStatus, function () {
                        fixFieldValueOnDevice(aResponseData, "root.screen.sleep", "screenofftime", "120", screenTimeoutPowerUsageTestStatus);
                    });
                } else {
                    setTestStatusOk(screenTimeoutPowerUsageTestStatus);
                }
            });
            
            return screenTimeoutPowerUsageTestStatus;
        }
        
                    
        function createHardwareTest(aTestTitle, aHardwareTestId, aCallbacks) {
            var hardwareTestStatus;
            hardwareTestStatus = createTestStatus(aTestTitle, aCallbacks);
            addFixToTestStatus("Start test", "", hardwareTestStatus, function () {

                var screenAction = {
                        action: "update",
                        fields: [{
                            action: "update",
                            field: aHardwareTestId,
                            value: "screen-start"
                        }]
                    };

                hardwareTestStatus.status = mmrsQuickCheckFactoryInstance.RUNNING;
                hardwareTestStatus.details = "<p>Test Running</p>";
                clearFixes(hardwareTestStatus);

                MMRSCLIENTGATEWAY.sendSetCommandToDevice("root.blackbelt-analyst.screens", screenAction, function (aRequestId, aResult, aSetCommandStatus) {
                    trace("sendSetCommandToDevice complete");

                    $rootScope.$evalAsync(function (scope) {
                        var parsedSetCommandStatus = JSON.parse(aSetCommandStatus),
                            changedNodes,
                            fields,
                            firstField,
                            firstChangedNode,
                            testResult;
                        
                        trace("aSetCommandStatus: " + JSON.stringify(aSetCommandStatus));

                        changedNodes = parsedSetCommandStatus.changedNodes;

                        if (changedNodes) {
                            firstChangedNode = changedNodes[0];
                            if (firstChangedNode) {
                                fields = firstChangedNode.fields;
                                if (fields) {
                                    firstField = fields[0];
                                    if (firstField) {
                                        testResult = firstField.value;
                                        trace("testResult : " + testResult);

                                        if (testResult === "ACTION_FAILED") {
                                            hardwareTestStatus.status = mmrsQuickCheckFactoryInstance.FAILED;
                                            hardwareTestStatus.details = "Test Failed";
                                            sendCommentToLogger("critical warning", "Hardware test failed", aTestTitle);
                                            
                                            if (hardwareTestStatus.failureAssistPage) {
                                                trace("*** hardware test with failure assist page");
                                                // clearFixes(hardwareTestStatus);
                                                addFixToTestStatus(hardwareTestStatus.failureAssistPageLinkText, hardwareTestStatus.failureAssistPageDetailsText, hardwareTestStatus,
                                                    function () {
                                                        hardwareTestStatus.callbacks.displayDialog(hardwareTestStatus.failureAssistPageId);
                                                    });
                                            }
                                        } else if (testResult === "ACTION_COMPLETE") {
                                            hardwareTestStatus.status = mmrsQuickCheckFactoryInstance.PASSED;
                                            hardwareTestStatus.details = "Test Passed";
                                            sendCommentToLogger("info", "Hardware test passed", aTestTitle);
                                        }
                                    }
                                }
                            }
                        }
                    });
                });
            });
            
            return hardwareTestStatus;
        }
        
        function createBatteryAutoScreenBrightnessTest(aBatcher, aCallbacks) {
            var screenAutomaticBrightnessTestStatus;
            screenAutomaticBrightnessTestStatus = createCheckingTestStatus("Automatic Screen Brightness", aCallbacks);
            addGetRequestToBatcher("root.screen.brightness", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var screenBrightnessLevel = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnesslevel"),
                    automaticScreenBrightnessSetting = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnessmode");

                if (automaticScreenBrightnessSetting === "0") { // 1 == automatic, 0 == manual
                    setTestStatusWarning(screenAutomaticBrightnessTestStatus, "Screen brightness is set to \"manual\". This may cause excessive battery drain and make the screen harder to see in some conditions.");
                    addFixToTestStatus("Change screen brightness to \"Automatic\"", "", screenAutomaticBrightnessTestStatus, function () {
                        fixFieldValueOnDevice(aResponseData, "root.screen.brightness", "brightnessmode", 1, screenAutomaticBrightnessTestStatus); // TODO: use contant for 1 (automatic)
                    });
                } else {
                    setTestStatusOk(screenAutomaticBrightnessTestStatus);
                }
            });

            return screenAutomaticBrightnessTestStatus;
        }
        
        function createBatteryScreenBrightnessLevelTest(aBatcher, aCallbacks) {
            var screenBrightnessLevelTestStatus = createCheckingTestStatus("Screen Brightness Level", aCallbacks);
            
            addGetRequestToBatcher("root.screen.brightness", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var screenBrightnessLevel = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnesslevel"),
                    screenBrightnessPercent = (screenBrightnessLevel / 255) * 100, // TODO: use max value rather than hard coded 255
                    automaticScreenBrightnessSetting = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnessmode");

                if (screenBrightnessPercent > 75) {
                    setTestStatusWarning(screenBrightnessLevelTestStatus, "Screen brightness is set to " + Math.ceil(screenBrightnessPercent) + "%. This may cause excessive battery drain.");
                    addFixToTestStatus("Change screen brightness to 75%", "", screenBrightnessLevelTestStatus, function () {
                        fixFieldValueOnDevice(aResponseData, "root.screen.brightness", "brightnesslevel", "190", screenBrightnessLevelTestStatus); // TODO: use contant for 75% + use max value rather than 255
                    });
                } else {
                    setTestStatusOk(screenBrightnessLevelTestStatus);
                }
            });
                        
            return screenBrightnessLevelTestStatus;
        }
        
        function createAudioCallVolumeTest(aBatcher, aCallbacks) {
            var callVolumeTestStatus = createCheckingTestStatus("Call Volume", aCallbacks);
            addGetRequestToBatcher("root.sound.voice", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var callVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "voice"),
                    testFailed = false;

                if (callVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                    setTestStatusWarning(callVolumeTestStatus, "Call volume is set to 0% (mute). Calls will be silent.");
                    testFailed = true;
                } else if (callVolumePercent < 25) {
                    setTestStatusWarning(callVolumeTestStatus, "Call volume is set very low (" + callVolumePercent + "%). Calls may be difficult to hear at this volume.");
                    testFailed = true;
                } else {
                    setTestStatusOk(callVolumeTestStatus);
                }

                if (testFailed) {
                    addFixToTestStatus("Change call volume to 100%", "", callVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.voice", "voice", "100", callVolumeTestStatus);
                    });
                    addFixToTestStatus("Change call volume to 75%", "", callVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.voice", "voice", "75", callVolumeTestStatus);
                    });
                    addFixToTestStatus("Change call volume to 50%", "", callVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.voice", "voice", "50", callVolumeTestStatus);
                    });
                }
            });
            
            return callVolumeTestStatus;
        }
        
        function createAudioMediaVolumeTest(aBatcher, aCallbacks) {
            var musicVolumeTestStatus = createCheckingTestStatus("Music and Media Volume", aCallbacks);
            addGetRequestToBatcher("root.sound.music", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var musicVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "music"),
                    testFailed = false;

                if (musicVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                    setTestStatusWarning(musicVolumeTestStatus, "Music and media volume is set to 0% (mute). Music and other media audio will not play.");
                    testFailed = true;
                } else if (musicVolumePercent < 25) {
                    setTestStatusWarning(musicVolumeTestStatus, "Music and media volume is set very low (" + musicVolumePercent + "%). Music and other media may be difficult to hear at this volume.");
                    testFailed = true;
                } else {
                    setTestStatusOk(musicVolumeTestStatus);
                }

                if (testFailed) {
                    addFixToTestStatus("Change music and media volume to 100%", "", musicVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.music", "music", "100", musicVolumeTestStatus);
                    });
                    addFixToTestStatus("Change music and media volume to 75%", "", musicVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.music", "music", "75", musicVolumeTestStatus);
                    });
                    addFixToTestStatus("Change music and media volume to 50%", "", musicVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.music", "music", "50", musicVolumeTestStatus);
                    });
                }
            });
            
            return musicVolumeTestStatus;
        }
        
        function createAudioRingVolumeTest(aBatcher, aCallbacks) {
            var ringVolumeTestStatus = createCheckingTestStatus("Ring Volume", aCallbacks);
            addGetRequestToBatcher("root.sound.ring", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var ringVolumePercent = MMRSUTILS.getPercentValueFromResponseData(aResponseData, "ring"),
                    testFailed = false;

                if (ringVolumePercent === 0) { // TODO: the value seems to be set as a string, hence the "==" as opposed to "==="
                    setTestStatusWarning(ringVolumeTestStatus, "Ring volume is set to 0% (mute). Incoming calls will not ring.");
                    testFailed = true;
                } else if (ringVolumePercent < 25) {
                    setTestStatusWarning(ringVolumeTestStatus, "Ring volume is set very low (" + ringVolumePercent + "%). The ring for incoming calls may be difficult to hear at this volume.");
                    testFailed = true;
                } else {
                    setTestStatusOk(ringVolumeTestStatus);
                }

                if (testFailed) {
                    addFixToTestStatus("Change ring volume to 100%", "", ringVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.ring", "ring", "100", ringVolumeTestStatus);
                    });
                    addFixToTestStatus("Change ring volume to 75%", "", ringVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.ring", "ring", "75", ringVolumeTestStatus);
                    });
                    addFixToTestStatus("Change ring volume to 50%", "", ringVolumeTestStatus, function () {
                        fixFieldValueOnDeviceAsPercentage(aResponseData, "root.sound.ring", "ring", "50", ringVolumeTestStatus);
                    });
                }
            });
            
            return ringVolumeTestStatus;
        }
        
        function createMobileNetworkSimStatusTest(aBatcher, aCallbacks) {
            var simStatusTestStatus = createCheckingTestStatus("SIM", aCallbacks);
            
            addGetRequestToBatcher("root.mobile_network.sim", aBatcher, function (aRequestId, aSuccess, aSimResponseData) {
                // Test SIM status
                var simStatus = MMRSUTILS.getValueFromResponseData(aSimResponseData, "sim_status");
                if (simStatus === "OK") {
                    setTestStatusOk(simStatusTestStatus);
                } else if ((simStatus === "No SIM Detected") || (simStatus === "Unknown")) {
                    setTestStatusCriticalWarning(simStatusTestStatus, "<p>No SIM card was detected.</p><p>Without a SIM card this device will not be able to make calls, recive calls or use mobile data.</p><p>If a SIM card is inserted then try the following:</p><p>* Remove the SIM card then reinsert it</p><p>* Try rebooting the device</p><p>* Try a replacement SIM card</p>");
                } else if (simStatus === "Network PIN Required") {
                    setTestStatusWarning(simStatusTestStatus, "This phone is locked by the network operator and cannot be used with this SIM. It may be possible to unlock this device with a code from the network operator.");
                } else if (simStatus === "User PIN Required") {
                    setTestStatusWarning(simStatusTestStatus, "This SIM is locked. You must enter your PIN before you can make calls, receive calls or use mobile data."); // TODO: provide more info
                } else if (simStatus === "PUK Required") {
                    setTestStatusWarning(simStatusTestStatus, "This SIM is locked. You must enter your PUK before you can make calls, receive calls or use mobile data. Contact your network operator to get your PUK code.");
                } else {
                    setTestStatusWarning(simStatusTestStatus, "SIM status is: " + simStatus);
                }
            });
            
            return simStatusTestStatus;
        }
        
        function createMobileNetworkFlightModeTest(aBatcher, aCallbacks) {
            var flightModeStatusTestStatus = createCheckingTestStatus("Flight Mode Setting", aCallbacks);
            
            addGetRequestToBatcher("root.mobile_network.current_mobile_network", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var flightModeValue = MMRSUTILS.getValueFromResponseData(aResponseData, "flight_mode");
                if (flightModeValue === "No") {
                    setTestStatusOk(flightModeStatusTestStatus);
                } else {
                    addFixToTestStatus("Go to the flight mode setting screen", "", flightModeStatusTestStatus, function () {
                        startScreen("root.intents.screens", "android/settings/AIRPLANE_MODE_SETTINGS", flightModeStatusTestStatus);
                    });

                    setTestStatusCriticalWarning(flightModeStatusTestStatus, "Flight mode is switched on. This will prevent you from using the phone for calls, text messages and mobile data.");
                }
            });
            
            return flightModeStatusTestStatus;
        }
        
        function createMobileNetworkServiceTest(aBatcher, aCallbacks) {
            var mobileNetworkServiceAvailabilityTestStatus = createCheckingTestStatus("Mobile Service Availability", aCallbacks);
            
            addGetRequestToBatcher("root.mobile_network.current_mobile_network", aBatcher, function (aRequestId, aSuccess, aResponseData) {

                var selectionModeValue = MMRSUTILS.getValueFromResponseData(aResponseData, "selection_mode"),
                    serviceAvailabilityValue = MMRSUTILS.getValueFromResponseData(aResponseData, "service_state");
                
                if (serviceAvailabilityValue === "Service Available") {
                    setTestStatusOk(mobileNetworkServiceAvailabilityTestStatus);
                } else {
                    if (selectionModeValue === "Manual") {
                        addFixToTestStatus("Go to the settings screen where you can check that the correct mobile network is selected.", "", mobileNetworkServiceAvailabilityTestStatus, function () {
                            // We're launching the data roaming settings because this usually has the network selection settings
                            // We're not using ACTION_NETWORK_OPERATOR_SETTINGS because it just hangs searching for devices on at least some devices
                                // See: https://code.google.com/p/android/issues/detail?id=205030
                            startScreen("root.intents.screens", "android/settings/DATA_ROAMING_SETTINGS", mobileNetworkServiceAvailabilityTestStatus);
                        });

                        setTestStatusWarning(mobileNetworkServiceAvailabilityTestStatus, "<p>No mobile service is available.</p><p>This device is set to manual network selection (as opposed to automatic network selection). Check that the correct network is selected for this phone and SIM.</p><p>Alternatively, the device may be out of signal coverage for your network, or there may be an issue with your account or SIM.</p>");
                    } else {
                        setTestStatusWarning(mobileNetworkServiceAvailabilityTestStatus, "No mobile service is available. The device may be out of signal coverage for your network, or there may be an issue with your account or SIM.");
                    }
                }
                
            });
            
            return mobileNetworkServiceAvailabilityTestStatus;
        }
        
        function createMobileNetworkSignalStrengthTest(aBatcher, aCallbacks) {
            var mobileSignalStrengthTestStatus = createCheckingTestStatus("Mobile Signal Strength", aCallbacks);
            
            addGetRequestToBatcher("root.mobile_network.current_mobile_network", aBatcher, function (aRequestId, aSuccess, aResponseData) {

                var signalStrengthStringValue = MMRSUTILS.getValueFromResponseData(aResponseData, "signal_strength"),
                    signalStrengthNumberValue;
                
                signalStrengthNumberValue = parseInt(signalStrengthStringValue, 10);
                if ((signalStrengthNumberValue < SIGNAL_STRENGTH_POOR_THRESHOLD) && (signalStrengthNumberValue >= 0)) {
                    setTestStatusWarning(mobileSignalStrengthTestStatus, "Signal strength is poor at this location. Consider using Wi-Fi for data, and Wi-Fi calling for voice calls."); // TODO: improve this text + only  + link to Wi-Fi calling (if supported)
                } else {
                    setTestStatusOk(mobileSignalStrengthTestStatus);
                }
                
            });
            
            return mobileSignalStrengthTestStatus;
        }
        
        function createMobileNetworkMobileDataTest(aBatcher, aCallbacks) {
            var mobileDataEnbaledTestStatus = createCheckingTestStatus("Mobile Data Settings", aCallbacks);
            
            addGetRequestToBatcher("root.mobile_network.mobile_data", aBatcher, function (aRequestId, aSuccess, aResponseData) {

                var mobileDataEnabledValue = MMRSUTILS.getValueFromResponseData(aResponseData, "data_enabled");
                trace("mobileDataEnabledValue: " + mobileDataEnabledValue);
                if (mobileDataEnabledValue === "Mobile Data Disabled") {
                    setTestStatusCriticalWarning(mobileDataEnbaledTestStatus, "Mobile data is switched off. This will prevent you from browsing the Internet, checking email and using apps on your mobile network.");
                    addFixToTestStatus("Go to the settings screen where you can enable mobile data.", "", mobileDataEnbaledTestStatus, function () {
                        startScreen("root.intents.screens", "com/android/settings", mobileDataEnbaledTestStatus);
                    });
                } else {
                    setTestStatusOk(mobileDataEnbaledTestStatus);
                }
                
            });
            
            return mobileDataEnbaledTestStatus;
        }
        
        function createMobileNetworkMobileDataRoamingTest(aBatcher, aCallbacks) {
            var mobileDataRoamingTestStatus = createCheckingTestStatus("Mobile Data Roaming", aCallbacks),
                currentlyRoamingValue;
            
            addGetRequestToBatcher("root.mobile_network.current_mobile_network", aBatcher, function (aRequestId, aSuccess, aResponseData) {

                currentlyRoamingValue = MMRSUTILS.getValueFromResponseData(aResponseData, "currently_roaming");
                
            });
            
            addGetRequestToBatcher("root.mobile_network.mobile_data", aBatcher, function (aRequestId, aSuccess, aResponseData) {

                var dataRoamingEnabledValue = MMRSUTILS.getValueFromResponseData(aResponseData, "data_roaming_enabled");
                if (currentlyRoamingValue === "Roaming") {
                    if (dataRoamingEnabledValue === "Enabled") {
                        setTestStatusOk(mobileDataRoamingTestStatus);
                    } else {
                        addFixToTestStatus("Go to the settings screen where you can enable data roaming. Note that if you do this, data roaming charges may apply.", "", mobileDataRoamingTestStatus, function () {
                            startScreen("root.intents.screens", "android/settings/DATA_ROAMING_SETTINGS", mobileDataRoamingTestStatus);
                        });

                        setTestStatusCriticalWarning(mobileDataRoamingTestStatus, "This device is roaming, but the data roaming setting is switched off. This will prevent you from browsing the Internet, checking email and using apps on your mobile network.");
                    }
                } else {
                    setTestStatusOk(mobileDataRoamingTestStatus);
                }
                
            });
            
            return mobileDataRoamingTestStatus;
        }
        
        function createScreenLowBrightnessTest(aBatcher, aCallbacks) {
            var screenBrightnessLevelTestStatus = createCheckingTestStatus("Screen Brightness Level", aCallbacks);
            
            addGetRequestToBatcher("root.screen.brightness", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var screenBrightnessLevel = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnesslevel"),
                    screenBrightnessPercent = (screenBrightnessLevel / 255) * 100, // TODO: use max value rather than hard coded 255
                    automaticScreenBrightnessSetting = MMRSUTILS.getValueFromResponseData(aResponseData, "brightnessmode");

                if (screenBrightnessPercent < 50) {
                    setTestStatusWarning(screenBrightnessLevelTestStatus, "Screen brightness is set to " + Math.ceil(screenBrightnessPercent) + "%. This may make the display hard to read.");
                    addFixToTestStatus("Change screen brightness to 50%", "", screenBrightnessLevelTestStatus, function () {
                        fixFieldValueOnDevice(aResponseData, "root.screen.brightness", "brightnesslevel", "128", screenBrightnessLevelTestStatus); // TODO: use contant for 50% + use max value rather than128
                    });
                } else {
                    setTestStatusOk(screenBrightnessLevelTestStatus);
                }
            });
            
            return screenBrightnessLevelTestStatus;
        }
        
        function createScreenAutoRotateTest(aBatcher, aCallbacks) {
            var testStatus = createCheckingTestStatus("Auto-Rotate", aCallbacks);
            
            addGetRequestToBatcher("root.screen.autorotate", aBatcher, function (aRequestId, aSuccess, aResponseData) {
                var autorotateValue = MMRSUTILS.getValueFromResponseData(aResponseData, "autorotate");
                
                if (autorotateValue === "1") {
                    setTestStatusOk(testStatus);
                } else {
                    setTestStatusWarning(testStatus, "Autorotate is switched off. This will prevent the contents of the screen from updating when the device is rotated.");
                    addFixToTestStatus("Go to the screen settings on the device to change the screen rotation settings.", "", testStatus, function () {
                        startScreen("root.intents.screens", "android/settings/DISPLAY_SETTINGS", testStatus);
                    });
                }

                trace("*** autorotateValue : " + autorotateValue);
            });
            
            return testStatus;
        }

        mmrsQuickCheckFactoryInstance = {
            // Create a quick check and return the quick check status object
            // aCallbacks
                // aCallbacks.setDialogContent(aPageId) // Quick checks may launch a dialog page (if a dialog is already open then the page should be pushed to it)
            createQuickCheck : function (aTestId, aBatcher, aCallbacks) {
                if (aTestId === "tests.battery.health") {
                    return createBatteryHealthTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.battery.screen-auto-brightness") {
                    return createBatteryAutoScreenBrightnessTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.battery.screen-timeout") {
                    return createBatteryScreenTimeoutTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.battery.screen-brightness-level") {
                    return createBatteryScreenBrightnessLevelTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.audio.callVolume") {
                    return createAudioCallVolumeTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.audio.mediaVolume") {
                    return createAudioMediaVolumeTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.audio.ringVolume") {
                    return createAudioRingVolumeTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.mobilenetwork.sim") {
                    return createMobileNetworkSimStatusTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.mobilenetwork.flightmode") {
                    return createMobileNetworkFlightModeTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.mobilenetwork.service") {
                    return createMobileNetworkServiceTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.mobilenetwork.signalstrength") {
                    return createMobileNetworkSignalStrengthTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.mobilenetwork.data") {
                    return createMobileNetworkMobileDataTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.mobilenetwork.dataroaming") {
                    return createMobileNetworkMobileDataRoamingTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.screen.lowbrightness") {
                    return createScreenLowBrightnessTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.screen.autorotate") {
                    return createScreenAutoRotateTest(aBatcher, aCallbacks);
                } else if (aTestId === "tests.hardware.speaker") {
                    return createHardwareTest("Speaker Test", "speaker-test", aCallbacks);
                } else if (aTestId === "tests.hardware.screen.displayandtouch") {
                    return createHardwareTest("Screen Display and Touch Test", "screen-test", aCallbacks);
                } else if (aTestId === "tests.hardware.camera.camera") {
                    return createHardwareTest("Camera Test", "camera-test", aCallbacks);
                } else if (aTestId === "tests.hardware.microphone") {
                    return createHardwareTest("Microphone Test", "microphone-test", aCallbacks);
                } else if (aTestId === "tests.hardware.dimmer") {
                    return createHardwareTest("Dimmer Test", "dimming-test", aCallbacks);
                } else if (aTestId === "tests.hardware.lightsensor") {
                    return createHardwareTest("Light Sensor Test", "light-sensor-test", aCallbacks);
                } else if (aTestId === "tests.hardware.nfc") {
                    return createHardwareTest("NFC Test", "nfc-test", aCallbacks);
                } else if (aTestId === "tests.hardware.proximitysensor") {
                    return createHardwareTest("Proximity Sensor Test", "proximity-test", aCallbacks);
                } else if (aTestId === "tests.hardware.camera.flash") {
                    return createHardwareTest("Camera Flash Test", "camera-flash-test", aCallbacks);
                } else if (aTestId === "tests.hardware.vibra") {
                    return createHardwareTest("Vibra Test", "vibra-test", aCallbacks);
                } else if (aTestId === "tests.hardware.gps") {
                    return createHardwareTest("GPS Test", "gps-test", aCallbacks);
                } else if (aTestId === "tests.hardware.sensors") {
                    return createHardwareTest("All Sensors Test", "sensors-test", aCallbacks);
                } else if (aTestId === "tests.hardware.backlight") {
                    return createHardwareTest("Backlight Test", "backlight-test", aCallbacks);
                } else if (aTestId === "tests.hardware.call") {
                    return createHardwareTest("Call Test", "call-test", aCallbacks);
                } else if (aTestId === "tests.hardware.wifi") {
                    return createHardwareTest("Wi-Fi Test", "wifi-test", aCallbacks);
                } else if (aTestId === "tests.hardware.bluetooth") {
                    return createHardwareTest("Bluetooth Test", "bluetooth-test", aCallbacks);
                }
            },
            setFailureAssistPageLink : function (aTestStatus, aLinkText, aDetailsText, aPageId) {
                aTestStatus.failureAssistPage = true;
                aTestStatus.failureAssistPageLinkText = aLinkText;
                aTestStatus.failureAssistPageDetailsText = aDetailsText;
                aTestStatus.failureAssistPageId = aPageId;
            },
            NOT_STARTED : "NOT_RUN",
            RUNNING : "CHECKING",
            FAILED : "FAILED",
            PASSED : "PASSED"
        };
        
        return mmrsQuickCheckFactoryInstance;
    }]);
}());