var MMRSSPLASH = (function() {
"use strict"

var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();

window.onerror = function(aMessage) { error(aMessage); }

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }

var mRequestHostUrl       = null;
var mDoEvergreenSwap      = false;
var mHasDLCPlan           = null;
var mServiceId            = null;

var mTheApp = angular.module('app', []);

mTheApp.controller('controller', function($scope)
{
    var HOME_PAGE_URL = "mmrshome.html?background=backgroundwelcome3&hideNavBar=true&isHome=true&Home"; 
    /*  var HOME_PAGE_URL = "mmrshome.html?background=WelcomeBackground2&hideNavBar=true&isHome=true";*/

    var mStopHomePageLaunch = false;

    var ACCOUNT_START_DATE_PROPERTY     = "accountContractStartDate";
    var ACCOUNT_LEASE_PLAN_PROPERTY     = "accountOnLeasePlan";
    var ACCOUNT_ASSURE_PLAN_PROPERTY    = "accountOnAssurePlan";
    var ACCOUNT_PRODUCTS                = "accountProducts";
    var OPTION_ALLOW_SWAP               = "optionAllowSwap";
    var SELF_ASSESSMENT_REPORT_PROPERTY = "UserReportSwapAssessment";


    $scope.onMenuButtonClick = function(aAction)
    {
        trace("onMenuButtonClick, Action: " +aAction);

        DEVICEBRIDGE.onMenuButtonClick(aAction);
    }

    //====================================================================================
    // Which App Services to Try/Enable
    //====================================================================================
    //
    function fetchServices()
    {
        var servicesFile = DEVICEBRIDGE.getValue("services-file");

        if (servicesFile == null || servicesFile == "")
        {
            servicesFile = "servicesweb.json";
        }

        var url = mRequestHostUrl + servicesFile;

        //trace("fetchServices, Url: " +url);

        DEVICEBRIDGE.fetchUrl(url, onServicesReceived); 
    }    

    function onServicesReceived(aUrl, aSuccess, aHeader, aText)
    {
        trace("onServicesReceived, Success: " +aSuccess);

        if (! aSuccess)
        {
            error("onServicesReceived, Unable to fetch services");
            launchHomeScreen(false);
            return;
        }

        //trace("onServicesReceived, Services: " +aText);

        var services = null;

        try
        {
            services = JSON.parse(aText);
        }
        catch(err)
        {
            error("onServicesReceived, Json: " +err);
            services = null;
        }

        if (services == null)
        {
            launchHomeScreen(false);
            return;
        }

        saveServicesValues(services);

        var evergreenOption = getServiceOption(services, "doEvergreen");
        
        mDoEvergreenSwap = evergreenOption == "true";

        //trace("onServicesReceived, Do Evergreen: " +mDoEvergreenSwap);

        fetchAccountInformation(); // PROD

        //fetchFAQText(); //>> TEST
    }    

    //====================================================================================
    // Assessment Report
    //====================================================================================
    //
    function fetchAssessmentReport(aServiceNumber)
    {
        trace(">> fetchAssessmentReport"); //, Service Number: " +aServiceNumber);

        DEVICEBRIDGE.writePage(SELF_ASSESSMENT_REPORT_PROPERTY, "");

        NETSERVICES.fetchAssessmentReport(aServiceNumber, function(aSuccess, aResponse)
        {
            //trace("fetchAssessmentReport, Success: " +aSuccess+ ", Response: " +aResponse);

            var response = null;
            
            try
            {
                response = JSON.parse(aResponse);
            }
            catch(err)
            {
                trace("fetchAssessmentReport, Could not parse response: " +aResponse);
                response = null;
            }

            if (response == null || response.retrieveDeviceAssessmentResponse == null)
            {
                trace("fetchAssessmentReport, No report downloaded");
                launchHomeScreen(true);            
                return;
            }

            var report = response.retrieveDeviceAssessmentResponse;

            report.swapPrice = report.aux1;

            var reportStr = JSON.stringify(report);

            DEVICEBRIDGE.writePage(SELF_ASSESSMENT_REPORT_PROPERTY, reportStr);

            trace("fetchAssessmentReport, Report saved locally");

            launchHomeScreen(true);            
        });
    }    

    //====================================================================================
    // Fetch Account Information
    //====================================================================================
    //
    function fetchAccountInformation()
    {
        //trace("fetchAccountInformation");

        var accountInfoUrl = DEVICEBRIDGE.getValue("accountInfoUrl");

        if (accountInfoUrl == null || accountInfoUrl == "")
        {
            warn("fetchAccountInformation, No mobile account info URL in services");            
            launchHomeScreen(false);
            return;            
        }

        accountInfoUrl = getRequestUrl(accountInfoUrl);

        if (accountInfoUrl == null)
        {
            warn("fetchAccountInformation, Bad URL");
            launchHomeScreen(false);
            return;            
        }        

        var headers = {};

        headers["Accept"]        = DEVICEBRIDGE.getParam("accountAccept");
        headers["source-system"] = DEVICEBRIDGE.getValue("sourceSystem");

        //trace("fetchAccountInformation, Url: " +accountInfoUrl);

        var requestId = DEVICEBRIDGE.getCellularUrl(accountInfoUrl, headers, onAccountInfoReceived); 

        //var requestId = DEVICEBRIDGE.fetchUrl(accountInfoUrl, onAccountInfoReceived, headers);  // >>> TEST OVER WIFI

        //trace("fetchAccountInformation, Request Id: " +requestId);
    }

    function onAccountInfoReceived(aUrl, aSuccess, aHeader, aBody)
    {
        //trace("onAccountInfoReceived, Url:  " +aUrl+ ", Success: " +aSuccess);

        trace("onAccountInfoReceived, Success: " +aSuccess);

        if (!aSuccess)
        {
            launchHomeScreen(false);
            return;            
        }

        if (aHeader != null)
        {
            //trace("onAccountInfoReceived, Header: " +aHeader);            
        }

        if (aBody == null)
        {
            warn("onAccountInfoReceived, No response body");            
            return;
        }

        //trace("onAccountInfoReceived, aBody: " +aBody);            

        var responseBody = null;

        try
        {
            responseBody = JSON.parse(aBody);
        }
        catch (err)
        {
            error("onAccountInfoReceived, Exception: " +err);
            responseBody = null;
        }

        if (responseBody == null)
        {
            launchHomeScreen(false);
            return;
        }

        //trace("onAccountInfoReceived, parsed response");

        var success = saveAccountDetails(responseBody);

        if (! success)
        {
            trace("onAccountInfoReceived, Could not save account details");
            launchHomeScreen(false);
            return;
        }

        if (! mDoEvergreenSwap)
        {
            trace("onAccountInfoReceived, Evergreen swap option is disabled");            
            launchHomeScreen(false);
            return;
        }
 
        if (mHasDLCPlan != "true")
        {
            trace("onAccountInfoReceived, User is not on DLC (Evergreen) Plan");
            launchHomeScreen(false);
            return;            
        }

        fetchFAQText();
    }    

    function saveAccountDetails(aRecord)
    {
        var serviceDetails = aRecord["ServiceDetails"]
        
        if (serviceDetails == null)
        {
            error("saveAccountDetails, Could not get ServiceDetails");
            return false;
        }
 
        var service = getFieldValue(serviceDetails, "service");

        if (service == null)
        {
            error("saveAccountDetails, Could not get service");
            return false;
        }

        var contractDateStr = getContractDate(service);

        if (contractDateStr != null)
        {
            var contractDate = MMRSUTILS.parseDate(contractDateStr);

            if (contractDate != null && contractDate != "")
            {
                contractDate = Math.floor(contractDate / 1000);
                DEVICEBRIDGE.setValue(ACCOUNT_START_DATE_PROPERTY, contractDate);            
            }
        }

        var serviceId = getFieldValue(service, "serviceId");

        if (serviceId != null)
        {
            DEVICEBRIDGE.setValue("msisdn", serviceId);
            mServiceId = serviceId;            
        }

        var serviceAttribs = getFieldValue(service, "attributes");

        var networkIMEI= getAttributeValue(serviceAttribs, "networkIMEI");

        var imei = null;

        if (networkIMEI != null)
        {
            imei = MMRSUTILS.makeDeviceImei(networkIMEI);
        }

        if (imei != null)
        {
            DEVICEBRIDGE.setValue("imei", imei); 
        }

        var accountIMEI = getAttributeValue(serviceAttribs, "IMEI");

        if (accountIMEI != null && accountIMEI != "")
        {
            DEVICEBRIDGE.setValue("accountIMEI", accountIMEI);                        
        }

        var planAttribs = getFieldValue(service, "plan", "attributes");

        if (planAttribs == null)
        {
            trace("saveAccountDetails, No plan attributes");
            return false;
        }

        mHasDLCPlan = getAttributeValue(planAttribs, "DLC_PLAN");

        var products = getProductsList(service);

        var matchAssurePlan = matchProduct("MOBILE_SWAP_ASSURE", products);

        var leasePlan  = mHasDLCPlan     ? "true" : "false";
        var assurePlan = matchAssurePlan ? "true" : "false";

        DEVICEBRIDGE.setValue(ACCOUNT_LEASE_PLAN_PROPERTY,  leasePlan);   // Evergreen Lease Plan
        DEVICEBRIDGE.setValue(ACCOUNT_ASSURE_PLAN_PROPERTY, assurePlan);  // Assure Lease Option

        DEVICEBRIDGE.setValue(ACCOUNT_PRODUCTS, products);    

        return true;
    }

    //====================================================================================
    // Download Swap FAQs
    //====================================================================================
    //
    function fetchFAQText()
    {
        //trace("fetchFAQText");

        var url = DEVICEBRIDGE.getValue("swapFAQUrl");

        if (url == null || url == "")
        {
            warn("fetchFAQText, No FAQ Text URL");            
            launchHomeScreen(false);
            return;            
        }

        url = getRequestUrl(url);

        if (url == null)
        {
            warn("fetchFAQText, Bad URL");
            launchHomeScreen(false);
            return;            
        }        

        DEVICEBRIDGE.fetchUrl(url, onFAQTextReceived); 
    }

    function onFAQTextReceived(aUrl, aSuccess, aHeader, aText)
    {
        trace("onFAQTextReceived, Success: " +aSuccess);
        //trace("onFAQTextReceived, Text: " +aText);

        if (! aSuccess)
        {
            launchHomeScreen(false);
            return;
        }

        DEVICEBRIDGE.writePage("swap-faqs", aText);

        if (mServiceId == null)
        {            
            launchHomeScreen(false); // Should never be here
            return;
        }

        fetchAssessmentReport(mServiceId);
    }

    //====================================================================================
    // Utilities
    //====================================================================================
    //    
    function getFieldValue()
    {
        //trace("getFieldValue, Arguments: " +arguments.length);

        if (arguments.length < 2)
        {
            error("getFieldValue, Insufficient function arguments");
            return null;
        }

        var lastFieldIndex = arguments.length - 1;

        var record = arguments[0];

        var value = null;

        for (var index = 1; index < arguments.length; index++)
        {
            if (record == null)
            {
                break;                
            }

            var thisField = arguments[index];

            var thisValue = record[thisField];

            //trace("getFieldValue, Field: " +thisField+ ", Value: " +thisValue);

            if (index == lastFieldIndex)
            {
                value = thisValue;
                break;
            }

            record = thisValue;
        }

        return value;
    }

    function getAttributeValue(aAttributes, aName)
    {
        if (aAttributes == null)
        {
            return null;
        }

        var value = null;

        for (var index in aAttributes)
        {
            var thisAttrib = aAttributes[index];

            if (thisAttrib.name == aName)
            {
                value = thisAttrib.value;
                break;
            }
        }

        //trace("getAttributeValue, Name: " +aName+ ", Value: " +value);

        return value;
    }

    function getListValueForType(aItems, aType, aField)
    {
        if (aItems == null || aType == null)
        {
            return null;
        }

        var value = null;

        for (var index in aItems)
        {
            var thisItem = aItems[index];

            if (thisItem.type == aType)
            {
                value = thisItem[aField];
                break;
            }
        }

        //trace("getListValueForType, Type: " +aType+ ", Value: " +value);

        return value;
    }


    function getProductsList(aService)    
    {
        if (aService == null)
        {
            return "";
        }

        var products = aService.products;

        if (products == null)
        {
            trace("getProductsList, No products");
            return "";
        }

        var list = "";

        for (var index in products)
        {
            var thisProduct = products[index];

            var productType = thisProduct.productType;

            //trace("getProductsList, Type: " +productType);

            if (productType == null)
            {
                continue;
            }

            if (index > 0)
            {
                list += ",";
            }

            list += productType;
        }

        //trace("getProductsList: " +list);

        return list;
    }


    function getContractDate(aRecordList)
    {
        if (aRecordList == null)
        {
            trace("getContractDate, No service detaiuls");
            return null;
        }

        var contracts = getFieldValue(aRecordList, "contracts");        

        if (contracts == null)
        {
            trace("getContractDate, No contract information");
            return null;
        }

        var installedDate = getListValueForType(contracts, "PLAN", "installedDate");

        if (installedDate == null)
        {
            trace("getContractDate, No contract started date");            
            return null;
        }

        //trace("getContractDate, Contract started date: " +installedDate);   

        return installedDate;
    }


    function saveServicesValues(aServices)
    {
        if (aServices == null || aServices.values == null)
        {
            return;
        }

        var values = aServices.values;

        for (var thisKey in values)
        {
            var thisValue = values[thisKey];

            //trace("saveServicesValues, Key: " +thisKey+ ", Value: " +thisValue);
            DEVICEBRIDGE.setValue(thisKey, thisValue);
        }
    }

    function getServiceOption(aServices, aKey)
    {
        if (aServices == null || aServices.options == null)
        {
            return null;
        }

        var value = aServices.options[aKey];

        return value;
    }

    function getRequestUrl(aUrl)
    {
        if (aUrl == null)
        {
            warn("getRequestUrl, Url is null");            
            return null;
        }        

        if (aUrl.substr(0, 4) == "http")
        {
            return aUrl;
        }

        var relativeUrl = mRequestHostUrl + aUrl;

        return relativeUrl;
    }


    //====================================================================================
    // UI Interactions
    //====================================================================================
    //
    function launchHomeScreen(aDoSwap)
    {
        trace("launchHomeScreen, Do Swap: " +aDoSwap);
/*
        NETSERVICES.fetchAppleAccDetails("123", function(aSuccess, aResponse)
        {
            console.log("fetchAppleAccDetails, Success: " +aSuccess+ ", Response: " +aResponse);
        });
*/

        //showSpinner(false);

        var doSwap = aDoSwap ? "true" : "false";

        DEVICEBRIDGE.setValue(OPTION_ALLOW_SWAP, doSwap);

        if (mProgressTimer != null)
        {
            clearTimeout(mProgressTimer);
            mProgressTimer = null;
        }

        updateProgress(100);

        if (!mStopHomePageLaunch)
        {
            setTimeout(doLaunchHomeScreen, mProgressInterval*2);
        }
    }

    function doLaunchHomeScreen()
    {
         window.location.href = HOME_PAGE_URL;
    }

    function showSpinner(aIsActive)
    {
        DEVICEBRIDGE.showSpinner(aIsActive);
    }

    function startSpinner()
    {
        var spinnerHolderPos = MMRSUTILS.getElementPosition("spinnerholder");

        if (spinnerHolderPos == null)
        {
            spinnerHolder(true);
            return;
        }

        var spinnerY = spinnerHolderPos.y;

        //trace("startSpinner, Spinner: " +spinnerY);

        var screenSize = MMRSUTILS.getScreenDimensions();

        if (screenSize == null)
        {
            showSpinner(true);
            return;
        }

        var screenHeight = screenSize.h;

        //trace("startSpinner, Screen Height: " +screenHeight);

        if (spinnerY == 0 || screenHeight == 0)
        {
            showSpinner(true);            
            return;
        }

        var spinnerPos = spinnerY / screenHeight;

        //trace("startSpinner, Spinner Pos: " +spinnerPos);

        DEVICEBRIDGE.showSpinner(true, spinnerPos, false);
    }

    function matchProduct(aProduct, aProductList)
    {         
        if (aProduct == null || aProductList == null || aProductList == "")
        {
            trace("matchProduct, Bad parameters");            

            return false;
        }

        var list = aProductList.split(",");

        for (var index in list)
        {
            var thisProduct = list[index];

            var hasProduct = thisProduct.indexOf(aProduct) >= 0;

            if (hasProduct)
            {
                //trace("matchProduct, Found " +aProduct+ ", In List: " +aProductList);
                return true;
            }
        }

        //trace("matchProduct, NO MATCH: " +aProduct+ ", In List: " +aProductList);
        return false;
    }

    var mProgressBarPercent = 0;

    function updateProgress(aPercent)
    {
    //--    trace("updateProgress, aPercent: " +aPercent);

        var progressBar = document.getElementById('progressbar');

        if (progressBar == null)
        {
            error("updateProgress, Cannot find progress bar");
            return;        
        }

        var holderSize = MMRSUTILS.getElementPosition("progressholder");

        var width = Math.round(aPercent / 100 * holderSize.w);

        if (width > holderSize.w) width = holderSize.w;

        progressBar.style.maxWidth = width + "px";

        mProgressBarPercent = aPercent;
    }


    var mMaxProgressTime  = 15;
    var mProgressStep     = 100 / mMaxProgressTime;
    var mProgressInterval = 1000;
    var mProgressTimer    = null;

    function simulateProgress()
    {
        if (mProgressTimer == null)
        {
            return;
        }

        var progress = mProgressBarPercent + mProgressStep;

        if (progress > 100)
        {
            progress = 100;
        }

        updateProgress(progress);

        if (progress >= 100)
        {
            mProgressTimer = null;
            return;
        }

        mProgressTimer = setTimeout(simulateProgress, mProgressInterval);
    }

    //====================================================================================
    // MAIN Loader
    //====================================================================================
    //
    function doLoad()
    {   
        //trace(">> doLoad");

        //window.localStorage.clear();

        var options = MMRSUTILS.getOptions();

        mStopHomePageLaunch = options.stopLaunch == "true";

        DEVICEBRIDGE.setValue(OPTION_ALLOW_SWAP,            "false");  // Show swap option
        DEVICEBRIDGE.setValue(ACCOUNT_LEASE_PLAN_PROPERTY,  "false");  // Evergreen Lease Plan
        DEVICEBRIDGE.setValue(ACCOUNT_ASSURE_PLAN_PROPERTY, "false");  // Assure Lease Option

        //var imei = MMRSUTILS.makeDeviceImei("49015420323751");
        //var imei = MMRSUTILS.makeDeviceImei("35695906715276");
        //var imei = MMRSUTILS.makeDeviceImei("35329107867309");

        mRequestHostUrl = DEVICEBRIDGE.getRequestHostUrl() + "/" +DEVICEBRIDGE.getParam("deviceDir") + "/";

        //trace("doLoad, Request Url: " +mRequestHostUrl);
        //trace("doLoad, Relative Url: " +MMRSUTILS.getRequestUrl(mRequestHostUrl, "abc/def"));
        //trace("doLoad, Absolute Url: " +MMRSUTILS.getRequestUrl(mRequestHostUrl, "/abc/def"));

        //startSpinner();

        //trace("doLoad, Fetching Services");

        fetchServices(); //>>> PROD

        //fetchAccountInformation(); //>>> TEST

        //launchHomeScreen(true); //>>> TEST

        mProgressTimer = setTimeout(simulateProgress, 1000);
        
        //trace("<< doLoad");
    }

    doLoad();
});
    

var exports = {};

exports.name    = "mmrssplash";

exports.onLoad  = function() { onLoad(); }

return exports;

}());
