var MMRSSWAP = (function() {
"use strict"

var mIsNative  = MMRSUTILS.isNativeRun();

var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();

var GENERAL_HEADING_CHECKING         = "Checking upgrade offers";
var GENERAL_HEADING_ERROR            = "We were unable to check upgrade offers at the moment";

var SWAP_HEADING_NO_OFFERS           = "There are no upgrades available at the moment";
var SWAP_DETAIL_NO_OFFERS            = "Please read the following FAQs for more information.";

var SWAP_HEADING_UPGRADE_AVAILABLE   = "Phone is eligible for an upgrade";
var SWAP_DETAIL_UPGRADE_AVAILABLE    = "Upgrading your phone with Go Mobile Swap is easy.<BR>" +
                                       "Please read the following FAQs for more information.";

var SWAP_HEADING_UPGRADE_FUTURE      = "Your phone will be eligible for an upgrade on "; 
var SWAP_DETAIL_UPGRADE_FUTURE       = "Please read the following FAQs for more information.";

var ASSURE_HEADING_UPGRADE_FUTURE    = "Your phone will be eligible for an upgrade on ";
var ASSURE_DETAIL_UPGRADE_FUTURE     = "As you have Swap Assure, you are eligible for a replacement device any time.<BR>" +
                                       "Please read the following FAQs for more information.";

var ASSURE_HEADING_UPGRADE_AVAILABLE = "Phone is eligible for an upgrade";
var ASSURE_DETAIL_UPGRADE_AVAILABLE  = "Upgrading your phone with Go Mobile Swap is easy.<BR>" +
                                       "As you have Swap Assure, you are eligible for a replacement device any time.<BR>" +
                                       "Please read the following FAQs for more information.";

var CONDITION_GOOD_TEXT  = "Good Working Order";
var CONDITION_MINOR_TEXT = "Minor Damage";
var CONDITION_MAJOR_TEXT = "Major Damage";

var CONDITION_GOOD_INDEX  = 1;
var CONDITION_MINOR_INDEX = 2;
var CONDITION_MAJOR_INDEX = 3;


var DAY_NAMES   = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
var MONTH_NAMES = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

var mServiceNum = null;

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }

var mTheApp = angular.module('app', ['ngSanitize']);

mTheApp.controller('controller', function($scope, $sanitize)
{
    $scope.faqItems = [];

    $scope.swapStatus  = GENERAL_HEADING_CHECKING;
    $scope.swapPrice   = "";
    $scope.swapDetails = ""; 

    $scope.showSwapButtton = false;

    var mRequestHostUrl    = null;

    var mSwapStartDate     = null;
    var mSwapPeriodStarted = false;
    var mUserHasAssure     = false;

    $scope.onCheckClick = function()
    {
        trace("onCheckClick");

        window.location.href = "mmrsswapcheck.html";
    }

    $scope.onFAQItemClick = function(aItem)
    {
        //trace("onFAQItemClick, Title: " +aItem.title+ ", ID: " +aItem.id);

        var itemId = aItem.id;

        var appState = itemId == "assure" ? 
            MMRSAPPSTATE.EVERGREEN_VIEW_ASSURE_FAQ : MMRSAPPSTATE.EVERGREEN_VIEW_GENERAL_FAQ;

        var item = document.getElementById(itemId);

        if (item == null)
        {
            trace("onFAQItemClick, Could not find item: " +itemId);
            return;
        }

        var faqHeaderId = itemId + "-header";

        if (item.expanded)
        {
            aItem.expanded = false;
            collapseElement(item);
        }
        else
        {
            DEVICEBRIDGE.setAppState(appState);            

            aItem.expanded = true;
            item.addEventListener("webkitTransitionEnd", onExpandDone);

            expandElement(item);
        }
    }

    function onExpandDone(aEvent)
    {
        //trace("onExpandDone");

        var element = aEvent.target;

        if (element == null)
        {
            warn("onExpandDone, Event as no target");
            return;
        }

        var elementId = element.id;

        if (elementId == null)
        {
            warn("onExpandDone, Event as no element ID");
            return;
        }

        //trace("onExpandDone, Element Id: " +elementId);

        element.removeEventListener("webkitTransitionEnd", onExpandDone);

        var faqHeaderId = elementId + "-header";

        setTimeout(smoothScrollTo, 0, faqHeaderId);
    }

    function smoothScrollTo(aElementId)
    {
        //trace("smoothScrollTo");

        var body = document.body;
        var html = document.documentElement; 
        
        var windowHeight  = window.innerHeight;
        var contentHeight = Math.max(body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight);
        var currentScroll = window.pageYOffset;

        var elementRect = MMRSUTILS.getElementPosition(aElementId);

        if (elementRect == null)
        {
            trace("smoothScrollTo, No Element: " +aElementId);
            return;
        }

        var targetPos = elementRect.y;

        //trace("smoothScrollTo, Window Height:  " +windowHeight);
        //trace("smoothScrollTo, Content Height: " +contentHeight);        
        //trace("smoothScrollTo, Current Scroll: " +currentScroll);        
        //trace("smoothScrollTo, Target Pos:     " +targetPos);

        if (contentHeight <= windowHeight)
        {
            trace("smoothScrollTo, document shorter than window");
            return;
        }

        var scrollRequired = targetPos - currentScroll;

        //trace("smoothScrollTo, Scroll Required: " +scrollRequired);

        var targetTail = contentHeight - targetPos;

        //trace("smoothScrollTo, Target Tail: " +targetTail);

        if (targetTail < windowHeight)
        {
            scrollRequired = targetTail;
            //trace("smoothScrollTo, No enough tail - adjusting scroll: " +scrollRequired);
        }

        //window.scrollBy(0, scrollRequired);

        doSmoothScrollBy(scrollRequired);
    }    

    var mScrollJump  = 10;
    var mScrollTimer = 5;

    function doSmoothScrollBy(aRemainder)
    {
        if (aRemainder <= 0)
        {
            return;
        }

        var jump = Math.min(mScrollJump, aRemainder);

        window.scrollBy(0, jump);

        var remainder = aRemainder - jump;

        setTimeout(doSmoothScrollBy, mScrollTimer, remainder);
    }

    function collapseOtherItems(aItemId)
    {
        for (var index in $scope.faqItems)
        {
            var thisItem = $scope.faqItems[index];

            if (thisItem.id == aItemId)
            {
                continue;
            }

            var element = document.getElementById(thisItem.id);

            if (element == null)
            {
                warn("collapseOtherItems, Could not find element: " +thisItem.id);
                continue;
            }

            collapseElement(element);
        }
    }

    function expandElement(aElement)
    {
        var height = aElement.scrollHeight;

        aElement.style.maxHeight = height+ "px";

        aElement.expanded = true;
    }

    function collapseElement(aElement)
    {        
        aElement.expanded = false;
        aElement.style.maxHeight = "0px";

        aElement.expanded = false;
    }

    function readFAQItems()
    {
        $scope.faqItems = [];

        var faqData = DEVICEBRIDGE.readPage("swap-faqs");

        if (faqData == null || faqData == "")
        {
            error("readFAQItems, Could not read FAQ Items from storage");
            return false;
        }

        var faqItems = null;

        try
        {
            faqItems = JSON.parse(faqData);
        }
        catch(err)
        {
            error("readFAQItems, Could not parse FAQ data: " +err);
            return false;
        }

        var products = null;

        var productsList = DEVICEBRIDGE.getValue("accountProducts");

        //trace("readFAQItems, Products: " +productsList);

        if (productsList != null)
        {
            products = productsList.split(",");
        }

        var userHasAssure = false;

        for (var index in faqItems)
        {
            var thisItem = faqItems[index];

            var showFAQ = matchProductFAQ(products, thisItem);

            if (! showFAQ)
            {
                continue;
            }

            if (thisItem.id == "assure")
            {
                trace("readFAQItems,  User has assure");
                userHasAssure = true;
            }

            var newFAQ = {};

            newFAQ.expanded = false;
            newFAQ.title = thisItem.title;
            newFAQ.id    = thisItem.id;
            newFAQ.text  = thisItem.text.join(" ");

            //trace("readFAQItems, new FAQ Text" +newFAQ.text);
            
            $scope.faqItems.push(newFAQ);
        }

        return userHasAssure;
    }

    function matchProductFAQ(aProducts, aFAQ)
    {
        if (aFAQ.productTypes == null)
        {
            return true;
        }

        if (aProducts == null)
        {
            return false;
        }

        for (var faqIndex = 0; faqIndex < aFAQ.productTypes.length; faqIndex++)
        {
            var thisFAQType = aFAQ.productTypes[faqIndex];

            //trace("matchProductFAQ, FAQ: " +thisFAQType);

            for (var productIndex = 0; productIndex < aProducts.length; productIndex++)
            {
                var thisProductType = aProducts[productIndex];

                //trace("matchProductFAQ, Product: " +thisProductType);

                if (thisFAQType == thisProductType)
                {
                    //trace("matchProductFAQ, Product and FAQ Match");
                    return true;
                }

            }
        }

        trace("matchProductFAQ, No matches in Products and FAQs");
        //return false;
    }

    //====================================================================================
    // Display Swap Information
    //====================================================================================
    //

    function showSwapButtton(aVisible)
    {
        $scope.$applyAsync(function()
        {
            $scope.showSwapButtton = aVisible;
        });        
    }

    function setUpgradePrice(aPrice)
    {
        if (aPrice == null)
        {
            return;
        }

        $scope.$applyAsync(function()
        {
            $scope.swapPrice = aPrice;                
        });        
    }

    function setSwapStatus(aStatus, aDetails)
    {
        $scope.$applyAsync(function()
        {
            if (aStatus != null)
            {
                $scope.swapStatus = aStatus;                
            }

            if (aDetails != null)
            {
                $scope.swapDetails = aDetails;                                
            }
        });        
    }

    function getSwapDateString(aTime)
    {
        var swapDate = new Date(aTime);

        var dayOfWeek  = swapDate.getDay();
        var dayOfMonth = swapDate.getDate();    

        var month = swapDate.getMonth();
        var year  = swapDate.getFullYear();

        var dateStr = DAY_NAMES[dayOfWeek]+ ", " +dayOfMonth+ " " +MONTH_NAMES[month];
 
        var dateNow = new Date();
        var yearNow  = dateNow.getFullYear();

        if (year != yearNow)
        {
            dateStr += " " +year;
        }

        return dateStr;
    }

/*
    function checkAndDisplaySwapStatus(aSwapStart, aHasAssure)
    {
        if (aSwapStart == null)
        {
            setSwapStatus(GENERAL_HEADING_ERROR, "");
            return false;            
        }

        var timeNow = getTimeNow();

        if (timeNow < aSwapStart)
        {
            trace("checkAndDisplaySwapStatus, Swap period has not started");

            DEVICEBRIDGE.setAppState(MMRSAPPSTATE.EVERGREEN_OUTSIDE_PERIOD);            
            
            var heading = aHasAssure ? ASSURE_HEADING_UPGRADE_FUTURE : SWAP_HEADING_UPGRADE_FUTURE;
            var detail  = aHasAssure ? ASSURE_DETAIL_UPGRADE_FUTURE  : SWAP_DETAIL_UPGRADE_FUTURE;

            heading += getSwapDateString(aSwapStart);

            setSwapStatus(heading, detail);
            return aHasAssure;
        }

        trace("checkAndDisplaySwapStatus, We are within the swap period");

        DEVICEBRIDGE.setAppState(MMRSAPPSTATE.EVERGREEN_INSIDE_PERIOD);     

        var heading = aHasAssure ? ASSURE_HEADING_UPGRADE_AVAILABLE : SWAP_HEADING_UPGRADE_AVAILABLE;
        var detail  = aHasAssure ? ASSURE_DETAIL_UPGRADE_AVAILABLE  : SWAP_DETAIL_UPGRADE_AVAILABLE;

        setSwapStatus(heading, detail);   

        return true;
    }
*/

    function displaySwapStatus(aSwapStarted, aHasAssure)
    {
        // Swap period has not started
        //
        if (! aSwapStarted)
        {
            var heading = null;
            var detail  = null;

            if (aHasAssure)
            {
                // Early Assure
                ///
                heading = GENERAL_HEADING_CHECKING;
                detail  = ASSURE_DETAIL_UPGRADE_FUTURE;
            }               
            else 
            {
                // Evergreen Only but not in swap period
                //
                heading = SWAP_HEADING_UPGRADE_FUTURE + getSwapDateString(mSwapStartDate);
                detail  = SWAP_DETAIL_UPGRADE_FUTURE;
            }

            setSwapStatus(heading, detail);                    
            return;            
        }

        // We are in the swap period
        //
        var heading = GENERAL_HEADING_CHECKING;

        var detail  = aHasAssure ? ASSURE_DETAIL_UPGRADE_AVAILABLE  : SWAP_DETAIL_UPGRADE_AVAILABLE;            

        setSwapStatus(heading, detail);        
    }

    function getSwapStartDate()
    {
        var contractTime = DEVICEBRIDGE.getValue("accountContractStartDate");

        if (contractTime == null || contractTime == "")
        {
            error("getSwapStartDate, No contract start date stored");
            return null;
        }

        trace("getSwapStartDate, Contract Start: " +contractTime); 

        contractTime *= 1000;  // Convert from UNIX epoc time (seconds) to Javascript epoc time (milliseconds)

        var swapStartDate = new Date(contractTime);

        //trace("getSwapStartDate, Contract Date: " +swapStartDate.toISOString());

        swapStartDate.setFullYear(swapStartDate.getFullYear() + 1);        

        //trace("getSwapStartDate, Swap Start: " +swapStartDate.toISOString());

        var swapStartTime = swapStartDate.getTime();

        return swapStartTime;
    }


    //====================================================================================
    // Fetch Swap Information
    //====================================================================================
    //
    function fetchSwapInformation()
    {
        trace("fetchSwapInformation");

        var swapInfoUrl = DEVICEBRIDGE.getValue("swapInfoUrl");

        if (swapInfoUrl == null || swapInfoUrl == "")
        {
            trace("fetchSwapInformation, No swap info URL stored");
            return;
        }

        swapInfoUrl = getRequestUrl(swapInfoUrl);

        var headers = {};

        headers["source-system"] = DEVICEBRIDGE.getValue("sourceSystem");
        headers["Authorization"] = DEVICEBRIDGE.getParam("swapName");
        headers["Encoding"]      = "UTF-8";
        headers["Accept"]        = "application/json";

        swapInfoUrl += "?serviceType=MobileService&category=Lease+Plan&deviceStatus=Redemption";

        swapInfoUrl += "&serviceId=" +mServiceNum;

        trace("fetchAccountInformation, Swap Info Url: " +swapInfoUrl);

        var requestId = DEVICEBRIDGE.getCellularUrl(swapInfoUrl, headers, onSwapInfoReceived); 

        //var requestId = DEVICEBRIDGE.fetchUrl(swapInfoUrl, onSwapInfoReceived, headers);  // >>> TEST OVER WIFI

        //trace("fetchAccountInformation, Request Id: " +requestId);
    }

    function onSwapInfoReceived(aUrl, aSuccess, aHeader, aBody)
    {
        trace("onSwapInfoReceived, Url:  " +aUrl+ ", Success: " +aSuccess);

        showSpinner(false);

        /// aSuccess = false; //>>>> TEST

        if (!aSuccess)
        {
            setSwapStatus(GENERAL_HEADING_ERROR, null);
            return;            
        }

        if (aHeader != null)
        {
            //trace("onSwapInfoReceived, Header: " +aHeader);            
        }

        if (aBody == null)
        {
            trace("onSwapInfoReceived, No response body");  
            setSwapStatus(GENERAL_HEADING_ERROR, null);

            return;
        }

        trace("onSwapInfoReceived, aBody: " +aBody);            

        var response = null;

        try
        {
            response = JSON.parse(aBody);
        }
        catch (err)
        {
            error("onSwapInfoReceived, Exception: " +err);
            setSwapStatus(GENERAL_HEADING_ERROR, null);
            return;
        }

        trace("onSwapInfoReceived, parsed response");

        var offerRecord = getOffersRecord(response);

        if (offerRecord == null)
        {
            trace("onSwapInfoReceived, No offers");
            setSwapStatus(GENERAL_HEADING_ERROR, null);            
            return;            
        }

        var upgradePrices = getUpgradePrices(offerRecord);

        if (upgradePrices == null)
        {
            warn("onSwapInfoReceived, No upgrade prices");
        }

        var upgradeFrom = upgradePrices[1];

        setUpgradePrice("From $" +upgradeFrom);

        var pricesStr = JSON.stringify(upgradePrices);

        DEVICEBRIDGE.setValue("swapPrices", pricesStr);  

        // We should only be here is we are within the swap period or customer has assure cover
        //
        if (! mUserHasAssure)
        {
            setSwapStatus(SWAP_HEADING_UPGRADE_AVAILABLE, SWAP_DETAIL_UPGRADE_AVAILABLE);
        }
        else
        {
            var heading = mSwapPeriodStarted ? ASSURE_HEADING_UPGRADE_AVAILABLE : ASSURE_HEADING_UPGRADE_FUTURE;
            var detail  = mSwapPeriodStarted ? ASSURE_DETAIL_UPGRADE_AVAILABLE  : ASSURE_DETAIL_UPGRADE_FUTURE;            

            if (! mSwapPeriodStarted)
            {
                heading += getSwapDateString(mSwapStartDate);
            }

            setSwapStatus(heading, detail);
        }

        showSwapButtton(true);
    }    

    function getOffersRecord(aResponse)
    {
        if (aResponse == null)
        {
            warn("getOffersRecord, Null Response");            
            return null;
        }

        var swapRecord = aResponse;

        var offers = MMRSUTILS.getFieldValue(swapRecord, "data", "offers");

        if (offers == null)
        {
            warn("getOffersRecord, Response Swap Record has no Offers field");
            return null;
        }

        if (offers.length == null || offers.length == 0)
        {
            warn("getOffersRecord, Offers is not an array");  
            return null;
        }

        var offerRecord = offers[0];

        return offerRecord;
    }    

    function getUpgradePrices(aOfferRecord)
    {
        if (aOfferRecord == null)
        {
            warn("getUpgradePrices, Null Response");            
            return null;
        }

        var pricesTable = MMRSUTILS.getFieldValue(aOfferRecord, "fees", "redemption");

        if (pricesTable == null)
        {
            warn("getUpgradePrices, Could not get upgrade prices");  
            return null;            
        }

        var prices = [];

        prices[0] = "";

        for (var index in pricesTable)
        {
            var thisItem = pricesTable[index];

            var thisCondition = thisItem.condition;
            var thisAmount    = thisItem.amount;

            trace("getUpgradePrices, Condition: " +thisCondition+ ", Amount: " +thisAmount);

            if (thisCondition == null || thisAmount == null)
            {
                error("g, Bad upgrade prices");
                return null;
            }

            if (thisCondition == CONDITION_GOOD_TEXT)
            {
                prices[CONDITION_GOOD_INDEX] = thisAmount;
                continue;
            }

            if (thisCondition == CONDITION_MINOR_TEXT)
            {
                prices[CONDITION_MINOR_INDEX] = thisAmount;
                continue;
            }

            if (thisCondition == CONDITION_MAJOR_TEXT)
            {
                prices[CONDITION_MAJOR_INDEX] = thisAmount;
                continue;
            }            
        }

        if (prices.length != 4)
        {
            error("getUpgradePrices, Incomplete upgrade price table received");
            return null;            
        }

        trace("getUpgradePrices, Got prices array: " +JSON.stringify(prices));

        return prices;
    }


    function getTimeNow()
    {
        var dateNow = new Date();
        var timeNow = dateNow.getTime();     

        return timeNow;        
    }

    function getRequestUrl(aUrl)
    {
        if (aUrl == null)
        {
            warn("getRequestUrl, Url is null");            
            return null;
        }        

        if (aUrl.substr(0, 4) == "http")
        {
            return aUrl;
        }

        var relativeUrl = mRequestHostUrl + aUrl;

        return relativeUrl;
    }

    function showSpinner(aIsActive)
    {
        DEVICEBRIDGE.showSpinner(aIsActive);
    }

    function startSpinner()
    {
        var subHeadingPos = MMRSUTILS.getElementPosition("swapSubHeading");

        if (subHeadingPos == null)
        {
            showSpinner(true);
            return;
        }

        var spinnerY = subHeadingPos.y;

        //trace("startSpinner, Spinner: " +spinnerY);

        var screenSize = MMRSUTILS.getScreenDimensions();

        if (screenSize == null)
        {
            showSpinner(true);
            return;
        }

        var screenHeight = screenSize.h;

        //trace("startSpinner, Screen Height: " +screenHeight);

        if (spinnerY == 0 || screenHeight == 0)
        {
            showSpinner(true);            
            return;
        }

        var spinnerPos = spinnerY / screenHeight;

        //trace("startSpinner, Spinner Pos: " +spinnerPos);

        DEVICEBRIDGE.showSpinner(true, spinnerPos, true);
    }

    function doLoad()
    {   
        //trace(">> doLoad");        

        //var test = parseDate("2016-05-26");
        //var test = parseDate("2016-05-26T08:50:00+10:00");
        //var test = parseDate("2016-05-26T08:50:00Z");

        mRequestHostUrl = DEVICEBRIDGE.getRequestHostUrl() + "/" +DEVICEBRIDGE.getParam("deviceDir") + "/";

        $scope.$applyAsync(function()
        {
            mUserHasAssure = readFAQItems();

            mServiceNum = DEVICEBRIDGE.getValue("msisdn");

            if (mServiceNum == null || mServiceNum == "")
            {
                warn("doLoad, Could not retrieve service number");
                return;
            }            

            mSwapStartDate = getSwapStartDate();

            var timeNow = getTimeNow();

            mSwapPeriodStarted = timeNow >= mSwapStartDate;

            displaySwapStatus(mSwapPeriodStarted, mUserHasAssure);

            var swapAllowed = mSwapPeriodStarted || mUserHasAssure;

            if (swapAllowed)
            {
                setTimeout(startSpinner);
                fetchSwapInformation();              
            }

        });

        //trace("<< doLoad");
    }

    doLoad();
});
    
var mOptions = null;

var mMenuItems = [];

function onLoad()
{
    //trace(">> onLoad");

    //trace("<< onLoad");    
}

var exports = {};

exports.name    = "mmrsswap";

exports.onLoad  = function(aOptions) { onLoad(); }

return exports;

}());
