var MMRSSWAPCHECK = (function() {
	"use strict"


var mAngularScope = null;

var WORKFLOW_DIR         = "workflow/flows/swap";

var STATUS_ICON_INFO     = "images/statusInfo.png";
var STATUS_ICON_PASS     = "images/statusPass.png";
var STATUS_ICON_WARNING  = "images/statusWarning.png";
var STATUS_ICON_CRITICAL = "images/statusCritical.png"; 
var STATUS_ICON_FIXED    = "images/statusFixed.png";
var STATUS_ICON_PENDING  = "images/statusPending.gif";

var ACTION_COMPLETE = "ACTION_COMPLETE";
var ACTION_FAILED   = "ACTION_FAILED";
var ACTION_CANCELED = "ACTION_CANCELED";
var ACTION_ERROR    = "ACTION_ERROR";

var ACCOUNT_ASSURE_PLAN_PROPERTY = "accountOnAssurePlan";

var TEST_PENDING_TIME = 1200;

var mIsNativeRun = MMRSUTILS.isNativeRun();

var TEST_MODE = !mIsNativeRun;

var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();    

var SWAP_PRICES_PROPERTY  = "ExchangePrices";

var HW_TEST_SCREENS_TITLE     = "Test%20This%20Device";
var DEVICE_TEST_SCREENS_TITLE = "Device%20Condition";

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }

window.onerror = function(aText) { error(aText); }

var mTheApp = angular.module('mmrsApp', ['ngSanitize', 'ngAnimate']);

mTheApp.directive('script', WORKFLOWLOADER.init);

mTheApp.controller('mmrsController', function($scope)
{
    var mWorkflowConstants  = null;
    var mWorkflowProperties = null;    

    var mActiveTest         = null; 

    var mSwapPricesList     = null;

    var mPageClosing        = false;

    var mAssessmentReport   = null;

    var mNumDeviceLocks     = -1;

    $scope.workflowNamesFile = WORKFLOW_DIR + "/names.js";

    $scope.workflowNames = []   

    $scope.showNextButtton = false;


    $scope.onNextClick = function()
    {
        trace("onNextClick");

        DEVICEBRIDGE.showSpinner(true);

        fetchDeviceLocks(sendAssessmentReport);
    }

    function sendAssessmentReport()
    {
        mAssessmentReport = makeAssessmentReport();

        if (mAssessmentReport == null)
        {
            DEVICEBRIDGE.showSpinner(false);

            error("onNextClick, Could not make the assessment report");
            return;
        }

        console.log(mAssessmentReport);

        NETSERVICES.sendAssessmentReport(mAssessmentReport, onAssessmentReportSent);        
    }

    function onAssessmentReportSent(aSuccess)
    {
        trace("onAssessmentReportSent, Success: " +aSuccess);

        DEVICEBRIDGE.showSpinner(false);

        if (!aSuccess)
        {
            warn("onAssessmentReportSent, Could not send report");
            window.location.href = "workflow/workflow.html?workflow=flows/swap/index.js&page=save-failed&title=Device%20Assessment";
            return;
        }

        var reportStr = JSON.stringify(mAssessmentReport);

        DEVICEBRIDGE.writePage(mWorkflowProperties.UserReportSwapAssessment, reportStr);

        window.location.href = "workflow/workflow.html?workflow=flows/swap/index.js&page=swap-price&title=Device%20Assessment";
    }


    // Method for testing simulated back events on web browser.
    //

    var mTestsStarted = false;

    $scope.onSimulatedClear = function()
    {
        trace("+++ onSimulatedClear");   

        if (!TEST_MODE)
        {
            return;
        }


        clearSwapChecks();
    }    

    $scope.onSimulatedComplete = function()
    {
        trace("+++ onSimulatedComplete");   

        if (!TEST_MODE)
        {
            return;
        }
    
        completeSwapChecks();
    }    


    $scope.onReTestClick = function(aTest)
    {
        trace(">> onReTestClick: " +aTest.title+ ", Test: " +aTest.test+ ", Field: " +aTest.field);  

        if (!TEST_MODE)
        {
            return;
        }

        startTest(aTest);
    }


    function makeTests()
    {
        var tests = 
        [
            /* How to launch native native tests. Although all tests for Evergreen are HTML based.
            {
                title:      "Screen display",
                test:       "root.hardware-tests.screens",
                field:      "screen-pixel-test",
                failed:     false,            
                iconStatus: null,
                property:   mWorkflowProperties.PixelStatus
            },
            {
                title:      "Screen touch",
                test:       "root.hardware-tests.screens",
                field:      "screen-drawing-test",
                failed:     false,            
                iconStatus: null,
                property:   mWorkflowProperties.TouchStatus                           
            },
            */
            {
                title:      "Screen Pixel Test",
                url:        "workflow/workflow.html?workflow=flows/swap/index.js&page=test-pixels-launch&finishOnPush=true&title=" +HW_TEST_SCREENS_TITLE,
                failed:     false,            
                iconStatus: null,
                property:   mWorkflowProperties.PixelStatus,
                testIndex:  1
            },
            {
                title:      "Screen Drawing Test",
                url1:       "workflow/workflow.html?workflow=flows/swap/index.js&page=test-touch-launch&finishOnPush=true&title=" +HW_TEST_SCREENS_TITLE,
                url:        "workflow/workflow.html?workflow=flows/swap/index.js&page=test-touch-check&title=" +HW_TEST_SCREENS_TITLE,
                failed:     false,            
                iconStatus: null,
                property:   mWorkflowProperties.TouchStatus,
                testIndex:  2                
            },
            {
                title:      "Screen Damage",
                url:        "workflow/workflow.html?workflow=flows/swap/index.js&page=screen-check&title=" +DEVICE_TEST_SCREENS_TITLE,
                failed:     false,            
                iconStatus: null,
                property:   mWorkflowProperties.ScreenStatus,
                testIndex:  3
            },
            {
                title:      "Body Damage",
                url:        "workflow/workflow.html?workflow=flows/swap/index.js&page=case-check&title=" +DEVICE_TEST_SCREENS_TITLE,          
                failed:     false,            
                iconStatus: null,
                property:   mWorkflowProperties.CaseStatus,
                testIndex:  4                
            },
            {
                title:      "Liquid Damage",
                url:        "workflow/workflow.html?workflow=flows/swap/index.js&page=liquid-check&title=" +DEVICE_TEST_SCREENS_TITLE,
                failed:     false,            
                iconStatus: null,
                iconRedo:   null,
                property:   mWorkflowProperties.LiquidStatus,
                testIndex:  5                
            }
        ]; 

        return tests;  
    }


    function showTestStatusPending(aTest)
    {
        trace("showTestStatusPending, Title " + aTest.title);

        aTest.failed     = false;             
        aTest.iconStatus = STATUS_ICON_PENDING;
    }        

    function showTestStatusPassed(aTest)
    {
        trace("showTestStatusPassed, Title " + aTest.title);

        aTest.failed     = false;             
        aTest.iconStatus = STATUS_ICON_PASS;
    }    

    function showTestStatusFailed(aTest)
    {
        trace("showTestStatusFailed, Title: " +aTest.title);

        aTest.failed     = true;     

        aTest.iconStatus = STATUS_ICON_CRITICAL;     

        sendCommentToLogger(aTest.title, "failed");
    }

    function showTestStatusCancelled(aTest)
    {
        trace("showTestStatusCancelled, +++ Title: " +aTest.title);

        aTest.failed     = true;     

        aTest.iconStatus = STATUS_ICON_CRITICAL;   

        sendCommentToLogger(aTest.title, "cancelled");
    }


    //=========================================================================
    // HTML Based Tests
    //=========================================================================
    //
    function doSetDeviceConditionTestResult(aTestItem, aLevel)
    {
        trace(">> doSetDeviceConditionTestResult");

        if (aLevel == 1)
        {
            showTestStatusPassed(aTestItem);
            return;            
        }

        showTestStatusFailed(aTestItem);
    }

    function setDeviceConditionTestResult(aTest, aLevel)
    {
        $scope.$applyAsync(function()
        {
            doSetDeviceConditionTestResult(aTest, aLevel);
        });         
    }
 

    //=========================================================================
    //
    //=========================================================================
    //
    function sendCommentToLogger(aTestTitle, aTestStatus)
    {
        trace("sendCommentToLogger, Title: " +aTestTitle+ ", Status: " + aTestStatus);
    }

    //=========================================================================
    //  Test Launching
    //=========================================================================
    //    
    var mNextTestIndex = 0;
    var mNumTests      = -1;

    function startTest(aTest)
    {
        if (mPageClosing)
        {
            trace("startTest, Ignoring timer as page is closing");
            return;
        }

        mActiveTest = aTest;
/*
        if (aTest.test != null)
        {
            startHarwdareTest(aTest);
            return;
        }
*/

        if (aTest.url != null)
        {
            trace("startTest, Test: " +aTest.title+ ", Property: " +aTest.property+ ", Url: " +aTest.url);

            var status = JSON.stringify(
            {
                details: mWorkflowConstants.CONDITION_CANCELLED_TEXT,
                level:   mWorkflowConstants.CONDITION_CANCELLED_LEVEL
            });

            DEVICEBRIDGE.setValue(aTest.property, status);                 
            
            window.location.href = aTest.url;
            return;
        }

        mActiveTest = null;

        error("startTest, Unknown Test Type: " +aTest.title);
    }

    function startNextTest()
    {
        trace("startNextTest, Next: " +mNextTestIndex+ ", Tests: " +mNumTests); 
     
        if (mNextTestIndex >= mNumTests)
        {
            trace("startNextTest, All tests complete: " +mNextTestIndex);
            completeSwapChecks();
            return;
        }

        var thisTest = $scope.tests[mNextTestIndex];

        $scope.$applyAsync(function()
        {
            showTestStatusPending(thisTest);
        });

        setTimeout(startTest, TEST_PENDING_TIME, thisTest);

        mNextTestIndex++;
    }

    function completeSwapChecks()
    {  
        setTimeout(showNextButtton, 500, true);        
    }


    //=========================================================================
    // Assessment Report Generation
    //=========================================================================
    // 
    function makeAssessmentReport()
    {  
        var testResults    = [];
        var maxDamageLevel = 0;

        var liquidDamage   = false;

        for (var index in $scope.tests)
        {
            var thisTest      = $scope.tests[index];
            var thisProperty  = thisTest.property;
            var thisTestIndex = thisTest.testIndex;

            var thisStatusStr = DEVICEBRIDGE.getValue(thisProperty);

            if (thisStatusStr == "" || thisStatusStr == null)
            {
                error("makeAssessmentReport, No status for: " +thisProperty);
                return;
            }

            var thisStatus = JSON.parse(thisStatusStr);

            //thisStatus.title = thisTest.title;

            trace("makeAssessmentReport, Property: " +thisProperty+ ", Status: " +thisStatusStr);

            var thisCostLevel     = thisStatus.level - 0;
            var thisReportedLevel = thisStatus.reported;

            if (thisReportedLevel == null)
            {
                thisReportedLevel = thisCostLevel;
            }

            var thisTestResult = 
            {
                test  : thisTestIndex,
                level : thisReportedLevel - 0
            }

            testResults.push(thisTestResult);

            if (thisCostLevel > maxDamageLevel)
            {
                maxDamageLevel = thisCostLevel;
            }

            if (thisProperty == mWorkflowProperties.LiquidStatus && thisCostLevel > mWorkflowConstants.CONDITION_GOOD)
            {
                liquidDamage = true;
            }
        }   

        // For simplicity if a test was canceled it will have a "damage" level of 4.
        // So we reduce it to MAJOR DAMAGE (3) to work out the swap price etc.
        //
        if (maxDamageLevel > mWorkflowConstants.CONDITION_MAJOR_DAMAGE) 
        {
            maxDamageLevel = mWorkflowConstants.CONDITION_MAJOR_DAMAGE;
        }

        var swapPrice = mSwapPricesList[maxDamageLevel];

        var report = createAssessmentPayload(maxDamageLevel, liquidDamage, swapPrice, testResults, mNumDeviceLocks);

        return report;
    }


    function createAssessmentPayload(aMaxDamageLevel, aHasLiquidDamage, aSwapPrice, aTestResults, aNumLocks)
    {
        trace("createAssessmentPayload, Level: " +aMaxDamageLevel+ ", Liquid: " +aHasLiquidDamage+ ", Price: " +aSwapPrice+ ", Locks: " +aNumLocks);

        var DEVICE_CONDITION_MAP = {};
        
        DEVICE_CONDITION_MAP[mWorkflowConstants.CONDITION_GOOD]         = 0;
        DEVICE_CONDITION_MAP[mWorkflowConstants.CONDITION_MINOR_DAMAGE] = 2;
        DEVICE_CONDITION_MAP[mWorkflowConstants.CONDITION_MAJOR_DAMAGE] = 1;

        var report = WORKFLOWPAYLOAD.initCreateAssessmentReport();

        console.log("createAssessmentPayload, Report: ", report);

        report.source          = "TDC App";

        report.aux1            = aSwapPrice; // Temporary field until one is allocated in API spec
        report.swapPrice       = aSwapPrice;

        report.aux2            = makeResultsString(aTestResults);

        report.deviceOs        = MMRSUTILS.isAndroidRun() ? "android" : "ios";

        report.deviceCondition = DEVICE_CONDITION_MAP[aMaxDamageLevel];

        report.liquidDamage    = aHasLiquidDamage ? "Yes" : "No";

        var deviceIMEI = DEVICEBRIDGE.getValue("imei");

        if (deviceIMEI != null && deviceIMEI.length > 0)
        {
            report.imei = deviceIMEI;
        }

        var imeiMatch = true;

        var accountIMEI = DEVICEBRIDGE.getValue("accountIMEI");

        if (accountIMEI != null && accountIMEI.length > 0)
        {
            imeiMatch = deviceIMEI == accountIMEI;
        }

        report.isImeiMatch = imeiMatch ? "true" : "false";

        var msisdn = DEVICEBRIDGE.getValue("msisdn");

        if (msisdn != null && msisdn.length > 0)
        {
            report.msisdn = msisdn;
        }

        var onAssurePlan = DEVICEBRIDGE.getValue(ACCOUNT_ASSURE_PLAN_PROPERTY) == "true";

        report.assure = onAssurePlan ? "Y" : "N";

        var checkImei = MMRSUTILS.makeDeviceImei(deviceIMEI);

        var imeiValid = checkImei != null && checkImei == deviceIMEI;

        report.isImeiValid = imeiValid ? "Y" : "N";

        if (aNumLocks < 0) // Unable to get lock status
        {
            report.deviceFreeOfLocks = "No";       // TODO: Clarify what should be sent when we are unable to
            //report.deviceLock        = null;     // get the lock status (Apple only probably)
        }
        else if (aNumLocks > 0) // We have locks
        {
            report.deviceFreeOfLocks = "No";
            report.deviceLock        = "Active";
        }
        else // aNumLocks == 0 - No Locks
        {
            report.deviceFreeOfLocks = "Yes";
            report.deviceLock        = "Non-Active";
        }

        return report;
    }

    function makeResultsString(aList)
    {
      if (aList == null)
      {
          warn("makeResultsString, No results saved");
          return "";
      }

      var results = ["A"]; // First letter is the version

      for (var index in aList)
      {
          var thisTest = aList[index];

          var testIndex = thisTest.test;
          var level     = thisTest.level;

          if (level <= 0)
          {
              error("makeResultsString, Invalid level: " +level+ ", Test Index: " +testIndex);
              level = 0;
          }

          results[testIndex] = level;
      } 

      var resultsStr = results.join("");

      trace("makeResultsString, String: " +resultsStr);

      return resultsStr;     
    }

    function clearSwapChecks()
    {
        trace("+++ clearSwapChecks");

        for (var index in $scope.tests)
        {
            var thisProperty = $scope.tests[index].property;

            DEVICEBRIDGE.setValue(thisProperty, "");

            //trace("initSwapChecks, Index: " +index+ ", Property: " +thisProperty);
        }     

        DEVICEBRIDGE.writePage(mWorkflowProperties.UserReportSwapAssessment, "");
    }


    //=========================================================================
    // Device Lock Checks
    //=========================================================================
    // 
    function fetchDeviceLocks(aCallback) 
    {
        var isAndroid = MMRSUTILS.isAndroidRun();

        if (isAndroid)
        {
            fetchAndroidDeviceLocks(aCallback);
        }
        else
        {
            fetchAppleDeviceLocks(aCallback);
        }
    }

    function fetchAndroidDeviceLocks(aCallback)
    {
        trace("fetchAndroidDeviceLocks");

        var requestId = BRIDGE.sendGetCommand("root.accounts.locks", function(aRequestId, aSuccess, aResponse)
        {
            trace("fetchAndroidDeviceLocks, Success: " +aSuccess+ ", Response: " +JSON.stringify(aResponse));
            
            mNumDeviceLocks = extractLockCount(aSuccess, aResponse);

            trace("fetchAndroidDeviceLocks, Lock Count: " +mNumDeviceLocks);

            aCallback();

        });
    }

    function fetchAppleDeviceLocks(aCallback)
    {
        trace("fetchAppleDeviceLocks");

        mNumDeviceLocks = -1;

        var deviceIMEI = DEVICEBRIDGE.getValue("imei");

        ///deviceIMEI = "353041096143471"; // Test: Valid iPhone IMEI 

        if (deviceIMEI == null || deviceIMEI == "")
        {
            trace("fetchAppleDeviceLocks, No IMEI set");

            aCallback();
            return;
        }

        NETSERVICES.fetchAppleAccDetails(deviceIMEI, function(aSuccess, aResponse)
        {
            trace("fetchAppleDeviceLocks, Success: " +aSuccess+ ", Response: " +JSON.stringify(aResponse));

            if (!aSuccess)
            {
                aCallback();
                return;
            }

            var isLocked = aResponse["fmip-enabled"];

            if (isLocked != null)
            {
                mNumDeviceLocks = isLocked ? 1 : 0;
            }

            trace("fetchAppleDeviceLocks, Device Locks: " +mNumDeviceLocks);

            aCallback();
        });

    }

    function extractLockCount(aSuccess, aResponse)
    {
        trace("extractLockCount");

        if (!aSuccess)
        {
            trace("extractLockCount, Request Failed");
            return -1;
        }

        var response = null;

        try
        {
            response = JSON.parse(aResponse);
        }
        catch (err)
        {
            error("extractLockCount, Could not parse response: " +err);
            response = null;
        }

        if (response == null)
        {
            return -1;
        }

        var lockCount = MMRSUTILS.getDeviceSettingValue(response, "count");

        lockCount = lockCount - 0; // Convert to a number

        trace("extractLockCount, Lock Count: " +lockCount);

        return lockCount;
    }

    //=========================================================================
    // For HTML based tests we receive a onPageEvent('back') when the test 
    // page has dissappeared (popped from the view / activity stack)
    //=========================================================================
    //    
    $scope.onPageEvent = function(aEvent)
    {
        trace("+++ onPageEvent, Event: " +aEvent);   

        if (aEvent == "close")
        {
            mPageClosing = true;
            return;
        }

        if (aEvent != "back")
        {
            return;
        }

        if (mActiveTest == null)
        {
            warn("onPageEvent, No active test");
            return;
        }

        var statusStr = DEVICEBRIDGE.getValue(mActiveTest.property);

        if (statusStr == null || statusStr == "")
        {
            warn("+++ onPageEvent, Test did not set a result: " +mActiveTest.property);
            return;
        }

        var status = JSON.parse(statusStr);

        trace("+++ onPageEvent, Test: " +mActiveTest.property+ ", Status: " +statusStr);   

        var damageLevel = status.level;            

        setDeviceConditionTestResult(mActiveTest, damageLevel);

        startNextTest();
    }

    //=========================================================================
    // Create the assessment report
    //=========================================================================
    //       
    

    //=========================================================================
    //  Once we have the workflow names loaded we are good to start
    //=========================================================================
    //
    $scope.onWorkflowNamesLoaded = function()
    {
        trace("onWorkflowNamesLoaded");

        if (mWorkflowProperties != null)
        {
            return;
        }

        var workflowNames = WORKFLOWLOADER.makeNamesDictionary($scope.workflowNames); 

        mWorkflowConstants  =  workflowNames.constants;      
        mWorkflowProperties =  workflowNames.properties;     

        NETSERVICES.init(workflowNames); 

        if (mIsNativeRun)
        {
            //window.localStorage.clear();
        }

        var swapPrices = DEVICEBRIDGE.getValue(SWAP_PRICES_PROPERTY);

        if (swapPrices == null)
        {
            error("onWorkflowNamesLoaded, No swap prices defined");
            return;
        }

        trace("onWorkflowNamesLoaded, Swap Prices: " +swapPrices);

        mSwapPricesList = JSON.parse(swapPrices);

        $scope.tests = makeTests();

        mNumTests = $scope.tests.length;     

        DEVICEBRIDGE.setAppState(MMRSAPPSTATE.EVERGREEN_START_PREASSESMENT);

        if (TEST_MODE)
        {
            trace("onWorkflowNamesLoaded, Test Mode");

            var leftButton  = document.getElementById("testButtonLeft");
            var rightButton = document.getElementById("testButtonRight");

            leftButton.style.display  = "block";
            rightButton.style.display = "block";
            return;
        }

        clearSwapChecks();
        startNextTest();
    }

    ///////////////////////////////////////////////////////////////////////////////////////  

    mAngularScope = $scope;

    function getSecondsNow()
    {
        var dateNow    = new Date();
        var timeNow    = dateNow.getTime();     
        var secondsNow = Math.floor(timeNow / 1000);

        return secondsNow;        
    }

    // Once all the tests are complete we show the next navigation button to 
    // move on to the summary page.
    //
    function showNextButtton(aVisible)
    {
        $scope.$applyAsync(function()
        {
            $scope.showNextButtton = aVisible;
        });          
    }    

});

var exports = {};

exports.name = "mmrsswapcheck";

exports.onPageEvent = function(aEvent) { return mAngularScope.onPageEvent(aEvent); }

return exports;

}());



