var MMRSSPLASH = (function() {
"use strict"

var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();

window.onerror = function(aMessage) { error(aMessage); }

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }

var mRequestHostUrl       = null;

var mTheApp = angular.module('app', []);

mTheApp.controller('controller', function($scope)
{
    $scope.mspStatus    = "--";
    $scope.customerType = "--";
    $scope.dlcPlan      = "--";
    $scope.msisdn       = "--";
    $scope.contractIMEI = "--";
    $scope.networkIMEI  = "--";
    $scope.products     = "--";

    $scope.onMenuButtonClick = function(aAction)
    {
        trace("onMenuButtonClick, Action: " +aAction);

        DEVICEBRIDGE.onMenuButtonClick(aAction);
    }

    //====================================================================================
    // Which App Services to Try/Enable
    //====================================================================================
    //
    function fetchServices()
    {
        var servicesFile = DEVICEBRIDGE.getValue("services-file");

        if (servicesFile == null || servicesFile == "")
        {
            servicesFile = "servicesweb.json";
        }

        var url = mRequestHostUrl + servicesFile;

        //trace("fetchServices, Url: " +url);

        DEVICEBRIDGE.fetchUrl(url, displayServicesReceived); 
    }    

    function displayServicesReceived(aUrl, aSuccess, aHeader, aText)
    {
        $scope.$applyAsync(function()
        {
            onServicesReceived(aUrl, aSuccess, aHeader, aText)
        });          
    }

    function onServicesReceived(aUrl, aSuccess, aHeader, aText)
    {
        trace("onServicesReceived, Success: " +aSuccess);

        if (! aSuccess)
        {
            error("onServicesReceived, Unable to fetch services");
            $scope.mspStatus = "Unable to fetch Config from MM";
            return;
        }

        //trace("onServicesReceived, Services: " +aText);

        var services = null;

        try
        {
            services = JSON.parse(aText);
        }
        catch(err)
        {
            error("onServicesReceived, Json: " +err);
            services = null;
        }

        if (services == null)
        {
            $scope.mspStatus = "Bad services record from MM";
            return;
        }

        saveServicesValues(services);
        
        fetchAccountInformation(); 

    }    

    //====================================================================================
    // Fetch Account Information
    //====================================================================================
    //
    function fetchAccountInformation()
    {
        //trace("fetchAccountInformation");

        var accountInfoUrl = DEVICEBRIDGE.getValue("accountInfoUrl");

        if (accountInfoUrl == null || accountInfoUrl == "")
        {
            warn("fetchAccountInformation, No mobile account info URL in services");            
            return;            
        }

        accountInfoUrl = getRequestUrl(accountInfoUrl);

        if (accountInfoUrl == null)
        {
            warn("fetchAccountInformation, Bad URL");
            return;            
        }        

        var headers = {};

        headers["Accept"]        = DEVICEBRIDGE.getParam("accountAccept");
        headers["source-system"] = DEVICEBRIDGE.getValue("sourceSystem");

        var requestId = DEVICEBRIDGE.getCellularUrl(accountInfoUrl, headers, displayAccountInfoReceived); 

        //var requestId = DEVICEBRIDGE.fetchUrl(accountInfoUrl, onAccountInfoReceived, headers);  // TEST OVER WIFI

        //trace("fetchAccountInformation, Request Id: " +requestId);
    }

    function displayAccountInfoReceived(aUrl, aSuccess, aHeader, aBody)
    {
        $scope.$applyAsync(function()
        {
            onAccountInfoReceived(aUrl, aSuccess, aHeader, aBody);
        });           
    }


    function onAccountInfoReceived(aUrl, aSuccess, aHeader, aBody)
    {
        //trace("onAccountInfoReceived, Url:  " +aUrl+ ", Success: " +aSuccess);
        trace("onAccountInfoReceived, Success: " +aSuccess);

        if (!aSuccess)
        {
            $scope.mspStatus = "Request Failed";
            return;            
        }

        if (aHeader != null)
        {
            //trace("onAccountInfoReceived, Header: " +aHeader);            
        }

        if (aBody == null)
        {
            warn("onAccountInfoReceived, No response body");  
            $scope.mspStatus = "No payload received";
            return;
        }

        trace("onAccountInfoReceived, aBody: " +aBody);            

        var responseBody = null;

        try
        {
            responseBody = JSON.parse(aBody);
        }
        catch (err)
        {
            error("onAccountInfoReceived, Exception: " +err);
            $scope.mspStatus = "Bad payload received";            
            responseBody = null;
        }

        if (responseBody == null)
        {
            return;
        }

        //trace("onAccountInfoReceived, parsed response");

        var success = saveAccountDetails(responseBody);
    }    

    function saveAccountDetails(aRecord)
    {
        var serviceDetails = aRecord["ServiceDetails"]
        
        if (serviceDetails == null)
        {
            error("saveAccountDetails, Could not get ServiceDetails");
            $scope.mspStatus = "No ServiceDetails received";            
            return false;
        }

        var service = getFieldValue(serviceDetails, "service");

        if (service == null)
        {
            error("saveAccountDetails, Could not get service");
            $scope.mspStatus = "No service record received";                        
            return false;
        }

        $scope.msisdn = getFieldValue(service, "serviceId");

        var serviceAttribs  = getFieldValue(service, "attributes");

        $scope.contractIMEI = getAttributeValue(serviceAttribs, "IMEI");
        
        var networkIMEI     = getAttributeValue(serviceAttribs, "networkIMEI");

        var imei = null;

        if (networkIMEI != null)
        {
            imei = MMRSUTILS.makeDeviceImei(networkIMEI);
            $scope.networkIMEI = imei;
        }
    
        var customerType = getFieldValue(serviceDetails, "customer", "customerType");

        trace("saveAccountDetails, Customer Type: " +customerType);
        $scope.customerType = customerType;

        var planAttribs = getFieldValue(service, "plan", "attributes");

        if (planAttribs == null)
        {
            trace("saveAccountDetails, No plan attributes");
            $scope.mspStatus = "No plan attributes received";                                    
            return false;
        }

        var hasDLCPlan = getAttributeValue(planAttribs, "DLC_PLAN");

        $scope.dlcPlan = hasDLCPlan ? "Yes" : "No";
        
        var products = getProductsList(service);

        $scope.products = products.replace(/,/g, ", ");

        $scope.mspStatus = "Response Received";

        return true;        
    }

  

    //====================================================================================
    // Utilities
    //====================================================================================
    //    
    function getFieldValue()
    {
        if (arguments.length < 2)
        {
            error("getFieldValue, Insufficient function arguments");
            return null;
        }

        var lastFieldIndex = arguments.length - 1;

        var record = arguments[0];

        var value = null;

        for (var index = 1; index < arguments.length; index++)
        {
            if (record == null)
            {
                break;                
            }

            var thisField = arguments[index];

            var thisValue = record[thisField];

            if (index == lastFieldIndex)
            {
                value = thisValue;
                break;
            }

            record = thisValue;
        }

        return value;
    }


    function getAttributeValue(aAttributes, aName)
    {
        if (aAttributes == null)
        {
            return null;
        }

        var value = null;

        for (var index in aAttributes)
        {
            var thisAttrib = aAttributes[index];

            if (thisAttrib.name == aName)
            {
                value = thisAttrib.value;
                break;
            }
        }

        //trace("getAttributeValue, Name: " +aName+ ", Value: " +value);

        return value;
    }

    function getProductsList(aService)    
    {
        if (aService == null)
        {
            return "";
        }

        var products = aService.products;

        if (products == null)
        {
            trace("getProductsList, No products");
            return "";
        }

        var list = "";

        for (var index in products)
        {
            var thisProduct = products[index];

            var productType = thisProduct.productType;

            //trace("getProductsList, Type: " +productType);

            if (productType == null)
            {
                continue;
            }

            if (index > 0)
            {
                list += ",";
            }

            list += productType;
        }

        //trace("getProductsList: " +list);

        return list;
    }

    function saveServicesValues(aServices)
    {
        if (aServices == null || aServices.values == null)
        {
            return;
        }

        var values = aServices.values;

        for (var thisKey in values)
        {
            var thisValue = values[thisKey];

            //trace("saveServicesValues, Key: " +thisKey+ ", Value: " +thisValue);
            DEVICEBRIDGE.setValue(thisKey, thisValue);
        }
    }

    function getServiceOption(aServices, aKey)
    {
        if (aServices == null || aServices.options == null)
        {
            return null;
        }

        var value = aServices.options[aKey];

        return value;
    }

    function getRequestUrl(aUrl)
    {
        if (aUrl == null)
        {
            warn("getRequestUrl, Url is null");            
            return null;
        }        

        if (aUrl.substr(0, 4) == "http")
        {
            return aUrl;
        }

        var relativeUrl = mRequestHostUrl + aUrl;

        return relativeUrl;
    }


    //====================================================================================
    // UI Interactions
    //====================================================================================
    //


    //====================================================================================
    // MAIN Loader
    //====================================================================================
    //
    function doLoad()
    {   
        trace(">> doLoad");

        mRequestHostUrl = DEVICEBRIDGE.getRequestHostUrl() + "/" +DEVICEBRIDGE.getParam("deviceDir") + "/";

        fetchServices(); //>>> PROD

        trace("<< doLoad");
    }

    doLoad();
});
    

var exports = {};

exports.name    = "msptest";

exports.onLoad  = function() { onLoad(); }

return exports;

}());
