var FAQ_SECTION_DIRECTIVE = (function()
{
  "use strict";

    mTheApp.directive('itemFaqSection',  FAQSectionDirective);

    var ASSURE_FAQ_ID = "assure";  // Use in Omniture logging to identify which FAQ section has be openned.

    var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();

    var mScope = null;
    var mClickCallback = null;

    function FAQSectionDirective()
    {
        return {
            restrict: 'E',
            
            scope: {
                item: '=',
                onFaqItemSelected: '='
            },
            
            replace: true,
            
            templateUrl: 'directives/item-faq-section.html',

            link: function(scope, element, attrs)
            {      
                //console.log("FAQSectionDirective - link");

                mScope = scope;
                mClickCallback = scope.onFaqItemSelected;

                scope.item.matchedFAQs = filterFAQItems(scope.item.match, scope.item.items);

                scope.onFAQItemClick = function(aFAQItem)
                {
                    scope.$applyAsync(function()
                    {              
                        processFAQItemClick(aFAQItem);
                        //scope.onFaqItemSelected(aFAQItem, itemExpanding);
                    });
                }   
            }
        }
    }

    function filterFAQItems(aProducts, aFAQItems)
    {
        var faqs = [];

        for (var index in aFAQItems)
        {
            var thisFAQ = aFAQItems[index];

            console.log("Item-Id: " +thisFAQ.id);

            var match = matchProductFAQ(aProducts, thisFAQ);

            if (!match)
            {
                continue;
            }

            console.log("filterFAQItems, Match: " +thisFAQ.id);

            var newFAQ = {};

            newFAQ.id    = thisFAQ.id;
            newFAQ.title = thisFAQ.title;
            newFAQ.onClick = thisFAQ.onClick;

            if (thisFAQ.text instanceof Array)
            {
                newFAQ.text = thisFAQ.text.join(" ");
            }
            else
            {
                newFAQ.text = thisFAQ.text;
            }

            faqs.push(newFAQ);
        }

        return faqs;
    }

    function matchProductFAQ(aProducts, aFAQ)
    {
        if (aFAQ.productTypes == null)
        {
            return true;
        }

        if (aProducts == null)
        {
            return false;
        }

        for (var faqIndex = 0; faqIndex < aFAQ.productTypes.length; faqIndex++)
        {
            var thisFAQType = aFAQ.productTypes[faqIndex];

            //trace("matchProductFAQ, FAQ: " +thisFAQType);

            for (var productIndex = 0; productIndex < aProducts.length; productIndex++)
            {
                var thisProductType = aProducts[productIndex];

                //trace("matchProductFAQ, Product: " +thisProductType);

                if (thisFAQType == thisProductType)
                {
                    //trace("matchProductFAQ, Product and FAQ Match");
                    return true;
                }

            }
        }

        trace("matchProductFAQ, No matches in Products and FAQs");
        return false;
    }

    function processFAQItemClick(aItem)
    {
        console.log("doFAQItemClick: " +aItem.id);
      
        var itemId = aItem.id;

        var element = document.getElementById(itemId);

        if (element == null)
        {
            trace("onFAQItemClick, Could not find item: " +itemId);
            return;
        }

        element.item = aItem;

        var faqHeaderId = itemId + "-header";

        if (aItem.expanded)
        {
            aItem.expanded = false
            collapseElement(element);
        }
        else
        {
            aItem.expanded = true;

            var appState = aItem.id == ASSURE_FAQ_ID ? 
                MMRSAPPSTATE.EVERGREEN_VIEW_ASSURE_FAQ :
                MMRSAPPSTATE.EVERGREEN_VIEW_GENERAL_FAQ;

            DEVICEBRIDGE.setAppState(appState);

            element.addEventListener("webkitTransitionEnd", onExpandDone);

            expandElement(element);
        }      
    }

    function expandElement(aElement)
    {
        var height = aElement.scrollHeight;

        aElement.style.maxHeight = height+ "px";

        aElement.expanded = true;
    }

    function collapseElement(aElement)
    {        
        aElement.style.maxHeight = "0px";

        aElement.expanded = false;
    }  


    function onExpandDone(aEvent)
    {
        console.log("onExpandDone");

        var element = aEvent.target;

        if (element == null)
        {
            console.log("onExpandDone, Event as no target");
            return;
        }

        var elementId = element.id;

        if (elementId == null)
        {
            console.log("onExpandDone, Event as no element ID");
            return;
        }

        var item = element.item;

        //console.log("onExpandDone, Element Id: " +elementId);

        element.removeEventListener("webkitTransitionEnd", onExpandDone);

        if (mClickCallback != null)
        {
            var headerElementId = elementId+ "-header";
            setTimeout(mClickCallback, 0, item, true, headerElementId);
        }
    }

    function trace(aText)
    {
        console.log(aText);
    }

})();
