var TEST_TOUCH_DIRECTIVE = (function()
{
  "use strict";

  mTheApp.directive('itemTestTouch',  TouchTestDirective);

  var mIsNative = MMRSUTILS.isNativeRun();

  var DEVICEBRIDGE = MMRSUTILS.getDeviceBridge();

  var LINE_WIDTH = 2;

  // Multi-Tap Catcher
  //
  //
  var mMultiTapStartTime = -1;
  var mMultiTapCount     =  0;
  var mMultiTapTrigger   =  3;  
  var mMultiTapPeriod    = 1500; //1.5 seconds

  var mIdlePeriod        = 2000;
  var mIdlePeriodTimer   = null;

  // Drawing Coordinates
  //
  //
  var mContext   = null;
  var mPrevX     = null;
  var mPrevY     = null;
  var mMouseDown = false;
  var mTestComplete = false;

  var mScope      = null;
  var mTestArea   = null;
  var mTestCanvas = null;

  function TouchTestDirective() {
    return {
        restrict: 'E',
        scope: {
            item: '=',
            onTestComplete: '='
        },
        replace: true,
        templateUrl: 'directives/item-test-touch.html',

        link: function(scope, element, attrs)
        {      
            console.log("TouchTestDirective - link");

            mScope = scope;

            mTestArea = getTestArea(element);

            if (mTestArea == null)
            {
                console.log("TouchTestDirective, Could not get test area");
                return;
            }

            mTestCanvas = findCanvas(mTestArea);

            if (mTestCanvas == null)
            {
                console.log("TouchTestDirective, Could not find canvas");
                return;
            }            

            initialiseCanvas(mTestCanvas);

            startIdleTimer();
        }
    }
  }

  function onMouseMove(aEvent)
  {
      //trace(">> onMouseMove"); 

      //aEvent.preventDefault();  // Required for Android version 4

      if (!mMouseDown)
      {
          return;
      }

      var event = aEvent;

      if (mIsNative)
      {
          var touches = aEvent.touches;

          var event = touches[0];       
      }

      var currentX = event.pageX;
      var currentY = event.pageY; 

      drawLine(mPrevX, mPrevY, currentX, currentY);

      mPrevX = currentX;
      mPrevY = currentY;
  }

  function onMouseDown(aEvent)
  {
      //trace(">> onMouseDown");  

      //aEvent.preventDefault();  // Required for Android version 4

      stopIdleTimer();

      if (mTestComplete)
      {
          return;
      }

      var event = aEvent;        

      if (mIsNative)
      {
          var touches = aEvent.touches;

          event = touches[0]; 
      }

      mPrevX = event.pageX;
      mPrevY = event.pageY;      

      mMouseDown = true;

      //mContext.beginPath();             
  }

  function onMouseUp(aEvent)
  {
      //trace(">> onMouseUp");   

      mMouseDown = false;

      mScope.onTestComplete(mTestArea, mTestCanvas);

/* Multi-tap logic - no longer required

      var testComplete = multiTapDetect();

      if (testComplete)
      {
          mTestComplete = true;
          stopIdleTimer();
          mScope.onTestComplete(mTestArea);
          return;
      }

      startIdleTimer();
*/
  }

  function drawLine(aX1, aY1, aX2, aY2)
  {
      //trace("drawLine, (" +aX1+ "," +aY1+ ") -> (" +aX2+ "," +aY2+ ")");

      mContext.beginPath();             

      mContext.moveTo(aX1, aY1);
      mContext.lineTo(aX2, aY2);
          
      mContext.stroke();      

      mContext.closePath();             

  }

  function getTestArea(aElements)
  {
      if (aElements == null || aElements.length == 0)
      {
          console.log("getTestArea-  No elements");
          return null;          
      }

      var area = aElements[0];

      if (area == null)
      {
          console.log("getTestArea - No children elements");
          return null;
      }

      return area;
  }

  function findCanvas(aTestArea)
  {
      if (aTestArea == null)
      {
          console.log("findCanvas - No children elements");
          return null;
      }
      
      var children = aTestArea.getElementsByTagName("CANVAS");

      if (children == null)
      {
          console.log("findCanvas - No child CANVAS");
          return null;
      }
      
      var canvas = children[0];

      if (canvas == null)
      {
          console.log("findCanvas - Cannot find canvas");
          return null;
      }            

      //console.log("findCanvas - Canvas found");

      return canvas;
  }

  function multiTapDetect()
  {
      var timeNow = getTimeNow();

      if (mMultiTapStartTime != -1)
      {
          var elapsed = timeNow - mMultiTapStartTime;

          //console.log("multiTapDetect, Elapsed: " +elapsed);

          if (elapsed > mMultiTapPeriod)
          {
              //console.log("multiTapDetect, Outside of period");        
              mMultiTapStartTime = -1;
          }      
      }

      if (mMultiTapStartTime == -1)
      {
          mMultiTapStartTime = timeNow;
          mMultiTapCount     = 1;
          console.log("multiTapDetect, New tap sequence");
          return false;
      }

      mMultiTapCount++;

      if (mMultiTapCount < mMultiTapTrigger)
      {
          //console.log("multiTapDetect, New tap inside period");                
          return false;
      }

      console.log("multiTapDetect, Got multi tap");
      mMultiTapStartTime = -1;

      return true;
  }

  function startIdleTimer()
  {
    /*
      if (mIdlePeriodTimer == null)
      {
          mIdlePeriodTimer = setTimeout(onIdlePeriodHandler, mIdlePeriod);
      }
    */
  }

  function stopIdleTimer()
  {
    /*
      clearTimeout(mIdlePeriodTimer);
      mIdlePeriodTimer = null;
    */
  }

  function onIdlePeriodHandler()
  {
      stopIdleTimer();

      if (!mTestComplete)
      {
          DEVICEBRIDGE.showOkDialog(null, "Tap 3 times anywhere on the screen to complete the test");
      }

  }

  function getTimeNow()
  {
      var dateNow = new Date();
      var timeNow = dateNow.getTime();
      return timeNow;
  }

  function initialiseCanvas(aCanvas)
  {  
      var screenSize = MMRSUTILS.getScreenDimensions();

      aCanvas.width  = "" + screenSize.w;
      aCanvas.height = "" + screenSize.h;

      var bubbleDown = true;

      var startEvent = mIsNative  ? "touchstart"  : "mousedown";
      var stopEvent  = mIsNative  ? "touchend"    : "mouseup";
      var moveEvent  = mIsNative  ? "touchmove"   : "mousemove";
      var cancelEvent = mIsNative ? "touchcancel" : "mouseup";

      aCanvas.addEventListener(startEvent,  onMouseDown, bubbleDown);
      aCanvas.addEventListener(stopEvent,   onMouseUp,   bubbleDown);
      aCanvas.addEventListener(moveEvent,   onMouseMove, bubbleDown);        
      aCanvas.addEventListener(cancelEvent, onMouseUp,   bubbleDown);

      mContext = aCanvas.getContext('2d');

      if (mContext == null)
      {
          trace("doLoad, Could not get context");
          return;
      }

      mContext.strokeStyle = "black";        
      mContext.lineWidth   = LINE_WIDTH; 
  }  

})();
