<script type="workflow-page">
	
	var page =
	{
	    items:
	    [	    
			{
				type: "HEADING",
				text: "Your lease plan"
			},
			{
				type: "SUBHEADING",
				text: "You started leasing this device, under a <B>Mobile Lease Plan</B> with " +
				      "<B>Mobile Swap Assure</B>, on " +
					  "<B>" +getContractStartDate()+ "</B>."
			},
			{
				type:    "FAQ",
				items:   getFAQItems(),
				match:   ["MOBILE_SWAP_ASSURE"],
				onClick: onFAQClick							
			},
			{
				id:   "swap-check",
				type: "SUBHEADING",
				text: "Checking upgrade offers"
			},
			{
				id:   "swap-text",				
				type: "SUBHEADING",
				text: "<B>You can upgrade to a new device when you take up a new plan with an " +
				      " eligible 24 / 36 month device payment and pay:</B>",
		        hide: true
			},
			{
				id:   "swap-price",
				type: "PRICE",
				text: "",
		        hide: true
			},	
			{
				id:   "fail-text-1",				
				type: "SUBHEADING",
				text: "Sorry, we’re unable to display your upgrade options at this time.",
		        hide: true
			},	
			{
				id:   "fail-text-2",				
				type: "TEXT",
				text: "To discuss your options, call us 24/7 on [#13 22 00#]",
				onClick: onDialClick,
		        hide: true
			}
		],

	    navigationItems:
	    [
			{
				id:   "how-to-button",
				type: "TEXT",
				text: "How to upgrade",
				hide: true,
				onClick: onHowToClick,
			},
			{
				id:   "fail-button",
				type: "TEXT",
				text: "Retry",
				hide: true,
				onClick: onFailureClick,
			}
      	]
	};

	function init()
	{
		console.log(">> init");

		fetchSwapPrices();
		
		console.log("<< init");		
	}

	function getFAQItems()
	{
		var faqsString = Workflow.readValue("swap-faqs");

		var faqs = [];

		try
		{
			faqs = JSON.parse(faqsString);
		}
		catch (err)
		{
			console.log("getFAQItems, Exception: " +err);
		}

		return faqs;
	}

	function getContractStartDate()
	{
		var contractTime = Workflow.getProperty(Properties.accountContractStartDate);
		
		var contractDate = new Date(contractTime*1000);

		var contractStr  = WorkflowUtils.makeShortDateString(contractDate);

		return contractStr;		
	}	

	function fetchSwapPrices()
	{
		var serviceNumber = Workflow.getProperty("msisdn");

		console.log("doFetchPrices, Fetching Prices For: " +serviceNumber);

		Workflow.showSpinner(true);		

		WorkflowUtils.fetchSwapPrices(serviceNumber, onPricesReceived);
	}

	function onPricesReceived(aSuccess, aPrices)
	{
		console.log(">> onPricesReceived");

		///aSuccess = false; //>>> TEST

		Workflow.showSpinner(false);		

        var swapCheckItem = WorkflowUtils.getArrayItemWithKey(page.items, "id", "swap-check");
        Workflow.setVisibility(swapCheckItem, false);

		if (! aSuccess)
		{
			var failTextItem1 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-1");
			var failTextItem2 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-2");

        	var failButton   = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "fail-button");

        	Workflow.setVisibility(failTextItem1, true);
        	Workflow.setVisibility(failTextItem2, true);        	
        	Workflow.setVisibility(failButton,   true);

			console.log("<< onPricesReceived - Failure");				
			return;
		}

        var swapTextItem  = WorkflowUtils.getArrayItemWithKey(page.items, "id", "swap-text");
        var swapPriceItem = WorkflowUtils.getArrayItemWithKey(page.items, "id", "swap-price");

        var howToButton   = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "how-to-button");

		//var swapPrice = Math.max(...aPrices);

		var swapPrice = getMaxValue(aPrices);

        swapPriceItem.text = "$" +swapPrice;

        Workflow.setVisibility(swapTextItem,  true);
        Workflow.setVisibility(swapPriceItem, true);
        Workflow.setVisibility(howToButton,   true);

        Workflow.setProperty(Properties.SwapPrice, swapPrice);

		Workflow.setAppState(MMRSAPPSTATE.EVERGREEN_UPGRADE_QUOTE_BAD);

		console.log("<< onPricesReceived");				
	}

	function onHowToClick()
	{
		console.log("onHowToClick");
		Workflow.setCurrentPageWithId("swap-howto");		
	}

	function onFailureClick()
	{
		console.log("onFailureClick");
		//Workflow.popPage();

        var swapCheckItem = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "swap-check");
        var swapTextItem  = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "swap-text");
        var swapPriceItem = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "swap-price");
		var failTextItem1 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-1");
		var failTextItem2 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-2");

        var howToButton   = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "how-to-button");
    	var doneButton    = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "fail-button");

        Workflow.setVisibility(swapCheckItem, true);

        Workflow.setVisibility(swapTextItem,  false);
        Workflow.setVisibility(swapPriceItem, false);
        Workflow.setVisibility(failTextItem1, false);
        Workflow.setVisibility(failTextItem2, false);

        Workflow.setVisibility(howToButton,   false);
        Workflow.setVisibility(doneButton,    false);

		fetchSwapPrices();		
	}	

	function onFAQClick(aItem, aExpanded, aHeaderId)
	{
		console.log("onFAQClick, Item: " +aItem.id+ ", Header: " +aHeaderId);	

		if (!aExpanded)
		{
			return;
		}

		WorkflowUtils.scrollTo(aHeaderId);
	}		

	function onDialClick()
	{
		window.location.href = "tel:132200";
	}

	function getMaxValue(aItems)
	{
		if (aItems == null)
		{
			return null;
		}

		var maxValue = null;

		for (var index in aItems)
		{
			var thisValue = aItems[index] - 0;

			if (maxValue == null || thisValue > maxValue)
			{
				maxValue = thisValue;
			}
		}

		return maxValue;
	}


</script>

