<script type="workflow-page">
	
	var page =
	{
	    items:
	    [    
			{
				type: "HEADING",
				text: "Upgrade your device"
			},
			{
				type: "SUBHEADING",
				text: getContractText()
			},
			{
				type:    "FAQ",
				items:   getFAQItems(),
				match:   getFAQMatches(),				
				onClick: onFAQClick						
			},
			{
				type:    "TEXT",
				text:    "<B>You can upgrade your device in any Telstra Store or online. " +
				         "[#Chat to us|" +Const.TELSTRA_CHAT_URL+ "#] or visit us at [#Telstra.com|" +Const.TELSTRA_STORE_URL+ "#]</B>",
				onClick: onLinkClick 
			},
			{
				id:   "swap-check",
				type: "SUBHEADING",
				text: "Checking upgrade offers"
			},									
			{
				id:   "swap-text",
				type: "SUBHEADING",
				text: "To determine your upgrade fee, you'll need to perform a device test to " +
				      "assess the condition of your device.",
		        hide: true
			},
			{
				id:   "fail-text-1",				
				type: "SUBHEADING",
				text: "Sorry, we’re unable to display your upgrade options at this time.",
		        hide: true
			},	
			{
				id:   "fail-text-2",				
				type: "TEXT",
				text: "To discuss your options, call us 24/7 on [#13 22 00#]",
				onClick: onDialClick,
		        hide: true
			}				
		],

	    navigationItems:
	    [	
			{
				id:   "start-button",				
				type: "TEXT",
				text: "Start device test",
				hide:  true,
				onClick: onNextClick
			},
			{
				id:   "fail-button",
				type: "TEXT",
				text: "Retry",
				hide: true,
				onClick: onFailureClick,
			}						    
      	]
	};
	
	function init()
	{
		fetchSwapPrices();		
	}	 

	function getFAQItems()
	{
		var faqsString = Workflow.readValue("swap-faqs");

		var faqs = [];

		try
		{
			faqs = JSON.parse(faqsString);
		}
		catch (err)
		{
			console.log("getFAQItems, Exception: " +err);
		}

		return faqs;
	}

	function getFAQMatches()
	{
		var onAssure = Workflow.getProperty(Properties.accountOnAssurePlan) == "true";
		
		var match = onAssure ? ["MOBILE_SWAP_ASSURE"] : null;

		return match;
	}

	function getContractText()
	{
		var onAssure = Workflow.getProperty(Properties.accountOnAssurePlan) == "true";

		var text;
		if (onAssure)
		{
			text = "You started leasing this device, under a <B>Go Mobile Swap plan</B> with Assure, on ";
		}
		else
		{
			text = "You started leasing this device, under a <B>Go Mobile Swap plan</B>, on ";
		}

		text += "<B>" +getContractStartDate()+ "</B>.";

		return text;
	}

	function getContractStartDate()
	{
		var contractTime = Workflow.getProperty(Properties.accountContractStartDate);
		
		var contractDate = new Date(contractTime*1000);

		var contractStr  = WorkflowUtils.makeShortDateString(contractDate);

		return contractStr;		
	}

	function fetchSwapPrices()
	{

		var serviceNumber = Workflow.getProperty("msisdn");

		console.log("doFetchPrices, Fetching Prices For: " +serviceNumber);

		Workflow.showSpinner(true);		

		WorkflowUtils.fetchSwapPrices(serviceNumber, onPricesReceived);
	}

	function onPricesReceived(aSuccess, aPrices)
	{
		console.log(">> onPricesReceived");

		//>> aSuccess = false;  /// TEST

		Workflow.showSpinner(false);

        var swapCheckItem = WorkflowUtils.getArrayItemWithKey(page.items, "id", "swap-check");
        Workflow.setVisibility(swapCheckItem, false);

		if (! aSuccess)
		{
			var failTextItem1 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-1");
			var failTextItem2 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-2");			
        	var failButton    = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "fail-button");

        	Workflow.setVisibility(failTextItem1, true);
        	Workflow.setVisibility(failTextItem2, true);       	
        	Workflow.setVisibility(failButton,    true);

			console.log("<< onPricesReceived - Failure");				
			return;
		}

        var swapTextItem  = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "swap-text");
        var startButton   = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "start-button");

        Workflow.setVisibility(swapTextItem,  true);
        Workflow.setVisibility(startButton,   true);

		console.log("<< onPricesReceived");				
	}

	function onNextClick()
	{
		console.log("onNextClick");
		window.location.href = "../mmrsswapcheck.html?title=Device%20Test";
	}
	
	function onFailureClick()
	{
		console.log("onFailureClick");
		//Workflow.popPage();

        var swapCheckItem = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "swap-check");
        var swapTextItem  = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "swap-text");

		var failTextItem1 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-1");
		var failTextItem2 = WorkflowUtils.getArrayItemWithKey(page.items,           "id", "fail-text-2");        
		
        var startButton   = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "start-button");
    	var failButton    = WorkflowUtils.getArrayItemWithKey(page.navigationItems, "id", "fail-button");

        Workflow.setVisibility(swapCheckItem, true);

        Workflow.setVisibility(swapTextItem,  false);
        Workflow.setVisibility(failTextItem1, false);
        Workflow.setVisibility(failTextItem2, false);

        Workflow.setVisibility(startButton,   false);
        Workflow.setVisibility(failButton,    false);

		fetchSwapPrices();		
	}	

	function onFAQClick(aItem, aExpanded, aHeaderId)
	{
		console.log("onFAQClick, Item: " +aItem.id+ ", Header: " +aHeaderId);	

		if (!aExpanded)
		{
			return;
		}

		WorkflowUtils.scrollTo(aHeaderId);
	}		

	function onDialClick()
	{
		window.location.href = "tel:132200";
	}	

	function onLinkClick(aLink)
	{
		console.log("onLinkClick, Link: " +aLink);
		window.location.href = aLink;
	}	

</script>

