var WORKFLOWPAGE = (function() {
"use strict"

function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
function error(aText) { MMRSUTILS.doError(exports.name, aText); }

mTheApp.directive('script', WORKFLOWLOADER.init);

mTheApp.controller('workflowCntl', ['$scope', workflowCntl]);

function workflowCntl($scope)
{
    var BRIDGE = MMRSUTILS.getDeviceBridge();

    var mIsWorkflowLoaded      = false;

    var mWorkflowPagesFile = null;
    var mWorkflowPages     = null;

    var mWorkflowNamesFile = null;    
    var mWorkflowItemsFile = null;    

    var mItemsPageFile     = null;    
    var mItemsPageId       = null;
    var mItemsPageIndex    = -1;

    var mWorkflowConstants  = null;
    var mWorkflowProperties = null;    

    $scope.workflowPageFile  = null;

    $scope.workflowFns   = [];
    $scope.workflowPages = [];
    $scope.workflowNames = [];
    
    $scope.page = {};

    var mWorkflowEngine = {};    

    $scope.getWorkflowFile = function()
    {
        if (mWorkflowPagesFile == null)
        {
            var options  = MMRSUTILS.getOptions();

            mWorkflowPagesFile = options.workflow;
            mItemsPageId       = options.page;

            var workflowDir = mWorkflowPagesFile.substring(0, mWorkflowPagesFile.lastIndexOf("/"));

            mWorkflowNamesFile = workflowDir+ "/" + "names.js";
        }

        //trace("== getWorkflowFile, File: " +mWorkflowPagesFile+ ", Page Id: " +mItemsPageId+ ", Names: " +mWorkflowNamesFile);

        return mWorkflowPagesFile;
    };

    $scope.onWorkflowLoaded = function()
    {
        trace("== onWorkflowLoaded");

        var workflowFn = $scope.workflowFns[0];

        if (workflowFn == null)
        {
            trace("== onWorkflowLoaded, No workflowFn defined");
            return;
        }

        mWorkflowPages  = readWorkflowPages(workflowFn);

        mItemsPageIndex = findWorkflowPageIndex(mWorkflowPages, mItemsPageId);

        if (mItemsPageIndex == -1)
        {
            trace("== onWorkflowLoaded, Could not get start page");            
            return;
        }

        var itemsPage = mWorkflowPages[mItemsPageIndex];

        mWorkflowItemsFile = itemsPage.page;

        trace("onWorkflowLoaded, Start Page: " +mWorkflowItemsFile);

        $scope.$applyAsync(function()
        {
            $scope.workflowNamesFile = mWorkflowNamesFile;
        });        
    }     

    $scope.onWorkflowNamesLoaded = function()
    {
        trace("+++ onWorkflowNamesLoaded");

        if (mWorkflowProperties != null)
        {
            return;
        }

        var namesDictionary = WORKFLOWLOADER.makeNamesDictionary($scope.workflowNames);

        mWorkflowConstants  = namesDictionary.constants;   
        mWorkflowProperties = namesDictionary.properties;
        
        // Now we have the names defined, start loading the workflow page 
        //
        $scope.$applyAsync(function()
        {
            $scope.workflowPageFile = mWorkflowItemsFile;
        });
    }

    $scope.onWorkflowPageLoaded = function()
    {
        trace("== onWorkflowPageLoaded");

        if (mIsWorkflowLoaded)
        {
            return;
        }

        mIsWorkflowLoaded = true;

        var numPages = $scope.workflowPages.length;

        trace("-- onWorkflowPageLoaded, Num Page: " +numPages);   

        if (numPages == 0)
        {
            trace("-- onWorkflowPageLoaded, No workflow definition page loaded");   
            return;
        }

        var pageFn = $scope.workflowPages[0];

        var exports = pageFn(mWorkflowEngine, mWorkflowConstants, mWorkflowProperties, mWorkflowUtils);  

        exports.init();

        var page = exports.getPageItems();

        /*
        var items = page.items;

        for (var index in items)
        {
            var thisItem = items[index];

            trace("-- onWorkflowPageLoaded, Item: " + JSON.stringify(thisItem));
        }
        */

        $scope.page.items           = page.items;
        $scope.page.tailItems       = page.tailItems;
        $scope.page.navigationItems = page.navigationItems;        
    }

    $scope.hasNavigationItems = function()
    {
        //console.log(">> hasNavigationItems");
        var hasItems = hasVisibleItems($scope.page.navigationItems);

        //console.log("<< hasNavigationItems, Items: " +hasItems);

        return hasItems;
    }

    $scope.hasTailItems = function()
    {
        var hasItems = hasVisibleItems($scope.page.tailItems);
        return hasItems;
    }

    function hasVisibleItems(aItems)
    {
        var isVisible = false;

        for (var index in aItems)
        {
            var thisItem = aItems[index];

            if (thisItem.hide == true)
            {
                continue;
            }

            isVisible = true;
            break;            
        }

        return isVisible;        

    }

    function readWorkflowPages(aWorkFlowFn)
    {
        var exports = aWorkFlowFn();

        var workflowPages = exports.getWorkflow();

        return workflowPages;
    }

    function findWorkflowPageIndex(aWorkflowPages, aPageId)
    {
        if (aWorkflowPages == null || aWorkflowPages.length == 0)
        {
            trace("== findWorkflowPageIndex, Workflow is empty");
            return -1;
        }

        if (aPageId == null)
        {
            trace("== findWorkflowPageIndex, No page Id passed - using first page");            
            return 0;
        }

        for (var index in aWorkflowPages)
        {
            var thisPage = aWorkflowPages[index];

            //trace("== findWorkflowPageIndex, Id: " +thisPage.id+ ", File: " +thisPage.page);

            if (thisPage.id == aPageId)
            {
                trace("== findWorkflowPageIndex, Found page: " +aPageId+ ", Index: " +index);
                return index-0;
            }
        }

        trace("== findWorkflowPageIndex, Workflow does not contain page: " +aPageId);
        return -1;
    }

    function getWorkflowFromIndex(aPageId)
    {
        var summary = null;

        for (var index in mWorkflowPages)
        {
            var thisPage = mWorkflowPages[index];

            trace("getWorkflowFromIndex, Id: " +thisPage.id+ ", File: " +thisPage.page+ ", Start: " +thisPage.isStart);

            if (thisPage.id == aPageId)
            {
                summary = thisPage;
                break;                
            }
        }

        return summary;
    }

    //==========================================================================================
    // Workflow Interface
    //==========================================================================================
    //

    mWorkflowEngine.trace = function(aMessage)
    {
        BRIDGE.doNativeLog("log", "workflow", aMessage);        
    }

    mWorkflowEngine.getPageItems = function()
    {
        return $scope.page.items;
    }

    mWorkflowEngine.getItemById = function(aItemId)
    {
        trace("getItemById, Item: " +aItemId);
        return document.getElementById(aItemId);
    }

    mWorkflowEngine.getTailItems = function()
    {
        return $scope.page.tailItems;
    }

    mWorkflowEngine.getNavigationItems = function()
    {
        return $scope.page.navigationItems;
    }

    mWorkflowEngine.setPageItemField = function(aItemId, aFieldId, aValue)
    {
        var items = $scope.page.items;

        var item = null;

        for (var index in items)
        {
            var thisItem = items[index];

            if (thisItem.id == aItemId)
            {
                item = thisItem;
                break;
            }
        }

        if (item != null)
        {
            $scope.$applyAsync(function()
            {
                item[aFieldId] = aValue;
            }); 
        }
    };

    mWorkflowEngine.setVisibility = function(aItem, aVisible)
    {
        $scope.$applyAsync(function()
        {
            //console.log("setVisibility, Item: " +aItem.id+ ", Visible: " +aVisible);
            aItem.hide = !aVisible;
        }); 
    }

    mWorkflowEngine.moveToNextPage = function()
    {
        trace("moveToNextPage, Current page index: " +mItemsPageIndex);     

        if (mItemsPageIndex == -1)
        {
            trace("moveToNextPage, Current page index is not set");     
            return;
        }

        var newPageIndex = mItemsPageIndex + 1;

        trace("moveToNextPage, New page index: " +newPageIndex);     

        var newPage = mWorkflowPages[newPageIndex];

        launchPage(newPage);        
    };    

    mWorkflowEngine.setCurrentPageWithId = function(aPageId, aIsBack)
    {
        trace("setCurrentPageWithId, Page: " +aPageId);

        var newPage = getWorkflowFromIndex(aPageId);

        if (newPage == null)
        {
            trace("setCurrentPageWithId, Could not find summary for page ID: " +aPageId);
            return;
        }

        launchPage(newPage);
    };


    mWorkflowEngine.setProperty = function(aProperty, aValue)
    {        
        trace("setProperty, Property: " +aProperty+ ", Value: " +aValue);   

        if (aProperty == null)
        {
            error("setProperty, No property supplied - check spelling");
            return;
        }

        BRIDGE.setValue(aProperty, aValue);     
    };

    mWorkflowEngine.getProperty = function(aProperty)
    {        
        if (aProperty == null)
        {
            error("getProperty, No property supplied - check spelling");
            return;
        }

        var value = BRIDGE.getValue(aProperty);

        //trace("getProperty, Property: " +aProperty+ ", Value: " +value);   

        return value;
    };    

    mWorkflowEngine.saveValue = function(aKey, aValue)
    {        
        trace("saveValue, Key: " +aKey+ ", Value: " +aValue);   

        BRIDGE.writePage(aKey, aValue);     
    };

    mWorkflowEngine.readValue = function(aKey)
    {        
        trace("readValue, Key: " +aKey);   

        var value = BRIDGE.readPage(aKey); 

        return value;
    };

    mWorkflowEngine.getDeviceSetting = function(aSetting, aCallback)
    {        
        if (aSetting == null)
        {
            error("getDeviceSetting, No setting supplied");
            return;
        }

        var requestId = BRIDGE.sendGetCommand(aSetting, function(aRequestId, aSuccess, aResponse)
        {
            if (!aSuccess)
            {
                aCallback(false, null);
                return;
            }

            var response = null;
            try
            {
                response = JSON.parse(aResponse);
            }
            catch (err)
            {
                error("getDeviceSetting, Could not parse response: " +err);
                response = null;
            }

            if (response == null)
            {
                aCallback(false, null);
                return;
            }

            aCallback(true, response);
        });

        trace("getDeviceSetting, Setting: " +aSetting+ ", Request-Id: " +requestId);   

        return requestId;
    };  

    mWorkflowEngine.popPage = function()    
    {
        BRIDGE.popPage();
    }

    mWorkflowEngine.popToHomePage = function()    
    {
        BRIDGE.popToHomePage();
    }    

    mWorkflowEngine.setPageBackground = function(aBackground)
    {        
        trace("setPageBackground, Background: " +aBackground);    

        BRIDGE.setPageBackground(aBackground);
    }

    mWorkflowEngine.showSpinner = function(aActive, aPos, aHasBackground)
    {
        BRIDGE.showSpinner(aActive, aPos, aHasBackground);       
    }

    mWorkflowEngine.setHeaderStyle = function(aShowStatusBar, aShowNavBar)
    {        
        trace("setHeaderStyle, Show Status: " +aShowStatusBar+ ", Show Nav: " +aShowNavBar);    

        BRIDGE.setHeaderStyle(aShowStatusBar, aShowNavBar);
    }

    mWorkflowEngine.setAppState = function(aState)
    {
        BRIDGE.setAppState(aState);       
    }

    //==========================================================================================
    // Utilities
    //==========================================================================================
    //    
    var DAY_NAMES   = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
    var MONTH_NAMES = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

    var mWorkflowUtils = {};

    mWorkflowUtils.isAndroidRun = function()
    {
        return MMRSUTILS.isAndroidRun();
    }

    mWorkflowUtils.isIPhoneRun = function()
    {
        return MMRSUTILS.isIPhoneRun();
    }

    mWorkflowUtils.getArrayItemWithKey = function(aArray, aKey, aValue)
    {
        var item = null;

        for (var index in aArray)
        {
            var thisItem = aArray[index];

            if (thisItem[aKey] == aValue)
            {
                item = thisItem;
                break;
            }
        }

        return item;
    }

    mWorkflowUtils.makeShortDateString = function(aDate)
    {
        var day   = aDate.getDate()     - 0;
        var month = aDate.getMonth()    - 0;
        var year  = aDate.getFullYear() - 0;

        month = month + 1;
        year  = year;

        var dateStr = day+ "/" +month+ "/" +year;

        return dateStr;
    }    


    mWorkflowUtils.makeDayDateString = function(aDate)
    {
        var dayOfWeek  = aDate.getDay();
        var dayOfMonth = aDate.getDate();    

        var month = aDate.getMonth();
        var year  = aDate.getFullYear();

        var dateStr = DAY_NAMES[dayOfWeek]+ ", " +dayOfMonth+ " " +MONTH_NAMES[month];
 
        var dateNow = new Date();
        var yearNow = dateNow.getFullYear();

        if (year != yearNow)
        {
            dateStr += " " +year;
        }

        return dateStr;
    }   


    mWorkflowUtils.getDeviceSettingValue = function(aResponseData, aFieldId)
    {
        if (aResponseData == null || aResponseData[0] == null)
        {
            trace("getDeviceSettingValue, No response data");
            return null;
        }

        var response = aResponseData[0];

        var fields = response.fields;

        if (fields == null)
        {
            trace("getDeviceSettingValue, No fields in data");
            return null;
        }

        var value = null;

        for (var index in fields)
        {
            var thisField = fields[index];

            trace("getDeviceSettingValue, Field: " +thisField.fieldid);

            if (thisField.fieldid == aFieldId)
            {
                value = thisField.fieldvalue;
                break;
            }
        }

        return value;
    };

    //====================================================================================
    // Scrolling
    //====================================================================================
    //
    var mWorkflowContent = document.getElementById("workflowContent");

    mWorkflowUtils.scrollTo = function(aElementId)
    {
        if (mWorkflowContent == null)
        {
            error("scrollTo, Cannot find workflow content element");
            return;
        }

        $scope.$applyAsync(function()
        {        
            smoothScrollTo(aElementId);
        });
    }

    function smoothScrollTo(aElementId)
    {
        trace("smoothScrollTo, Element: " +aElementId);

        //var element = document.getElementById(aElementId);
        //var elementHeight = element.scrollHeight;   
        //trace("smoothScrollTo, Element Height:  " +elementHeight);

        var body = document.body;
        var html = document.documentElement; 
        
        var windowHeight  = window.innerHeight;
        var contentHeight = Math.max(body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight);
        var currentScroll = window.pageYOffset;

        var elementRect = MMRSUTILS.getElementPosition(aElementId);

        if (elementRect == null)
        {
            trace("smoothScrollTo, No Element: " +aElementId);
            return;
        }

        var targetPos = elementRect.y;

        //trace("smoothScrollTo, Window Height:  " +windowHeight);
        //trace("smoothScrollTo, Content Height: " +contentHeight);        
        //trace("smoothScrollTo, Current Scroll: " +currentScroll);        
        //trace("smoothScrollTo, Element Pos:    " +targetPos);

        if (contentHeight <= windowHeight)
        {
            trace("smoothScrollTo, document shorter than window");
            //return;
        }

        var scrollRequired = targetPos - currentScroll;

        //trace("smoothScrollTo, Scroll Required: " +scrollRequired);

        var targetTail = contentHeight - targetPos;

        //trace("smoothScrollTo, Target Tail: " +targetTail);

/*
        if (targetTail < windowHeight)
        {
            scrollRequired = targetTail;
            trace("smoothScrollTo, No enough tail - adjusting scroll: " +scrollRequired);
        }
*/

        //window.scrollBy(0, scrollRequired);

        //trace("smoothScrollTo, Required: " +scrollRequired);

        doSmoothScrollBy(scrollRequired);
    }    

    var mScrollJump  = 5;
    var mScrollTimer = 5;

    function doSmoothScrollBy(aRemainder)
    {
        if (aRemainder <= 0)
        {
            return;
        }

        var jump = Math.min(mScrollJump, aRemainder);

        mWorkflowContent.scrollTop += jump;

        var remainder = aRemainder - jump;

        setTimeout(doSmoothScrollBy, mScrollTimer, remainder);
    }    

    //====================================================================================
    // Fetch Swap Information
    //====================================================================================
    //
    var mCallbackById = {};
    var mCallback = null;

    var SWAP_PRICES_PROPERTY      = "ExchangePrices";
    var SWAP_PRICES_PROPERTY_TEST = "ExchangePricesTest";

    mWorkflowUtils.fetchSwapPrices = function(aServiceNum, aCallback)
    {
        var prices    = null;
        var pricesStr = "";

        //setTimeout(aCallback, 1000, false, null); return;  // FAILURE TEST

        try
        {
            pricesStr = BRIDGE.getValue(SWAP_PRICES_PROPERTY);

            console.log("fetchSwapPrices, Cached Prices: " +pricesStr);

            if (pricesStr != null && pricesStr != "")
            {
                prices = JSON.parse(pricesStr);
            }
        }
        catch(err)
        {
            console.log("fetchSwapPrices, Bad Cached Prices: " +pricesStr);
            prices = null;
        }

        if (prices == null)
        {
            doFetchSwapPrices(aServiceNum, aCallback);    
            return;
        }

        console.log("fetchSwapPrices, Using Cached Prices");

        aCallback(true, prices);        
    }

    var mTestFetchPricesRetryCount = 0;

    function doFetchSwapPrices(aServiceNum, aCallback)
    {
        var testPricesStr = BRIDGE.getValue(SWAP_PRICES_PROPERTY_TEST);

        if (testPricesStr != null && testPricesStr != "")
        {
            if (mTestFetchPricesRetryCount == 0)
            {
                console.log("doFetchSwapPrices, Simulatind download failure");
                mTestFetchPricesRetryCount++;
                setTimeout(aCallback, 3000, false, null);
                return;
            }

            console.log("doFetchSwapPrices, Using Test Prices: " +testPricesStr);

            BRIDGE.setValue(SWAP_PRICES_PROPERTY, testPricesStr);

            var prices = JSON.parse(testPricesStr);

            setTimeout(aCallback, 1000, true, prices);
            return;
        }

        console.log("doFetchSwapPrices, Fetching Prices For: " +aServiceNum);

        sendSwapPricesRequest(aServiceNum, aCallback);
    }

    function sendSwapPricesRequest(aServiceNum, aCallback)
    {
        trace("sendSwapPricesRequest");

        var swapInfoUrl = BRIDGE.getValue("swapInfoUrl");

        if (swapInfoUrl == null || swapInfoUrl == "")
        {
            trace("sendSwapPricesRequest, No swap info URL stored");
            return;
        }

        var requestHostUrl = BRIDGE.getRequestHostUrl() + "/" +BRIDGE.getParam("deviceDir") + "/";

        swapInfoUrl = MMRSUTILS.getRequestUrl(requestHostUrl, swapInfoUrl);

        var headers = {};

        headers["source-system"] = BRIDGE.getValue("sourceSystem");
        headers["Authorization"] = BRIDGE.getParam("swapName");
        headers["Encoding"]      = "UTF-8";
        headers["Accept"]        = "application/json";

        swapInfoUrl += "?serviceType=MobileService&category=Lease+Plan&deviceStatus=Redemption";

        swapInfoUrl += "&serviceId=" +aServiceNum;

        //trace("sendSwapPricesRequest, Swap Info Url: " +swapInfoUrl);

        var requestId = BRIDGE.getCellularUrl(swapInfoUrl, headers, onSwapInfoReceived); 

        //var requestId = BRIDGE.fetchUrl(swapInfoUrl, onSwapInfoReceived, headers);  // >>> TEST OVER WIFI

        mCallbackById[requestId] = aCallback;
        mCallback = aCallback;

        //trace("sendSwapPricesRequest, Request Id: " +requestId);
    }

    function onSwapInfoReceived(aUrl, aSuccess, aHeader, aBody)
    {
        //trace("onSwapInfoReceived, Url:  " +aUrl+ ", Success: " +aSuccess);

        /// aSuccess = false; //>>>> TEST

        var callback = mCallback;

        if (callback == null)
        {
            return;
        }

        if (!aSuccess)
        {
            callback(false, null);
            return;            
        }

        if (aHeader != null)
        {
            //trace("onSwapInfoReceived, Header: " +aHeader);            
        }

        if (aBody == null)
        {
            trace("onSwapInfoReceived, No response body");  
            callback(false, null);
            return;
        }

        //trace("onSwapInfoReceived, aBody: " +aBody);            

        var response = null;

        try
        {
            response = JSON.parse(aBody);
        }
        catch (err)
        {
            error("onSwapInfoReceived, Exception: " +err);
            callback(false, null);
            return;
        }

        //trace("onSwapInfoReceived, parsed response");

        var offerRecord = getOffersRecord(response);

        if (offerRecord == null)
        {
            trace("onSwapInfoReceived, No offers");
            callback(false, null);
            return;            
        }

        var upgradePrices = getUpgradePrices(offerRecord);

        if (upgradePrices == null)
        {
            warn("onSwapInfoReceived, No upgrade prices");
            callback(false, null);
        }

        var pricesStr = JSON.stringify(upgradePrices);

        //console.log("onSwapInfoReceived, Prices: " +upgradePrices);   

        BRIDGE.setValue(SWAP_PRICES_PROPERTY, pricesStr);

        callback(true, upgradePrices);
    }    

    var CONDITION_GOOD_TEXT  = "Good Working Order";
    var CONDITION_MINOR_TEXT = "Minor Damage";
    var CONDITION_MAJOR_TEXT = "Major Damage";    

    function getOffersRecord(aResponse)
    {
        if (aResponse == null)
        {
            warn("getOffersRecord, Null Response");            
            return null;
        }

        var swapRecord = aResponse;

        var offers = MMRSUTILS.getFieldValue(swapRecord, "data", "offers");

        if (offers == null)
        {
            warn("getOffersRecord, Response Swap Record has no Offers field");
            return null;
        }

        if (offers.length == null || offers.length == 0)
        {
            warn("getOffersRecord, Offers is not an array");  
            return null;
        }

        var offerRecord = offers[0];

        return offerRecord;
    }    

    function getUpgradePrices(aOfferRecord)
    {
        if (aOfferRecord == null)
        {
            warn("getUpgradePrices, Null Response");            
            return null;
        }

        var pricesTable = MMRSUTILS.getFieldValue(aOfferRecord, "fees", "redemption");

        if (pricesTable == null)
        {
            warn("getUpgradePrices, Could not get upgrade prices");  
            return null;            
        }

        var prices = [];

        prices[0] = "";

        for (var index in pricesTable)
        {
            var thisItem = pricesTable[index];

            var thisCondition = thisItem.condition;
            var thisAmount    = thisItem.amount;

            trace("getUpgradePrices, Condition: " +thisCondition+ ", Amount: " +thisAmount);

            if (thisCondition == null || thisAmount == null)
            {
                error("getUpgradePrices, Bad upgrade prices");
                callback(false);
            }

            if (thisCondition == CONDITION_GOOD_TEXT)
            {
                prices[mWorkflowConstants.CONDITION_GOOD] = thisAmount;
                continue;
            }

            if (thisCondition == CONDITION_MINOR_TEXT)
            {
                prices[mWorkflowConstants.CONDITION_MINOR_DAMAGE] = thisAmount;
                continue;
            }

            if (thisCondition == CONDITION_MAJOR_TEXT)
            {
                prices[mWorkflowConstants.CONDITION_MAJOR_DAMAGE] = thisAmount;
                continue;
            }            
        }

        if (prices.length != 4)
        {
            error("getUpgradePrices, Incomplete upgrade price table received");
            return null;            
        }

        trace("getUpgradePrices, Got prices array: " +JSON.stringify(prices));

        return prices;
    }


    //==========================================================================================
    //==========================================================================================
    //
    function launchPage(aPage)
    {
        if (aPage == null)
        {
            trace("launchPage, Null page");
            return;
        }

        var pageId = aPage.id;

        var title  = aPage.title  == null ? "" : "&title=" + aPage.title;

        var options = aPage.options == null ? {} : aPage.options;

        var background = options.background != null ? "&background=" +options.background : "";
        
        var hideNavBar   = options.hideNavBar   == true ? "&hideNavBar=true"   : "";
        var fullScreen   = options.fullScreen   == true ? "&fullScreen=true"   : "";
        var finishOnPush = options.finishOnPush == true ? "&finishOnPush=true" : "";

        var workflowIndex = $scope.getWorkflowFile();
        var workflowPage  = $scope.workflowPageFile;

        trace("launchPage, Index: " +workflowIndex+ ", Page: " +workflowPage+ ", Options: " +JSON.stringify(options));

        var url = MMRSUTILS.getUrlPath();

        trace("launchPage, UrlPath: " +url);        

        url += "?workflow=" +workflowIndex;
        url += "&page="     +pageId;

        url += title;
        url += hideNavBar;
        url += fullScreen;
        url += finishOnPush;        
        url += background;

        trace("launchPage, Url: " +url);

        window.location.href = url;        
    }

    function doLoad()
    {   
        trace(">> doLoad");
        trace("<< doLoad");
    }

    doLoad();
}
    
function onLoad()
{
    trace(">> onLoad");


    trace("<< onLoad");    
}

var exports = {};

exports.name    = "workflow-cntl";

exports.onLoad  = function() { onLoad(); }

return exports;

}());
