var WORKFLOWLOADER = (function ()
{
  "use strict"

  //mTheApp.directive('script', WorkflowLoader);

  var mExports = ["init", "nextPage", "onDisplay"];

  function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
  function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
  function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
  function error(aText) { MMRSUTILS.doError(exports.name, aText); }  

  // PAGE ITEM
  //
  function initWorkflowLoader()
  {
      //console.log("WorkflowLoader");

      var directive =
      {
        restrict: 'E',
        scope:    false,
        replace:  true,

        link: function(aScope, aElement, aAttributes)
        {
            console.log("++ WorkflowScript, Type: " +aAttributes.type);              

            if (aAttributes.type === "workflow")
            {
                var code = aElement.text();

                code = createWorkflowExports(code);

                var f = new Function(code);

                aScope.workflowFns[0] = f;
                return;
            }

            if (aAttributes.type === "workflow-page")
            {
                //var workflowPageId = aAttributes.page

                var code = aElement.text();

                code = createWorkflowPageExports(code);

                var f = new Function(code);

                aScope.workflowPages[0] = f;
                return;
            }

            if (aAttributes.type === "workflow-names")
            {

                var code = aElement.text();

                code = createWorkflowNamesExports(code);

                var f = new Function(code);

                aScope.workflowNames[0] = f;
                return;
            }

        }
      }

      return directive;
  }

  // Add standard exports to function - enables its methods to be called later
  //
  function createWorkflowExports(aCode)
  {
      var exports = "";

      exports += "var workflowDefn = ";

      exports += aCode;

      exports += ";";

      exports += "var exports = {};";

      exports += "exports.getWorkflow = function() { return workflowDefn; };";

      exports += "return exports;"

      //console.log("createWorkflowExports: " +exports);

      return exports;
  }

  function createWorkflowPageExports(aCode)
  {
      var exports = "";

      exports += "var Workflow      = arguments[0];";
      exports += "var Const         = arguments[1];";      
      exports += "var Properties    = arguments[2];";
      exports += "var WorkflowUtils = arguments[3];";

      exports += aCode;

      exports += "var exports = {};";

      for (var index = 0; index < mExports.length; index++)
      {
          var thisFn = mExports[index];

          var thisExport = "exports." +thisFn+ " = function() { return typeof " +thisFn+ " !== 'undefined' ? " +thisFn+ "() : null; };";    

          exports += thisExport;      
      }

      exports += "exports.getPageItems = function() { return page; };";

      exports += "return exports;"

      //console.log("createWorkflowPageExports: " +exports);

      return exports;
  }

  function createWorkflowNamesExports(aCode)
  {
      var exports = "";

      exports += "var workflowNames = ";

      exports += aCode;

      exports += ";";

      exports += "var exports = {};";

      exports += "exports.getWorkflowNames = function() { return workflowNames; };";

      exports += "return exports;"

      //console.log("createWorkflowNamesExports: " +exports);

      return exports;
  }

  function makeNamesDictionary(aNamesFn)
  {
      trace("makeNamesDictionary");

      if (aNamesFn == null)
      {
          return null;
      }

      var numFunctions = aNamesFn.length;

      trace("makeNamesDictionary, Num Functions: " +numFunctions);   

      if (numFunctions == 0)
      {
          trace("makeNamesDictionary, No workflow names funtions");   
          return null;
      }

      var namesFn = aNamesFn[0];

      var exports = namesFn();            

      var names = exports.getWorkflowNames();

      var dictionary =
      {
          constants  : names.constants,
          properties : {}
      };

      for (var index in names.properties)
      {
          var thisName = names.properties[index];

          //trace("makeNamesDictionary, Name: " + thisName);

          dictionary.properties[thisName] = thisName;
      }        

      return dictionary;
  }

var exports = {};

exports.name = "workflowloader";

exports.init = function() { return initWorkflowLoader(); }

exports.makeNamesDictionary = function(aNamesFn) { return makeNamesDictionary(aNamesFn); }

return exports;


}());