
var NETSERVICES = (function ()
{
    "use strict";

    function trace(aText) { MMRSUTILS.doTrace(exports.name, aText); }
    function note(aText)  { MMRSUTILS.doTrace(exports.name, aText); }
    function warn(aText)  { MMRSUTILS.doWarn (exports.name, aText); }
    function error(aText) { MMRSUTILS.doError(exports.name, aText); }          

    var BRIDGE = MMRSUTILS.getDeviceBridge();

    var mSourceSystem = "TDCAndroid";      

    var mRequestHostUrl = BRIDGE.getRequestHostUrl(BRIDGE.getParam("deviceDir"));

    var mWorkflowProperties = null;
    var mWorkflowConstants  = null;      

    //====================================================================================
    // Which App Services to Try/Enable
    //====================================================================================
    //
    function fetchAppServices(aCallback)
    {
        var servicesFile = BRIDGE.getValue("services-file");

        if (servicesFile == null || servicesFile == "")
        {
            servicesFile = "servicesweb.json";
        }

        var url = mRequestHostUrl + servicesFile;

        //trace("fetchAppServices, Url: " +url);

        BRIDGE.fetchUrl(url, function(aUrl, aSuccess, aHeader, aText)
        {
            onAppServicesReceived(aSuccess, aText, aCallback); 
        });
    }    

    function onAppServicesReceived(aSuccess, aText, aCallback)
    {
        trace("onAppServicesReceived, Success: " +aSuccess);

        if (! aSuccess)
        {
            error("onAppServicesReceived, Unable to fetch services");
            aCallback(false, null);
            return;
        }

        //trace("onAppServicesReceived, Services: +" +aText+ "+");

        var services = null;

        try
        {
            services = JSON.parse(aText);
        }
        catch(err)
        {
            error("onAppServicesReceived, Json: " +err);
            services = null;
        }

        if (services == null)
        {
            aCallback(false, null);
            return;
        }

        aCallback(true, services);
    }  

    function saveServicesValues(aServices)
    {
        if (aServices == null || aServices.values == null)
        {
            return;
        }

        var values = aServices.values;

        for (var thisKey in values)
        {
            var thisValue = values[thisKey];

            //trace("saveServicesValues, Key: " +thisKey+ ", Value: " +thisValue);
            BRIDGE.setValue(thisKey, thisValue);
        }
    }    

    //====================================================================================
    // Fetch MSP Account Information
    //====================================================================================
    //
    var ACCOUNT_START_DATE_PROPERTY     = "accountContractStartDate";
    var ACCOUNT_LEASE_PLAN_PROPERTY     = "accountOnLeasePlan";
    var ACCOUNT_ASSURE_PLAN_PROPERTY    = "accountOnAssurePlan";
    var ACCOUNT_PRODUCTS                = "accountProducts";
    var ACCOUNT_LEGAL_LESSE_PROPERTY    = "accountIsLegalLesse";
    var ACCOUNT_FULL_AUTHORITY_PROPERTY = "accountHasFullAuthority";
    var ACCOUNT_SERVICE_ID              = "msisdn";
    var ACCOUNT_ID                      = "accountId";
    var ACCOUNT_CUSTOMER_ID             = "accountCustomerId";
    var ACCOUNT_CUSTOMER_TYPE           = "accountCustomerType";

    function fetchAccountInformation(aCallback)
    {
        //trace("fetchAccountInformation");

        var accountInfoUrl = BRIDGE.getValue("accountInfoUrl");

        if (accountInfoUrl == null || accountInfoUrl == "")
        {
            warn("fetchAccountInformation, No mobile account info URL in services");            
            aCallback(false, null);
            return;            
        }

        //trace("fetchAccountInformation, Url: " +accountInfoUrl);

        accountInfoUrl = MMRSUTILS.getRequestUrl(mRequestHostUrl, accountInfoUrl);

        if (accountInfoUrl == null)
        {
            warn("fetchAccountInformation, Bad URL");
            aCallback(false, null);
            return;            
        }        

        var headers = {};

        headers["Accept"]        = BRIDGE.getParam("accountAccept");
        headers["source-system"] = BRIDGE.getValue("sourceSystem");

        //trace("fetchAccountInformation, Url: " +accountInfoUrl);

        var requestId = BRIDGE.getCellularUrl(accountInfoUrl, headers, 
            function(aUrl, aSuccess, aHeader, aBody)
            {
                onAccountInfoReceived(aSuccess, aBody, aCallback); 
                return;
            });

        //trace("fetchAccountInformation, Request Id: " +requestId);
    }

    function onAccountInfoReceived(aSuccess, aBody, aCallback)
    {
        trace("onAccountInfoReceived, Success: " +aSuccess);

        if (!aSuccess)
        {
            aCallback(false, null);
            return;            
        }

        if (aBody == null)
        {
            warn("onAccountInfoReceived, No response body");            
            aCallback(false, null);
            return;
        }

        //trace("onAccountInfoReceived, aBody: " +aBody);            

        var accountInfo = null;

        try
        {
            accountInfo = JSON.parse(aBody);
        }
        catch (err)
        {
            error("onAccountInfoReceived, Exception: " +err);
            accountInfo = null;
        }

        if (accountInfo == null)
        {
            aCallback(false, null);
            return;
        }

        aCallback(true, accountInfo);
    }  

    function saveAccountDetails(aRecord)
    {
        var serviceDetails = aRecord["ServiceDetails"]
        
        if (serviceDetails == null)
        {
            error("saveAccountDetails, Could not get ServiceDetails");
            return false;
        }
 
        var account = MMRSUTILS.getFieldValue(serviceDetails, "account");

        if (account == null)
        {
            error("saveAccountDetails, Could not get account");
            return false;
        }

        var accountId = MMRSUTILS.getFieldValue(account, "accountId");

        if (accountId != null)
        {
            BRIDGE.setValue(ACCOUNT_ID, accountId);
        }

        //trace("saveAccountDetails, Account Id: " +accountId);

        var customer = MMRSUTILS.getFieldValue(serviceDetails, "customer");

        if (customer == null)
        {
            error("saveAccountDetails, Could not get customer");
            return false;
        }

        var customerId = MMRSUTILS.getFieldValue(customer, "customerId");

        if (customerId != null)
        {
            BRIDGE.setValue(ACCOUNT_CUSTOMER_ID, customerId);
        }

        var customerType = MMRSUTILS.getFieldValue(customer, "customerType");

        if (customerType != null)
        {
            BRIDGE.setValue(ACCOUNT_CUSTOMER_TYPE, customerType);
        }

        //trace("saveAccountDetails, Customer Type: " +customerType);        

        var service = MMRSUTILS.getFieldValue(serviceDetails, "service");

        if (service == null)
        {
            error("saveAccountDetails, Could not get service");
            return false;
        }

        var contractDateStr = getContractDate(service);

        if (contractDateStr != null)
        {
            var contractDate = MMRSUTILS.parseDate(contractDateStr);

            if (contractDate != null && contractDate != "")
            {
                contractDate = Math.floor(contractDate / 1000);
                BRIDGE.setValue(ACCOUNT_START_DATE_PROPERTY, contractDate);            
            }
        }

        var serviceId = MMRSUTILS.getFieldValue(service, "serviceId");

        if (serviceId != null)
        {
            BRIDGE.setValue(ACCOUNT_SERVICE_ID, serviceId);
        }

        //trace("saveAccountDetails, Service Id: " +serviceId);

        var legalLesse    = MMRSUTILS.getFieldValue(service, "legalLesse");
        var fullAuthority = MMRSUTILS.getFieldValue(service, "fullAuthority");

        if (legalLesse != null)
        {
            BRIDGE.setValue(ACCOUNT_LEGAL_LESSE_PROPERTY, legalLesse);
        }

        if (fullAuthority != null)
        {
            BRIDGE.setValue(ACCOUNT_FULL_AUTHORITY_PROPERTY, fullAuthority);
        }

        //trace("saveAccountDetails, Legal Lesse: " +legalLesse+ ", Full Auth: " +fullAuthority);

        var serviceAttribs = MMRSUTILS.getFieldValue(service, "attributes");

        var networkIMEI= getAttributeValue(serviceAttribs, "networkIMEI");

        var imei = null;

        if (networkIMEI != null)
        {
            imei = MMRSUTILS.makeDeviceImei(networkIMEI);
        }

        if (imei != null)
        {
            BRIDGE.setValue("imei", imei);            
        }

        var accountIMEI = getAttributeValue(serviceAttribs, "IMEI");

        if (accountIMEI != null && accountIMEI != "")
        {
            BRIDGE.setValue("accountIMEI", accountIMEI);                        
        }

        var planAttribs = MMRSUTILS.getFieldValue(service, "plan", "attributes");

        if (planAttribs == null)
        {
            trace("saveAccountDetails, No plan attributes");
            return false;
        }

        var hasDLCPlan = getAttributeValue(planAttribs, "DLC_PLAN");

        var products = getProductsList(service);

        var matchAssurePlan = matchProduct("MOBILE_SWAP_ASSURE", products);

        var leasePlan  = hasDLCPlan     ? "true" : "false";
        var assurePlan = matchAssurePlan ? "true" : "false";

        BRIDGE.setValue(ACCOUNT_LEASE_PLAN_PROPERTY,  leasePlan);   // Evergreen Lease Plan
        BRIDGE.setValue(ACCOUNT_ASSURE_PLAN_PROPERTY, assurePlan);  // Assure Lease Option

        BRIDGE.setValue(ACCOUNT_PRODUCTS, products);    

        return true;
    }    

    function getContractDate(aRecordList)
    {
        if (aRecordList == null)
        {
            trace("getContractDate, No service detaiuls");
            return null;
        }

        var contracts = MMRSUTILS.getFieldValue(aRecordList, "contracts");        

        if (contracts == null)
        {
            trace("getContractDate, No contract information");
            return null;
        }

        var installedDate = getListValueForType(contracts, "PLAN", "installedDate");

        if (installedDate == null)
        {
            trace("getContractDate, No contract started date");            
            return null;
        }

        //trace("getContractDate, Contract started date: " +installedDate);   

        return installedDate;
    }

    function getListValueForType(aItems, aType, aField)
    {
        if (aItems == null || aType == null)
        {
            return null;
        }

        var value = null;

        for (var index in aItems)
        {
            var thisItem = aItems[index];

            if (thisItem.type == aType)
            {
                value = thisItem[aField];
                break;
            }
        }

        //trace("getListValueForType, Type: " +aType+ ", Value: " +value);

        return value;
    }

    function getAttributeValue(aAttributes, aName)
    {
        if (aAttributes == null)
        {
            return null;
        }

        var value = null;

        for (var index in aAttributes)
        {
            var thisAttrib = aAttributes[index];

            if (thisAttrib.name == aName)
            {
                value = thisAttrib.value;
                break;
            }
        }

        //trace("getAttributeValue, Name: " +aName+ ", Value: " +value);

        return value;
    }

    function getProductsList(aService)    
    {
        if (aService == null)
        {
            return "";
        }

        var products = aService.products;

        if (products == null)
        {
            trace("getProductsList, No products");
            return "";
        }

        var list = "";

        for (var index in products)
        {
            var thisProduct = products[index];

            var productType = thisProduct.productType;

            //trace("getProductsList, Type: " +productType);

            if (productType == null)
            {
                continue;
            }

            if (index > 0)
            {
                list += ",";
            }

            list += productType;
        }

        //trace("getProductsList: " +list);

        return list;
    }

    function matchProduct(aProduct, aProductList)
    {         
        if (aProduct == null || aProductList == null || aProductList == "")
        {
            trace("matchProduct, Bad parameters");            

            return false;
        }

        var list = aProductList.split(",");

        for (var index in list)
        {
            var thisProduct = list[index];

            var hasProduct = thisProduct.indexOf(aProduct) >= 0;

            if (hasProduct)
            {
                //trace("matchProduct, Found " +aProduct+ ", In List: " +aProductList);
                return true;
            }
        }

        trace("matchProduct, NO MATCH: " +aProduct+ ", In List: " +aProductList);
        return false;
    }

    //====================================================================================
    // Returns Request Retrieval
    //====================================================================================
    //   
    var RETURNS_CREATED_TIME = "returnsCreatedTime";

    var RR_TIME_FORMAT = "MM/dd/yyyy HH:mm:ss";
    var RR_TIME_ZONE   = "Australia/Sydney";    

    function fetchReturnsRecord(aIMEI, aCallback)
    {
        trace("fetchReturnsRecord, IMEI: " +aIMEI);

        BRIDGE.setValue(RETURNS_CREATED_TIME, "");

        if (aIMEI == null || aIMEI == "")
        {
            warn("sendReturnsRetrievalRequest, No IMEI Passsed");
            aCallback(null, false, null, null);
            return;
        }

        var url = BRIDGE.getValue("rrRetrieveUrl");

        url = MMRSUTILS.getRequestUrl(mRequestHostUrl, url);

        if (url == null || url == "")
        {
            error("sendReturnsRetrievalRequest, No RR Retrieval URL defined");            
            aCallback(null, false, null, null);
            return;
        }

        //trace("sendReturnsRetrievalRequest, Sending RR Retrieve...");

        var headers = {};

        headers["source-system"] = BRIDGE.getValue("sourceSystem");

        headers["correlationId"] = uuid();
        headers["itemCategory"]  = "Handset";
        headers["imei"]          = aIMEI;

        var requestId = BRIDGE.getCellularUrl(url, headers, 
            function(aUrl, aSuccess, aHeader, aBody)
            {
                onReturnsRecordResponse(aSuccess, aBody, aCallback);
                return;
            });
    }    


    function onReturnsRecordResponse(aSuccess, aBody, aCallback)
    {
        //trace("onReturnsRecordResponse, Success: " +aSuccess+ ", Body: " +aBody);

        if (!aSuccess || aBody == null)
        {
            aCallback(false);
        }

        var rrInFlight = saveReturnsRecord(aBody);

        aCallback(rrInFlight);
    }       

    function saveReturnsRecord(aBody)
    {
        BRIDGE.setValue(RETURNS_CREATED_TIME, "");

        var record = null;

        try
        {
            record = JSON.parse(aBody);
        }
        catch (err)
        {
            error("saveReturnsRecord, Exception: " +err);
            record = null;
        }

        if (record == null)
        {
            return false;
        }

        var repairRecord = record["ccbRetrieveRrResponse"]
            
        if (repairRecord == null)
        {
            error("saveReturnsRecord, Could not get ccbRetrieveRrResponse");
            return false;
        }
 
        var repairDetails = repairRecord["ccbRepairRequestOutput"];

        if (repairDetails == null)
        {
            error("saveReturnsRecord, Could not get ccbRepairRequestOutput");
            return false;
        }

        var repairDateStr = repairDetails["createdDate"];
        var repairNumber  = repairDetails["repairRequestNo"];

        //trace("saveReturnsRecord, Date Str: " +repairDateStr+ ", Number: " +repairNumber);

        if (repairDateStr == null)
        {
            return false;
        }          

        var utcTime = BRIDGE.convertToUTCTime(repairDateStr, RR_TIME_FORMAT, RR_TIME_ZONE);

        //trace("saveReturnsRecord, Repair UTC Time: " +utcTime);

        if (utcTime > 0)
        {
            BRIDGE.setValue(RETURNS_CREATED_TIME, utcTime);

            return true;            
        }

        return false;
    }    

    function uuid()
    {
        var app = BRIDGE.getValue("sourceSystem");

        if (app == null)
        {
            app = "TDCApp";
        }

        var dateNow = new Date();
        var timeNow = dateNow.getTime();

        var random  = Math.floor(Math.random() * 100000);

        var uuid = "RRF-" +app+ "-" +timeNow+ "-" +random;

        return uuid;
    }    

    //====================================================================================
    // SR Micro Services
    //====================================================================================
    //
    function showErrorMessage(aErrorMessage, aRecordId)
    {
        alert('Query failed for record ' + aRecordId + '. ' + aErrorMessage);
    }

    function sendServicesQuery(aUrl, aRequestId, aCallback)
    {
        aUrl = MMRSUTILS.getRequestUrl(mRequestHostUrl, aUrl);      

        console.log("NETSERVICES::sendServicesQuery, Url: " +aUrl);

        if (aUrl == null)
        {
            console.log("sendServicesQuery, NULL query URL passed");
            aCallback(false, null, aRequestId);
            return;
        }

        var dateNow = new Date();
        var timeNow = dateNow.getTime();

        var timedUrl = aUrl+ "&time=" +timeNow;

        BRIDGE.getUrl(timedUrl, null, 
        function(aUrl, aSuccess, aHeader, aResponse)
        {
            console.log("sendServicesQuery, Response: " +aResponse);

            var response = null;

            try
            {
                response = JSON.parse(aResponse);
            }
            catch(err)
            {
                console.log("sendServicesQuery, Exception: " +err);
            }

            if (response == null)
            {
                showErrorMessage("No response received", aRequestId);
                console.log("sendServicesQuery, null response");
                aCallback(false, null, aRequestId, response);
            }
            else if (response.status !== "OK")
            {
                showErrorMessage("Bad status received: " + response.status, aRequestId);
                console.log("sendServicesQuery, Unexpected Status: " +response.status);
                aCallback(false, response.results, aRequestId, response);
            }
            else if (response.results === null)
            {
                showErrorMessage("No results received", aRequestId);
                console.log("sendServicesQuery, null results");
                aCallback(false, null, aRequestId, response);
            }
            else if (response.results.length === 0)
            {
                showErrorMessage("No results received", aRequestId);
                console.log("sendServicesQuery, No results");
                aCallback(false, response.results, aRequestId, response);
            }
            else
            {
                aCallback(true, response.results, aRequestId, response);
            }
        });
    }


    function fetchDeviceDetails(aIMEI, aCallback)
    {
        console.log("NETSERVICES::fetchDeviceDetails, IMEI: " +aIMEI);

        var queryUrl = BRIDGE.getValue("deviceQueryUrl");

        if (queryUrl == null)
        {
            console.log("fetchDeviceDetails, No device query URL set");
            aCallback(false, null);
            return;
        }

        queryUrl += "?imei=" +aIMEI;

        sendServicesQuery(queryUrl, aIMEI, aCallback);
    }

    function fetchPurchaseDetails(aIMEI, aCallback)
    {
        console.log("NETSERVICES::fetchPurchaseDetails, IMEI: " +aIMEI);

        var queryUrl = BRIDGE.getValue("purchaseQueryUrl");

        if (queryUrl == null)
        {
            console.log("fetchPurchaseDetails, No purchase query URL set");
            aCallback(false, null);
            return;
        }

        queryUrl += "?imei=" +aIMEI;

        sendServicesQuery(queryUrl, aIMEI, aCallback);
    }

    function fetchWarrantyDetails(aMake, aCallback)
    {
        console.log("NETSERVICES::fetchWarrantyDetails, Make: " +aMake);

        var queryUrl = BRIDGE.getValue("warrantyQueryUrl");

        if (queryUrl == null)
        {
            console.log("fetchWarrantyDetails, No warranty query URL set");
            aCallback(false, null);
            return;
        }

        queryUrl += "?allowDefault=true&deviceManufacturer=" +aMake;

        console.log("NETSERVICES::fetchWarrantyDetails, Url: " +queryUrl);

        sendServicesQuery(queryUrl, aMake, aCallback);
    }


    function fetchStockDetails(aReplacementId, aCallback)
    {
        console.log("NETSERVICES::fetchStockDetails, ReplacementId: " +aReplacementId);

        var queryUrl = BRIDGE.getValue("stockQueryUrl");

        if (queryUrl == null)
        {
            console.log("fetchStockDetails, No stock query URL set");
            aCallback(false, null);
            return;
        }

        queryUrl += "?productid=" +aReplacementId;

        sendServicesQuery(queryUrl, aReplacementId, aCallback);
    }

    //====================================================================================
    // Address Look Up
    //====================================================================================
    //  
    function fetchAddressDetails(aAddressId, aCallback)
    {
        console.log("NETSERVICES::fetchAddressDetails, Address Id: " +aAddressId);

        var queryUrl = BRIDGE.getValue("addressDetailsLookupUrl");

        if (queryUrl == null)
        {
            console.log("fetchStockDetails, No stock query URL set");
            aCallback(false, null);
            return;
        }

        queryUrl += "?id=" +encodeURI(aAddressId);

        sendServicesQuery(queryUrl, aAddressId,
            function(aSuccess, aResults, aRequestId, aResponse)
            {
                processAddressDetailsResponse(aSuccess, aResults, aRequestId, aResponse, aCallback);
            });
    }

    function processAddressDetailsResponse(aSuccess, aResults, aRequestId, aResponse, aCallback)
    {
        if (!aSuccess)
        {
            if (aResponse) {
                aCallback(false, null, null, aResponse.status);
            }
            else {
                aCallback(false, null, null, null);
            }
            return;
        }

        var results = aResults[0];
        var addressComponents = results.addressComponents;
        if (addressComponents == null)
        {
            showErrorMessage("No address components received", aRequestId);
            console.log("processAddressDetailsResponse, No address components provided");
            aCallback(false, null, null, aResponse.status);
            return; 
        }

        var premise        = getAddressComponent(addressComponents, "premise");
        var street_address = getAddressComponent(addressComponents, "street_address");
        var buildingNumber = getAddressComponent(addressComponents, "street_number");
        var route          = getAddressComponent(addressComponents, "route");

        var address = {};

        if (premise) {
            address.addressLine1 = premise;
        } else if (street_address) {
            address.addressLine1 = street_address;
        } else if (buildingNumber && route) {
            address.addressLine1 = buildingNumber + " " + route;
        }

        if (premise) {
          if (street_address) {
            address.addressLine2 = street_address;
          } else if (buildingNumber && route) {
            address.addressLine2 = buildingNumber + " " + route;
          } else if (route) {
            address.addressLine2 = route;
          }
        }

        address.locality     = getAddressComponent(addressComponents, "locality");
        address.state        = getAddressComponent(addressComponents, "administrative_area_level_1");
        address.postcode     = getAddressComponent(addressComponents, "postal_code");

        address.addressId    = results.id;

        var addressString    = results.formattedAddress;

        var addressOk = address.addressLine1 && address.locality && address.state && address.postcode;

        aCallback(addressOk, addressString, address, aResponse.status);
    }

    /* Address Components
        [
        {
          "longName": "VIC",
          "shortName": "VIC",
          "types": ["administrative_area_level_1"]
        },
        ....
    */

    function getAddressComponent(aComponents, aFieldId)
    {
        if (aComponents == null)
        {
            return null;
        }

        for (var compIndex in aComponents)
        {
            var thisComp = aComponents[compIndex];

            for (var typeIndex in thisComp.types)
            {
                var thisType = thisComp.types[typeIndex];

                if (aFieldId == thisType)
                {
                    var value = thisComp.longName;
                    //console.log("getAddressComponent, Field: " +aFieldId+ ", Value: " +value);
                    return value;
                }
            }
        }

        return null;
    }



    //====================================================================================
    // Customer Account Login
    //====================================================================================
    //   
    function requestUserAuthentication(aUser, aPass, aCallback)
    {
        trace("requestUserAuthentication");

        var url = BRIDGE.getValue("userAuthenticateUrl");

        if (url == null)
        {
            console.log("requestUserAuthentication, No userAuthenticateUrl set");            
            aCallback(false, null);            
            return;
        }

        url = MMRSUTILS.getRequestUrl(mRequestHostUrl, url);

        var headers = {};

        headers["Content-Type"]  = "application/x-www-form-urlencoded";
        headers["Authorization"] = BRIDGE.getValue("userAuthenticateToken");
        headers["source-system"] = BRIDGE.getValue("sourceSystem");

        var payload = "";

        payload += "userIdentifierType=EMAIL";
        payload += "&userIdentifier=" +aUser;
        payload += "&authToken="      +aPass;

        //trace("requestUserAuthentication, Url:     " +url);
        //trace("requestUserAuthentication, Payload: " +payload);

        var requestId = BRIDGE.postUrl(url, headers, payload,
            function(aUrl, aSuccess, aHeader, aBody)
            {
                processUserAuthenticationResponse(aSuccess, aHeader, aBody, aCallback);
            }
        ); 

        trace("requestUserAuthentication, Request Id: " +requestId);
    }

    function processUserAuthenticationResponse(aSuccess, aHeader, aBody, aCallback)
    {
        trace("processUserAuthenticationResponse, Success: " +aSuccess);
        //trace("processUserAuthenticationResponse, Body:    " +aBody);

        if (!aSuccess)
        {
            error("processUserAuthenticationResponse, Could not authenticate");  
            aCallback(false, null);          
            return;
        }

        var authToken = null;

        try
        {
            authToken = JSON.parse(aBody);
        }
        catch (err)
        {
            error("processUserAuthenticationResponse, Could not parse response: " +err);
            aCallback(false, null);
            return;
        }

        if (authToken.responseCode != 0)
        {
            error("processUserAuthenticationResponse, Authorisation Failed: " +authToken.responseMessage);
            aCallback(false, authToken.responseMessage);            
            return;
        }

        var artifactName  = authToken.data.artifactName;
        var artifactValue = authToken.data.artifactValue;

        var cookie = artifactName+ "=" +artifactValue;

        //trace("processUserAuthenticationResponse, Cookie: " +cookie);

        aCallback(true, cookie);
    }      
    // 
    //====================================================================================
    //====================================================================================

    //====================================================================================
    // Customer Account Details 
    //====================================================================================
    //       
    function requestAccountDetails(aCookie, aAccountId, aCallback)
    {
        var url = BRIDGE.getValue("userAccountDetailUrl");

        if (url == null)
        {
            console.log("requestAccountDetails, No userAccountDetailUrl set");            
            aCallback(false, null);            
            return;
        }

        url += "/" +aAccountId;

        var headers = {};

        headers["Accept"]        = "application/vnd.ACCOUNT.account-v1+json";
        headers["Cookie"]        = aCookie;
        headers["source-system"] = mSourceSystem; // "Android";

        //headers["Authorization"] = "Basic bXNwOmJpZ3AwbmQ=";
        //headers["guid"]          = "G02000063557";

        //trace("requestAccountDetails, Url: " +url);

        var requestId = BRIDGE.getUrl(url, headers,           
        function(aUrl, aSuccess, aHeader, aResponse)
        {
            trace("requestAccountDetails, Success: " +aSuccess+ ", Response: " +aResponse);

            processAccountDetailsResponse(aSuccess, aResponse, aCallback);
        });  
    } 

    function processAccountDetailsResponse(aSuccess, aResponse, aCallback)
    {
        trace("processAccountDetailsResponse, Success: " +aSuccess);

        if (!aSuccess)
        {
            error("processAccountDetailsResponse, Request failed");  
            aCallback(false, null);          
            return;
        }

        var response = null;

        try
        {
            response = JSON.parse(aResponse);
        }
        catch (err)
        {
            error("processAccountDetailsResponse, Could not parse response: " +err);
            aCallback(false, null);
            return;
        }


        //trace("processAccountDetailsResponse, Parsed: " + aResponse);

        aCallback(true, response);
    }      

    function requestAccountProfile(aCookie, aCallback)
    {
          var url = BRIDGE.getValue("userAccountProfileUrl");

          var headers = {};

          headers["Accept"]        = "application/vnd.USERPROFILE.userprofile-v1+json";
          headers["source-system"] = mSourceSystem;
          headers["Cookie"]        = aCookie;

          var requestId = BRIDGE.getUrl(url, headers, aCallback);  
    }    
    // 
    //====================================================================================
    //====================================================================================

    //====================================================================================
    // Assessment Reports
    //====================================================================================
    // 

    var TEST_PAYLOAD_1 = 
	{
	  "correlationId": "Test01",
	  "createDeviceAssessmentRequest":
	  {
	    "token": "6olrnk",
	    "msisdn": "0429795098",
	    "imei": "357673080370144",
	    "created": "2019-03-26T04:13:00+11:00",
	    "source": "tdc",
	    "isImeiMatch": "true",
	    "powerOn": "true",
	    "deviceCondition": "0",
	    "liquidDamage": "false",
	    "aux1":"AUX values",
	    "deviceLock": "Y",
	    "deviceLeftInStore": "false",
	    "deviceCapacity": "32GB",
	    "planStartDate": "2019-03-04T04:13:00+11:00",
	    "planEndDate": "2019-06-06T04:13:00+11:00",
	    "assure": "Y",
	    "ran": "true",
	    "redemptionProcessed": "N",
	    "transactionType": "Redemption",
	    "deviceLostOrStolen": "false",
	    "deviceOs": "ios",
	    "deviceFreeOfLocks": "1",
	    "isImeiValid": "Y"
	  }
	};

    function sendAssessmentReport(aReport, aCallback)
    {
        //delete aReport.swapPrice;
        aReport.deviceCondition = "" +aReport.deviceCondition;
        aReport.aux1            = "" +aReport.aux1;        
        //aReport.ran = "true";

        var url = BRIDGE.getValue("swapReportCreateUrl");

        if (url == null)
        {
            error("sendAssessmentReport, No swap assesment report create URL set");
            aCallback(false, null);
            return;
        }

        var correlationId = uuid();

        var request = 
        {
            correlationId: correlationId,
            createDeviceAssessmentRequest: aReport
        }

        //var request = TEST_PAYLOAD_1;

        url = MMRSUTILS.getRequestUrl(mRequestHostUrl, url);

        var headers = {};

        headers["Authorization"] = BRIDGE.getValue("swapAuthenticateToken");
        headers["source-system"] = BRIDGE.getValue("sourceSystem");
        headers["Content-Type"]  = "application/json";

        var payload = JSON.stringify(request);

        console.log("sendAssessmentReport, Url:     " +url);
        console.log("sendAssessmentReport, Headers: " +JSON.stringify(headers));
        console.log("sendAssessmentReport, Payload: " +payload);

        var requestId = BRIDGE.postCellularUrl(url, headers, payload,
            function(aUrl, aSuccess, aHeader, aBody)
            {
                //trace("sendAssessmentReport, Success: " +aSuccess+ ", Header: " +aHeader+ ", Body: " +aBody);

                var success = aSuccess;

                // If we are routing requests over wifi (i.e for testing) the response header will by empty.
                // So if request has succeeded we just assume we have a valid response (when in test mode).
                //
                if (success && aHeader != null)
                {
                	var status = getStatusCode(aHeader);

					trace("sendAssessmentReport, Response Status: " +status);          

                	success = status == "200" || status == "201"; // SPEC is wrong - it only lists 201 as valid
                }

                aCallback(success);
            }
        ); 

        trace("sendAssessmentReport, Request Id: " +requestId);
    }

    function fetchAssessmentReport(aMSISDN, aCallback)    
    {
        var url = BRIDGE.getValue("swapReportRetrieveUrl");

        if (url == null)
        {
            error("fetchAssessmentReport, No swap assesment report retrieve URL set");
            aCallback(false, null);
            return;
        }        

        url = MMRSUTILS.getRequestUrl(mRequestHostUrl, url);

        url += "/" +aMSISDN;

        var headers = {};

        headers["Authorization"] = BRIDGE.getValue("swapAuthenticateToken");
        headers["source-system"] = BRIDGE.getValue("sourceSystem");
        
        headers["correlationId"] = uuid();

        //trace("fetchAssessmentReport, Url: " +url);

        var requestId = BRIDGE.getCellularUrl(url, headers, function(aUrl, aSuccess, aHeader, aResponse)
        {
            //trace("fetchAssessmentReport, Success: " +aSuccess+ ", Header: " +aHeader+ ", Response: " +aResponse);    

            aCallback(aSuccess, aResponse);
        });  
    }

    function getStatusCode(aHeader)
    {
    	if (aHeader == null)
    	{
    		return null;
    	}

    	var lines = aHeader.split("\n");

    	var header = lines[0];

    	console.log("getStatusCode, Header Line: " +header);

    	if (header == null)
    	{
    		return null;
    	}

		header = header.toUpperCase();

    	// Should be of the form: HTTP/1.1 200 OK

    	var words = header.split(" ");

    	var numWords = words.length;

    	if (numWords < 3)
    	{
	    	console.log("getStatusCode, Not enough words on header line: " +header);
    		return null;    		
    	}

    	var httpWord = words[0];

    	if (! httpWord.startsWith("HTTP"))
    	{
	    	console.log("getStatusCode, Not an HTTP header line: " +header);
    		return null;
    	}

    	var status = words[1];

    	console.log("getStatusCode, HTTP Status: " +status);

    	return status;
    }
    // 
    //====================================================================================
    //====================================================================================

    function fetchServerTime(aCallback)
    {
        console.log("NETSERVICES::fetchServerTime");

        var url = BRIDGE.getValue("serverTimeUrl");

        if (url == null || url == "")
        {
            console.log("fetchServerTime, No Server Time URL set");
            aCallback(false, 0);
            return;
        }

        url = MMRSUTILS.getRequestUrl(mRequestHostUrl, url);  

        var requestId = BRIDGE.getUrl(url, null, function(aUrl, aSuccess, aHeader, aResponse)
        {
            //trace("fetchServerTime, Success: " +aSuccess+ ", Response: " +aResponse);

            var timeNow = 0;

            if (aSuccess)
            {
                timeNow = parseInt(aResponse, 10);

                if (isNaN(timeNow))
                {
                    trace("fetchServerTime, Invalid Time received: " +aResponse);

                    timeNow  = 0;
                    aSuccess = false;
                }
            }

            //trace("fetchServerTime, Time Now: " +timeNow);

            aCallback(aSuccess, timeNow);
        });  
    }

    //====================================================================================
    //====================================================================================
    //     
    function fetchAppleAccDetails(aImei, aCallback)
    {
        console.log("NETSERVICES::fetchAppleAccDetails, IMEI: " + aImei);

        fetchServerTime( function(aSuccess, aTimeNow)
        {
            if (! aSuccess)
            {
                error("fetchAppleAccDetails, Unable to fetch server time");            
                aCallback(false, null);
            }

            var url = BRIDGE.getValue("accDeviceInfoUrl");

            if (url == null || url == "")
            {
                console.log("fetchAppleAccDetails, No Apple ACC query URL set");
                aCallback(false, null);
                return;
            }

            url = MMRSUTILS.getRequestUrl(mRequestHostUrl, url);
            
            url += "?imei=" + aImei;

            var authSeed  = aTimeNow;
            var authText  = aImei+ "|" +authSeed;
            var authToken = BRIDGE.signValue(authText);

            //trace("fetchAppleAccDetails, Seed: " +authSeed+ ", Token: " +authToken);            

            var headers =
            {
                "authSeed"  : authSeed,            
                "authToken" : authToken
            };

            var requestId = BRIDGE.getUrl(url, headers, function(aUrl, aSuccess, aHeader, aResponse)
            {
                //trace("fetchAppleAccDetails, Success: " +aSuccess+ ", Response: " +aResponse);

                onAppleAccDetailsReceived(aSuccess, aResponse, aCallback);
            }); 
        });
    }

    function onAppleAccDetailsReceived(aSuccess, aResponse, aCallback)
    {
        trace("onAppleAccDetailsReceived, Success: " +aSuccess);

        if (! aSuccess)
        {
            error("onAppleAccDetailsReceived, Unable to fetch ACC details");
            aCallback(false, null);
            return;
        }

        //trace("onAppleAccDetailsReceived, Response: +" +aResponse+ "+");

        var response = null;

        try
        {
            response = JSON.parse(aResponse);
        }
        catch(err)
        {
            error("onAppleAccDetailsReceived, Exception: " +err);
            response = null;
        }

        if (response == null)
        {
            aCallback(false, null);
            return;
        }

        aCallback(true, response);
    }  

    // 
    //====================================================================================
    //====================================================================================


    function init(aNames)
    {
        console.log("NETSERVICES::init, Names: " +aNames);

        mWorkflowConstants  = aNames.constants;   
        mWorkflowProperties = aNames.fields;        
    }

    var exports = {};

    exports.name = "netServices";

    exports.ACCOUNT_START_DATE_PROPERTY     = ACCOUNT_START_DATE_PROPERTY;
    exports.ACCOUNT_LEASE_PLAN_PROPERTY     = ACCOUNT_LEASE_PLAN_PROPERTY;
    exports.ACCOUNT_ASSURE_PLAN_PROPERTY    = ACCOUNT_ASSURE_PLAN_PROPERTY;
    exports.ACCOUNT_PRODUCTS                = ACCOUNT_PRODUCTS;
    exports.ACCOUNT_LEGAL_LESSE_PROPERTY    = ACCOUNT_LEGAL_LESSE_PROPERTY;
    exports.ACCOUNT_FULL_AUTHORITY_PROPERTY = ACCOUNT_FULL_AUTHORITY_PROPERTY;
    exports.ACCOUNT_SERVICE_ID              = ACCOUNT_SERVICE_ID;
    exports.ACCOUNT_ID                      = ACCOUNT_ID;
    exports.ACCOUNT_CUSTOMER_ID             = ACCOUNT_CUSTOMER_ID;
    exports.ACCOUNT_CUSTOMER_TYPE           = ACCOUNT_CUSTOMER_TYPE;

    exports.RETURNS_CREATED_TIME            = RETURNS_CREATED_TIME;

    exports.init = function(aNames) { return init(aNames); }

    exports.fetchServerTime         = function(aCallback) { return fetchServerTime(aCallback); }

    exports.fetchAppServices        = function(aCallback) { return fetchAppServices(aCallback); }

    exports.fetchAccountInformation = function(aCallback) { fetchAccountInformation(aCallback); }

    exports.fetchReturnsRecord      = function(aIMEI, aCallback) { fetchReturnsRecord(aIMEI, aCallback); }

    exports.fetchDeviceDetails   = function(aIMEI, aCallback) { return fetchDeviceDetails(aIMEI, aCallback); }
 			
    exports.fetchPurchaseDetails = function(aIMEI, aCallback) { return fetchPurchaseDetails(aIMEI, aCallback); }

    exports.fetchWarrantyDetails = function(aMake, aCallback) { return fetchWarrantyDetails(aMake, aCallback); }

    exports.fetchStockDetails    = function(aReplacementId, aCallback) { return fetchStockDetails(aReplacementId, aCallback); }

    exports.fetchAddressDetails  = function(aAddressId, aCallback) { return fetchAddressDetails(aAddressId, aCallback); }

    exports.requestUserAuthentication = function(aUser, aPass, aCallback) { return requestUserAuthentication(aUser, aPass, aCallback); }

    exports.requestAccountDetails = function(aCookie, aAccountId, aCallback) { return requestAccountDetails(aCookie, aAccountId, aCallback); }

    exports.requestAccountProfile = function(aCookie, aCallback) { return requestAccountProfile(aCookie, aCallback); }

    exports.saveServicesValues    = function(aServices) { return saveServicesValues(aServices); }

    exports.saveAccountDetails    = function(aRecord) { return saveAccountDetails(aRecord); }

    exports.sendAssessmentReport  = function(aReport, aCallback) { return sendAssessmentReport(aReport, aCallback); }

    exports.fetchAssessmentReport = function(aMSISDN, aCallback) { return fetchAssessmentReport(aMSISDN, aCallback); }

    exports.fetchAppleAccDetails = function(aIMEI, aCallback) { return fetchAppleAccDetails(aIMEI, aCallback); }

    return exports;
}());



